@@ -1,6 +1,6 @@
 /*
 
-  $Id: cfgreader.c,v 1.79 2007/07/21 13:01:24 dforsi Exp $
+  $Id: cfgreader.c,v 1.84 2007/11/07 18:28:18 pkot Exp $
 
   G N O K I I
 
@@ -171,7 +171,7 @@ struct gn_cfg_header *cfg_memory_read(const char **lines)
 			continue;
 		}
 
-			/* Line not part of any heading */
+		/* Line not part of any heading */
 		fprintf(stderr, _("Orphaned line: %s\n"), line);
 
 		free(buf);
@@ -268,8 +268,7 @@ struct gn_cfg_header *cfg_file_read(const char *filename)
 
 		num_lines++;
 		buf = malloc(line_end - line_begin + 1);
-		strncpy(buf, line_begin, line_end - line_begin);
-		buf[line_end - line_begin] = 0;
+		snprintf(buf, line_end - line_begin + 1, "%s", line_begin);
 		split_lines = realloc(split_lines,
 				(num_lines + 1) * sizeof(char*));
 		split_lines[num_lines - 1] = buf;
@@ -342,6 +341,19 @@ GNOKII_API char *gn_cfg_get(struct gn_cfg_header *cfg, const char *section, cons
 	return NULL;
 }
 
+int cfg_section_exists(struct gn_cfg_header *cfg, const char *section)
+{
+	struct gn_cfg_header *h;
+
+	if (!cfg || !section)
+		return false;
+	/* Search for section name */
+	for (h = cfg; h != NULL; h = h->next)
+		if (!strcmp(section, h->section))
+			return true;
+	return false;
+}
+
 /*
  * Return all the entries of the given section.
  */
@@ -397,33 +409,39 @@ char *cfg_set(struct gn_cfg_header *cfg, const char *section, const char *key,
 	return NULL;
 }
 
-static bool cfg_psection_load(gn_config *cfg, const char *section, const gn_config *def)
+static gn_error cfg_psection_load(gn_config *cfg, const char *section, const gn_config *def)
 {
 	const char *val;
 	char ch;
 
 	memset(cfg, '\0', sizeof(gn_config));
 
+	if (!cfg_section_exists(gn_cfg_info, section)) {
+		fprintf(stderr, _("No %s section in the config file.\n"), section);
+		return GN_ERR_NOPHONE;
+	}
+
 	/* You need to specify at least model and port in the phone section */
-	if (!(val = gn_cfg_get(gn_cfg_info, section, "model")))
-		/* strcpy(cfg->model, def->model); */
-		return false;
-	else
+	if (!(val = gn_cfg_get(gn_cfg_info, section, "model"))) {
+		fprintf(stderr, _("You need to define '%s' in the config file.\n"), "model");
+		return GN_ERR_NOMODEL;
+	} else
 		snprintf(cfg->model, sizeof(cfg->model), "%s", val);
 
-	if (!(val = gn_cfg_get(gn_cfg_info, section, "port")))
-		/* strcpy(cfg->port_device, def->port_device); */
-		return false;
-	else
+	if (!(val = gn_cfg_get(gn_cfg_info, section, "port"))) {
+		fprintf(stderr, _("You need to define '%s' in the config file.\n"), "port");
+		return GN_ERR_NOPORT;
+	} else
 		snprintf(cfg->port_device, sizeof(cfg->port_device), "%s", val);
 
-	if (!(val = gn_cfg_get(gn_cfg_info, section, "connection")))
-		cfg->connection_type = def->connection_type;
-	else {
+	if (!(val = gn_cfg_get(gn_cfg_info, section, "connection"))) {
+		fprintf(stderr, _("You need to define '%s' in the config file.\n"), "connection");
+		return GN_ERR_NOCONNECTION;
+	} else {
 		cfg->connection_type = gn_get_connectiontype(val);
 		if (cfg->connection_type == GN_CT_NONE) {
 			fprintf(stderr, _("Unsupported [%s] %s value \"%s\"\n"), section, "connection", val);
-			return false;
+			return GN_ERR_NOCONNECTION;
 		}
 	}
 
@@ -487,12 +505,12 @@ static bool cfg_psection_load(gn_config *cfg, const char *section, const gn_conf
 	}
 
 	if (!(val = gn_cfg_get(gn_cfg_info, section, "connect_script")))
-		strcpy(cfg->connect_script, def->connect_script);
+		snprintf(cfg->connect_script, sizeof(cfg->connect_script), "%s", def->connect_script);
 	else
 		snprintf(cfg->connect_script, sizeof(cfg->connect_script), "%s", val);
 
 	if (!(val = gn_cfg_get(gn_cfg_info, section, "disconnect_script")))
-		strcpy(cfg->disconnect_script, def->disconnect_script);
+		snprintf(cfg->disconnect_script, sizeof(cfg->disconnect_script), "%s", def->disconnect_script);
 	else
 		snprintf(cfg->disconnect_script, sizeof(cfg->disconnect_script), "%s", val);
 
@@ -515,7 +533,7 @@ static bool cfg_psection_load(gn_config *cfg, const char *section, const gn_conf
 	/* There is no global setting. You need to set irda_string
 	 * in each section if you want it working */
 	if (!(val = gn_cfg_get(gn_cfg_info, section, "irda_string")))
-		strcpy(cfg->irda_string, "");
+		cfg->irda_string[0] = 0;
 	else {
 		snprintf(cfg->irda_string, sizeof(cfg->irda_string), "%s", val);
 		dprintf("Setting irda_string in section %s to %s\n", section, cfg->irda_string);
@@ -534,7 +552,7 @@ static bool cfg_psection_load(gn_config *cfg, const char *section, const gn_conf
 		cfg->use_locking = def->use_locking;
 	}
 
-	return true;
+	return GN_ERR_NONE;
 }
 
 static bool cfg_get_log_target(gn_log_target *t, const char *opt)
@@ -561,6 +579,7 @@ static bool cfg_get_log_target(gn_log_target *t, const char *opt)
 #define MAX_PATH_LEN 200
 GNOKII_API gn_error gn_cfg_read_default()
 {
+	gn_error error;
 	char *homedir;
 	char rcfile[MAX_PATH_LEN];
 #ifdef WIN32
@@ -580,20 +599,21 @@ GNOKII_API gn_error gn_cfg_read_default()
 	const char globalrc[] = "/etc/gnokiirc";
 
 	homedir = getenv("HOME");
-	if (homedir) strncpy(rcfile, homedir, MAX_PATH_LEN);
+	if (homedir)
+		strncpy(rcfile, homedir, MAX_PATH_LEN);
 	strncat(rcfile, "/.gnokiirc", MAX_PATH_LEN);
 #endif
 
 	/* Try opening .gnokirc from users home directory first */
-	if (gn_cfg_file_read(rcfile)) {
+	if ((error = gn_cfg_file_read(rcfile)) != GN_ERR_NONE) {
+		fprintf(stderr, _("Couldn't read %s config file.\n"), rcfile);
 		/* It failed so try for /etc/gnokiirc */
-		if (gn_cfg_file_read(globalrc)) {
+		if ((error == GN_ERR_NOCONFIG) && ((error = gn_cfg_file_read(globalrc)) != GN_ERR_NONE)) {
 			/* That failed too so exit */
-			fprintf(stderr, _("Couldn't open %s or %s.\n"), rcfile, globalrc);
-			return GN_ERR_NOCONFIG;
+			fprintf(stderr, _("Couldn't read %s config file.\n"), globalrc);
 		}
 	}
-	return GN_ERR_NONE;
+	return error;
 }
 
 /* DEPRECATED */
@@ -615,9 +635,10 @@ GNOKII_API gn_error gn_cfg_read(char **bindir)
 static gn_error cfg_file_or_memory_read(const char *file, const char **lines)
 {
 	char *val;
+	gn_error error;
 
 	if (file == NULL && lines == NULL) {
-		dprintf("Couldn't open a config file or memory.\n");
+		fprintf(stderr, _("Couldn't open a config file or memory.\n"));
 		return GN_ERR_NOCONFIG;
 	}
 
@@ -638,13 +659,13 @@ static gn_error cfg_file_or_memory_read(const char *file, const char **lines)
 	if (gn_cfg_info == NULL) {
 		/* this is bad, but the previous was much worse - bozo */
 		if (file)
-			dprintf("Couldn't open %s config file.\n", file);
+			fprintf(stderr, _("Couldn't read %s config file.\n"), file);
 		else
-			dprintf("Couldn't read config\n");
+			fprintf(stderr, _("Couldn't read config.\n"));
 		return GN_ERR_NOCONFIG;
 	}
-	strcpy(gn_config_default.model, "");
-	strcpy(gn_config_default.port_device, "");
+	gn_config_default.model[0] = 0;
+	gn_config_default.port_device[0] = 0;
 	gn_config_default.connection_type = GN_CT_Serial;
 	gn_config_default.init_length = 0;
 	gn_config_default.serial_baudrate = 19200;
@@ -652,17 +673,15 @@ static gn_error cfg_file_or_memory_read(const char *file, const char **lines)
 	gn_config_default.hardware_handshake = false;
 	gn_config_default.require_dcd = false;
 	gn_config_default.smsc_timeout = -1;
-	strcpy(gn_config_default.irda_string, "");
-	strcpy(gn_config_default.connect_script, "");
-	strcpy(gn_config_default.disconnect_script, "");
+	gn_config_default.irda_string[0] = 0;
+	gn_config_default.connect_script[0] = 0;
+	gn_config_default.disconnect_script[0] = 0;
 	gn_config_default.rfcomm_cn = 1;
 	gn_config_default.sm_retry = 0;
 	gn_config_default.use_locking = 0;
 
-	if (!cfg_psection_load(&gn_config_global, "global", &gn_config_default)) {
-		fprintf(stderr, _("No [global] section in %s config file.\n"), file);
-		return GN_ERR_NOPHONE;
-	}
+	if ((error = cfg_psection_load(&gn_config_global, "global", &gn_config_default)) != GN_ERR_NONE)
+		return error;
 
 	/* hack to support [sms] / smsc_timeout parameter */
 	if (gn_config_global.smsc_timeout < 0) {
@@ -698,13 +717,14 @@ GNOKII_API gn_error gn_cfg_memory_read(const char **lines)
 GNOKII_API gn_error gn_cfg_phone_load(const char *iname, struct gn_statemachine *state)
 {
 	char section[256];
+	gn_error error;
 
 	if (iname == NULL || *iname == '\0') {
 		state->config = gn_config_global;
 	} else {
 		snprintf(section, sizeof(section), "phone_%s", iname);
-		if (!cfg_psection_load(&state->config, section, &gn_config_global))
-			return GN_ERR_NOPHONE;
+		if ((error = cfg_psection_load(&state->config, section, &gn_config_global)) != GN_ERR_NONE)
+			return error;
 	}
 
 	if (state->config.model[0] == '\0') {
