@@ -1,6 +1,6 @@
 /*
 
-  $Id: compat.c,v 1.13 2007/07/05 22:04:09 pkot Exp $
+  $Id: compat.c,v 1.14 2007/10/16 11:43:59 dforsi Exp $
 
   G N O K I I
 
@@ -192,18 +192,3 @@ char *strndup(const char *src, size_t n)
 }
 #endif
 
-#ifndef HAVE_GETLINE
-int getline(char **line, size_t *len, FILE *stream)
-{
-	size_t size = 0, last = 0;
-
-	do {
-		size += BUFSIZ;
-		*line = realloc(*line, size);
-		fgets(*line + last, size, stream);
-		*len = strlen(*line);
-		last = *len - 1;
-	} while (!feof(stream) && *line[last] != '\n');
-	return 0;
-}
-#endif
