@@ -1,6 +1,6 @@
 /*
 
-  $Id: unixbluetooth.c,v 1.14 2006/01/07 18:41:59 dforsi Exp $
+  $Id: unixbluetooth.c,v 1.18 2007/11/11 15:16:12 pkot Exp $
  
   G N O K I I
 
@@ -37,7 +37,6 @@
 #if defined(HAVE_BLUETOOTH_BLUEZ) || defined(HAVE_BLUETOOTH_NETGRAPH)
 
 #include <stdlib.h>
-#include <unistd.h>
 #include <stdio.h>
 #include <fcntl.h>
 #include <errno.h>
@@ -95,10 +94,27 @@ static int str2ba(const char *str, bdaddr_t *ba)
 
 #endif
 
+/* From: http://www.kegel.com/dkftpbench/nonblocking.html */
+static int setNonblocking(int fd)
+{
+	int retcode, flags;
+
+#if defined(O_NONBLOCK)
+	if ((flags = fcntl(fd, F_GETFL, 0)) == -1)
+		flags = 0;
+	retcode = fcntl(fd, F_SETFL, flags | O_NONBLOCK);
+#else
+	flags = 1;
+	retcode = ioctl(fd, FIOASYNC, &flags);
+#endif
+
+	return retcode;
+}
+
 int bluetooth_open(const char *addr, uint8_t channel, struct gn_statemachine *state)
 {
 	bdaddr_t bdaddr;
-	struct sockaddr_rc laddr, raddr;
+	struct sockaddr_rc raddr;
 	int fd;
 
 	if (str2ba((char *)addr, &bdaddr)) {
@@ -111,15 +127,6 @@ int bluetooth_open(const char *addr, uint8_t channel, struct gn_statemachine *st
 		return -1;
 	}
 
-	memset(&laddr, 0, sizeof(laddr));
-	laddr.rc_family = AF_BLUETOOTH;
-	bacpy(&laddr.rc_bdaddr, BDADDR_ANY);
-	if (bind(fd, (struct sockaddr *)&laddr, sizeof(laddr)) < 0) {
-		perror(_("Can't bind socket"));
-		close(fd);
-		return -1;
-	}
-
 	memset(&raddr, 0, sizeof(raddr));
 	raddr.rc_family = AF_BLUETOOTH;
 	bacpy(&raddr.rc_bdaddr, &bdaddr);
@@ -130,6 +137,11 @@ int bluetooth_open(const char *addr, uint8_t channel, struct gn_statemachine *st
 		return -1;
 	}
 
+	/* Ignore errors. If the socket was not set in the async way,
+	 * we can live with that.
+	 */
+	setNonblocking(fd);
+
 	return fd;
 }
 
