@@ -1,6 +1,6 @@
 /*
 
-  $Id: unixserial.c,v 1.40 2006/04/23 19:44:31 deller Exp $
+  $Id: unixserial.c,v 1.45 2007/09/04 17:18:39 dforsi Exp $
 
   G N O K I I
 
@@ -160,7 +160,9 @@ int device_script(int fd, const char *section, struct gn_statemachine *state)
 int serial_close(int fd, struct gn_statemachine *state);
 
 
-/* Open the serial port and store the settings. */
+/* Open the serial port and store the settings.
+ * Returns a file descriptor on success, or -1 if an error occurred.
+ */
 int serial_open(const char *file, int oflag)
 {
 	int fd;
@@ -183,7 +185,9 @@ int serial_open(const char *file, int oflag)
 	return fd;
 }
 
-/* Close the serial port and restore old settings. */
+/* Close the serial port and restore old settings.
+ * Returns zero on success, -1 if an error occurred or fd was invalid.
+ */
 int serial_close(int fd, struct gn_statemachine *state)
 {
 	/* handle config file disconnect_script:
@@ -192,16 +196,12 @@ int serial_close(int fd, struct gn_statemachine *state)
 		dprintf("Gnokii serial_close: disconnect_script\n");
 
 	if (fd >= 0) {
-#if 1 /* HACK */
 		serial_termios.c_cflag |= HUPCL;	/* production == 1 */
-#else
-		serial_termios.c_cflag &= ~HUPCL;	/* debugging  == 0 */
-#endif
-
 		tcsetattr(fd, TCSANOW, &serial_termios);
+		return close(fd);
 	}
 
-	return close(fd);
+	return -1;
 }
 
 /* Open a device with standard options.
@@ -218,7 +218,7 @@ int serial_opendevice(const char *file, int with_odd_parity,
 	/* Open device */
 
 	/* O_NONBLOCK MUST be used here as the CLOCAL may be currently off
-	 * and if DCD is down the "open" syscall would be stuck wating for DCD.
+	 * and if DCD is down the "open" syscall would be stuck waiting for DCD.
 	 */
 	fd = serial_open(file, O_RDWR | O_NOCTTY | O_NONBLOCK);
 
@@ -269,7 +269,7 @@ int serial_opendevice(const char *file, int with_odd_parity,
 
 	retcode = fcntl(fd, F_SETFL, 0);
 	if (retcode == -1) {
-		perror("Gnokii serial_opendevice: fnctl(F_SETFL)");
+		perror("Gnokii serial_opendevice: fcntl(F_SETFL)");
 		serial_close(fd, state);
 		return -1;
 	}
@@ -287,32 +287,33 @@ int serial_opendevice(const char *file, int with_odd_parity,
 #if !(__unices__)
 	retcode = fcntl(fd, F_SETOWN, getpid());
 	if (retcode == -1) {
-		perror("Gnokii serial_opendevice: fnctl(F_SETOWN)");
+		perror("Gnokii serial_opendevice: fcntl(F_SETOWN)");
 		serial_close(fd, state);
 		return -1;
 	}
 #endif
 
 	/* Make filedescriptor asynchronous. */
-
-	/* We need to supply FNONBLOCK (or O_NONBLOCK) again as it would get reset
-	 * by F_SETFL as a side-effect!
-	 */
+	if (with_async) {
+		/* We need to supply FNONBLOCK (or O_NONBLOCK) again as it would get reset
+		 * by F_SETFL as a side-effect!
+		 */
 #ifdef FNONBLOCK
-	retcode = fcntl(fd, F_SETFL, (with_async ? FASYNC : 0) | FNONBLOCK);
+		retcode = fcntl(fd, F_SETFL, (with_async ? FASYNC : 0) | FNONBLOCK);
 #else
 #  ifdef FASYNC
-	retcode = fcntl(fd, F_SETFL, (with_async ? FASYNC : 0) | O_NONBLOCK);
+		retcode = fcntl(fd, F_SETFL, (with_async ? FASYNC : 0) | O_NONBLOCK);
 #  else
-	retcode = fcntl(fd, F_SETFL, O_NONBLOCK);
-	if (retcode != -1)
-		retcode = ioctl(fd, FIOASYNC, &with_async);
+		retcode = fcntl(fd, F_SETFL, O_NONBLOCK);
+		if (retcode != -1)
+			retcode = ioctl(fd, FIOASYNC, &with_async);
 #  endif
 #endif
-	if (retcode == -1) {
-		perror("Gnokii serial_opendevice: fnctl(F_SETFL)");
-		serial_close(fd, state);
-		return -1;
+		if (retcode == -1) {
+			perror("Gnokii serial_opendevice: fcntl(F_SETFL)");
+			serial_close(fd, state);
+			return -1;
+		}
 	}
 
 	return fd;
