@@ -1,6 +1,6 @@
 /*
 
-  $Id: gsm-api.c,v 1.68 2005/04/20 22:33:36 pkot Exp $
+  $Id: gsm-api.c,v 1.71 2007/11/22 23:49:11 dforsi Exp $
 
   G N O K I I
 
@@ -92,7 +92,7 @@ gn_error (*gn_gsm_f)(gn_operation op, gn_data *data, struct gn_statemachine *sta
 /* Define pointer to the gn_phone structure used by external code to
    obtain information that varies from model to model. This structure is also
    defined in gsm-common.h */
-API gn_phone *gn_gsm_info;
+GNOKII_API gn_phone *gn_gsm_info;
 
 /* Initialise interface to the phone. Model number should be a string such as
    3810, 5110, 6110 etc. Device is the serial port to use e.g. /dev/ttyS0, the
@@ -123,24 +123,30 @@ static gn_error register_driver(gn_driver *driver, const char *model, char *setu
 		return ret; \
 }
 
-API gn_error gn_gsm_initialise(struct gn_statemachine *sm)
+GNOKII_API gn_error gn_gsm_initialise(struct gn_statemachine *sm)
 {
 	gn_error ret;
 
 	dprintf("phone instance config:\n");
-	dprintf("model: %s\n", sm->config.model);
-	dprintf("port_device: %s\n", sm->config.port_device);
-	dprintf("connection_type: %d\n", sm->config.connection_type);
-	dprintf("init_length: %d\n", sm->config.init_length);
-	dprintf("serial_baudrate: %d\n", sm->config.serial_baudrate);
-	dprintf("serial_write_usleep: %d\n", sm->config.serial_write_usleep);
-	dprintf("hardware_handshake: %d\n", sm->config.hardware_handshake);
-	dprintf("require_dcd: %d\n", sm->config.require_dcd);
-	dprintf("smsc_timeout: %d\n", sm->config.smsc_timeout);
-	dprintf("connect_script: %s\n", sm->config.connect_script);
-	dprintf("disconnect_script: %s\n", sm->config.disconnect_script);
-	dprintf("rfcomm_cn: %d\n", (int)sm->config.rfcomm_cn);
-	dprintf("sm_retry: %s\n", sm->config.sm_retry ? "on" : "off");
+	dprintf("model = %s\n", sm->config.model);
+	dprintf("port = %s\n", sm->config.port_device);
+	dprintf("connection = %s\n", gn_lib_get_connection_name(sm->config.connection_type));
+	if (sm->config.init_length) {
+		dprintf("initlength = %d\n", sm->config.init_length);
+	} else {
+		dprintf("initlength = default\n");
+	}
+	dprintf("serial_baudrate = %d\n", sm->config.serial_baudrate);
+	dprintf("serial_write_usleep = %d\n", sm->config.serial_write_usleep);
+	dprintf("handshake = %s\n", sm->config.hardware_handshake ? "hardware" : "software");
+	dprintf("require_dcd = %d\n", sm->config.require_dcd);
+	dprintf("smsc_timeout = %d\n", sm->config.smsc_timeout / 10);
+	if (*sm->config.connect_script)
+		dprintf("connect_script = %s\n", sm->config.connect_script);
+	if (*sm->config.disconnect_script)
+		dprintf("disconnect_script = %s\n", sm->config.disconnect_script);
+	dprintf("rfcomm_channel = %d\n", (int)sm->config.rfcomm_cn);
+	dprintf("sm_retry = %d\n", sm->config.sm_retry);
 
 	if (sm->config.model[0] == '\0') return GN_ERR_UNKNOWNMODEL;
 	if (sm->config.port_device[0] == '\0') return GN_ERR_FAILED;
@@ -158,6 +164,9 @@ API gn_error gn_gsm_initialise(struct gn_statemachine *sm)
 	REGISTER_DRIVER(at, sm->config.model);
 	REGISTER_DRIVER(nokia_6160, NULL);
 	REGISTER_DRIVER(gnapplet, NULL);
+#ifdef HAVE_PCSC
+	REGISTER_DRIVER(pcsc, sm->config.model);
+#endif
 
 	return GN_ERR_UNKNOWNMODEL;
 }
