@@ -1,6 +1,6 @@
 /*
 
-  $Id: gsm-call.c,v 1.15 2006/07/05 21:51:46 dforsi Exp $
+  $Id: gsm-call.c,v 1.17 2007/11/07 18:28:18 pkot Exp $
 
   G N O K I I
 
@@ -63,8 +63,8 @@ GNOKII_API void gn_call_notifier(gn_call_status call_status, gn_call_info *call_
 		call->call_id = call_info->call_id;
 		call->status = GN_CALL_Ringing;
 		call->type = call_info->type;
-		strcpy(call->remote_number, call_info->number);
-		strcpy(call->remote_name, call_info->name);
+		snprintf(call->remote_number, sizeof(call->remote_number), "%s", call_info->number);
+		snprintf(call->remote_name, sizeof(call->remote_name), "%s", call_info->name);
 		gettimeofday(&call->start_time, NULL);
 		memset(&call->answer_time, 0, sizeof(call->answer_time));
 		call->local_originated = false;
@@ -86,8 +86,8 @@ GNOKII_API void gn_call_notifier(gn_call_status call_status, gn_call_info *call_
 			call->state = state;
 			call->call_id = call_info->call_id;
 			call->type = call_info->type;
-			strcpy(call->remote_number, call_info->number);
-			strcpy(call->remote_name, call_info->name);
+			snprintf(call->remote_number, sizeof(call->remote_number), "%s", call_info->number);
+			snprintf(call->remote_name, sizeof(call->remote_name), "%s", call_info->name);
 			gettimeofday(&call->start_time, NULL);
 			memcpy(&call->answer_time, &call->start_time, sizeof(call->answer_time));
 			call->local_originated = false;
@@ -130,8 +130,8 @@ GNOKII_API gn_error gn_call_dial(int *call_id, gn_data *data, struct gn_statemac
 	call->call_id = data->call_info->call_id;
 	call->status = GN_CALL_Dialing;
 	call->type = data->call_info->type;
-	strcpy(call->remote_number, data->call_info->number);
-	strcpy(call->remote_name, data->call_info->name);
+	snprintf(call->remote_number, sizeof(call->remote_number), "%s", data->call_info->number);
+	snprintf(call->remote_name, sizeof(call->remote_name), "%s", data->call_info->name);
 	gettimeofday(&call->start_time, NULL);
 	memset(&call->answer_time, 0, sizeof(call->answer_time));
 	call->local_originated = true;
@@ -181,7 +181,7 @@ GNOKII_API gn_call *gn_call_get_active(int call_id)
 GNOKII_API gn_error gn_call_check_active(struct gn_statemachine *state)
 {
 	gn_data data;
-	gn_call_active active[2];
+	gn_call_active active[GN_CALL_MAX_PARALLEL];
 	gn_call *call;
 	gn_error err;
 	int i, j, got;
@@ -197,7 +197,7 @@ GNOKII_API gn_error gn_call_check_active(struct gn_statemachine *state)
 	for (j = 0; j < GN_CALL_MAX_PARALLEL; j++) {
 		if (calltable[j].state != state) continue;
 		got = 0;
-		for ( i = 0; i < 2; i++) {
+		for (i = 0; i < GN_CALL_MAX_PARALLEL; i++) {
 			if (calltable[j].call_id == active[i].call_id) {
 				got = 1;
 				break;
@@ -209,7 +209,7 @@ GNOKII_API gn_error gn_call_check_active(struct gn_statemachine *state)
 		}
 	}
 
-	for (i = 0; i < 2; i++) {
+	for (i = 0; i < GN_CALL_MAX_PARALLEL; i++) {
 		if (active[i].state == GN_CALL_Idle) continue;
 
 		if (!(call = search_call(active[i].call_id, state))) {
@@ -225,8 +225,8 @@ GNOKII_API gn_error gn_call_check_active(struct gn_statemachine *state)
 			call->call_id = active[i].call_id;
 			call->status = active[i].state;
 			call->type = GN_CALL_Voice;
-			strcpy(call->remote_number, active[i].number);
-			strcpy(call->remote_name, active[i].name);
+			snprintf(call->remote_number, sizeof(call->remote_number), "%s", active[i].number);
+			snprintf(call->remote_name, sizeof(call->remote_name), "%s", active[i].name);
 			gettimeofday(&call->start_time, NULL);
 			memset(&call->answer_time, 0, sizeof(call->answer_time));
 			call->local_originated = false;
