@@ -1,6 +1,6 @@
 /*
 
-  $Id: gsm-encoding.c,v 1.77 2007/08/01 20:13:24 pkot Exp $
+  $Id: gsm-encoding.c,v 1.81 2007/11/07 18:28:18 pkot Exp $
 
   G N O K I I
 
@@ -119,7 +119,7 @@ static const char *get_langinfo_codeset(void)
 #  ifdef WIN32
 		/* As suggested by Ben Bryant, http://codesnipers.com/?q=node/46 */
 		char szCP[10];
-		sprintf(szCP, ".%d", GetACP());
+		snprintf(szCP, sizeof(szCP), ".%d", GetACP());
 		codeset = setlocale(LC_ALL, szCP);
 #  else
 		codeset = locale_charset();
@@ -149,7 +149,7 @@ static int char_mbtowc(wchar_t *dst, const char *src, int maxlen, MBSTATE *mbs)
 {
 #ifdef HAVE_ICONV
 	size_t nconv;
-	char *pin;
+	ICONV_CONST char *pin;
 	char *pout;
 	size_t inlen;
 	size_t outlen;
@@ -178,7 +178,7 @@ static int char_wctomb(char *dst, wchar_t src, MBSTATE *mbs)
 {
 #ifdef HAVE_ICONV
 	size_t nconv;
-	char *pin;
+	ICONV_CONST char *pin;
 	char *pout;
 	size_t inlen;
 	size_t outlen;
@@ -435,7 +435,7 @@ size_t char_hex_encode(char *dest, size_t dest_len, const char *src, size_t len)
 	int i, n = dest_len / 2 >= len ? len : dest_len / 2;
 
 	for (i = 0; i < n; i++)
-		sprintf(dest + i * 2, "%x", char_def_alphabet_encode(src[i]));
+		snprintf(dest + i * 2, 3, "%02x", char_def_alphabet_encode(src[i]));
 	return len * 2;
 }
 
@@ -508,7 +508,7 @@ size_t char_ucs2_encode(char *dest, size_t dest_len, const char *src, size_t len
 			return i * 4;
 		i += length;
 		/* XXX: We should probably check wchar_t size. */
-		sprintf(dest + (o_len << 2), "%04x", wc);
+		snprintf(dest + (o_len << 2), 5, "%04x", wc);
 	}
 	return len * 4;
 }
@@ -656,9 +656,10 @@ char *char_bcd_number_get(u8 *number)
 		buffer[length] = 0;
 		break;
 	case GN_GSM_NUMBER_International:
-		sprintf(buffer, "+");
+		snprintf(buffer, sizeof(buffer), "+");
 		i++;
-		if (length == GN_BCD_STRING_MAX_LENGTH) length--; /* avoid overflow */
+		if (length == GN_BCD_STRING_MAX_LENGTH)
+			length--; /* avoid overflow */
 	case GN_GSM_NUMBER_Unknown:
 	case GN_GSM_NUMBER_National:
 	case GN_GSM_NUMBER_Network:
@@ -680,8 +681,7 @@ char *char_bcd_number_get(u8 *number)
 
 void gn_char_set_encoding(const char* encoding)
 {
-	strncpy(application_encoding, encoding, sizeof(application_encoding) - 1);
-	application_encoding[sizeof(application_encoding) - 1] = 0;
+	snprintf(application_encoding, sizeof(application_encoding), "%s", encoding);
 }
 
 /* UTF-8 conversion functions */
@@ -794,11 +794,12 @@ int string_base64(const char *instring)
 /*
    Encodes a null-terminated input string with base64 encoding.
    The buffer outstring needs to be at least 1.333 times bigger than the input string length.
-   outlen is the size of the oustring buffer excluding null termination.
+   outlen is the size of the outstring buffer excluding null termination.
 */
 int base64_encode(char *outstring, int outlen, const char *instring, int inlen)
 {
-	char *pin, *pout;
+	const char *pin;
+	char *pout;
 	char *outtemp = NULL;
 	int inleft, outleft;
 
@@ -807,7 +808,7 @@ int base64_encode(char *outstring, int outlen, const char *instring, int inlen)
 	outleft = outlen;
 	pin = instring;
 
-	/* This is in case someone have us not appropriate buffer for outstring */
+	/* This is in case someone passes a buffer not appropriate for outstring */
 	while (outleft > 3 && inleft > 0) {
 		int a, b, c;
 		unsigned int i1, i2, i3, i4;
@@ -984,9 +985,9 @@ int strip_slashes(char *dest, char *src, int maxlen, int len)
 		case ';':
 		case ',':
 			if (slash_state) {
-				dest[j++] = src[i];
 				slash_state = 0;
 			}
+			dest[j++] = src[i];
 			break;
 		case '\\':
 			if (slash_state) {
@@ -1016,9 +1017,8 @@ int strip_slashes(char *dest, char *src, int maxlen, int len)
 			if (slash_state) {
 				dest[j++] = '\\';
 				slash_state = 0;
-			} else {
-				dest[j++] = src[i];
 			}
+			dest[j++] = src[i];
 			break;
 		}
 	}
