@@ -1,6 +1,6 @@
 /*
 
-  $Id: gsm-filetypes.c,v 1.68 2006/06/11 16:39:48 deller Exp $
+  $Id: gsm-filetypes.c,v 1.79 2007/11/20 18:36:17 dforsi Exp $
 
   G N O K I I
 
@@ -34,14 +34,14 @@
 
 */
 
+#include "config.h"
+#include "compat.h"
+
 #include <stdio.h>
 #include <stdlib.h>
-#include <string.h>
 #include <ctype.h>
 #include <sys/stat.h>
 
-#include "config.h"
-#include "compat.h"
 #include "gnokii-internal.h"
 #include "gnokii.h"
 #include "gsm-filetypes.h"
@@ -174,11 +174,11 @@ gn_error file_rtttl_load(FILE *file, gn_ringtone *ringtone)
 	/* This is for buggy RTTTL ringtones without name. */
 	if (buffer[0] != RTTTL_SEP[0]) {
 		strtok(buffer, RTTTL_SEP);
-		sprintf(ringtone->name, "%s", buffer);
+		snprintf(ringtone->name, sizeof(ringtone->name), "%s", buffer);
 		def = strtok(NULL, RTTTL_SEP);
 		notes = strtok(NULL, RTTTL_SEP);
 	} else {
-		sprintf(ringtone->name, "GNOKII");
+		snprintf(ringtone->name, sizeof(ringtone->name), "GNOKII");
 		def = strtok(buffer, RTTTL_SEP);
 		notes = strtok(NULL, RTTTL_SEP);
 	}
@@ -281,7 +281,7 @@ gn_error file_nokraw_load(FILE *file, gn_ringtone *ringtone)
 	int n;
 	gn_error err;
 
-	strcpy(ringtone->name, "GNOKII");
+	snprintf(ringtone->name, sizeof(ringtone->name), "GNOKII");
 
 	if ((n = fread(buf, 1, sizeof(buf), file)) < 0) return GN_ERR_UNKNOWN;
 
@@ -637,7 +637,7 @@ gn_error file_xpm_load(char *filename, gn_bmp *bitmap)
 	bitmap->size = ((bitmap->width + 7) / 8) * bitmap->height;
 
 	if (bitmap->size > GN_BMP_MAX_SIZE) {
-		fprintf(stdout, "Bitmap too large\n");
+		fprintf(stderr, _("Bitmap too large\n"));
 		return GN_ERR_INVALIDSIZE;
 	}
 
@@ -736,7 +736,7 @@ gn_error file_bmp_load(FILE *file, gn_bmp *bitmap)
 				fread(buffer, 1, 1, file);
 				sizeimage++;
 				i++;
-				if (i == 5) i = 1; /* each line is written in multiply of 4 bytes */
+				if (i == 5) i = 1; /* each line is written in multiples of 4 bytes */
 			}
 			if (x <= bitmap->width && y <= bitmap->height) { /* we have top left corner ! */
 				if (first_black) {
@@ -768,7 +768,7 @@ gn_error file_nol_load(FILE *file, gn_bmp *bitmap, gn_phone *info)
 
 
 	fread(buffer, 1, 20, file);
-	sprintf(bitmap->netcode, "%d %02d", buffer[6] + 256 * buffer[7], buffer[8]);
+	snprintf(bitmap->netcode, sizeof(bitmap->netcode), "%d %02d", buffer[6] + 256 * buffer[7], buffer[8]);
 
 	bitmap->width = buffer[10];
 	bitmap->height = buffer[12];
@@ -1052,44 +1052,6 @@ gn_error gn_file_bitmap_save(char *filename, gn_bmp *bitmap, gn_phone *info)
 	return GN_ERR_NONE;
 }
 
-
-/* FIXME - this should not ask for confirmation here - I'm not sure what calls it though */
-/* mode == 0 -> overwrite
- * mode == 1 -> ask
- * mode == 2 -> append
- */
-int gn_file_text_save(char *filename, char *text, int mode)
-{
-	FILE *file;
-	int confirm = -1;
-	char ans[5];
-	struct stat buf;
-
-	/* Ask before overwriting */
-	if ((mode == 1) && (stat(filename, &buf) == 0)) {
-		fprintf(stdout, _("File %s exists.\n"), filename);
-		while (confirm < 0) {
-			fprintf(stderr, _("Overwrite? (yes/no) "));
-			gn_line_get(stdin, ans, 4);
-			if (!strcmp(ans, _("yes"))) confirm = 1;
-			else if (!strcmp(ans, _("no"))) confirm = 0;
-		}
-		if (!confirm) return -1;
-	}
-
-	if (mode == 2) file = fopen(filename, "a");
-	else file = fopen(filename, "w");
-
-	if (!file) {
-		fprintf(stderr, _("Can't open file %s for writing!\n"),  filename);
-		return -1;
-	}
-	fprintf(file, "%s\n", text);
-	fclose(file);
-	return 2;
-}
-
-
 #ifdef XPM
 void file_xpm_save(char *filename, gn_bmp *bitmap)
 {
@@ -1164,13 +1126,13 @@ void file_bmp_save(FILE *file, gn_bmp *bitmap)
 			if (pos == 7) { //new byte !
 				sizeimage++;
 				i++;
-				if (i == 5) i = 1; //each line is written in multiply of 4 bytes
+				if (i == 5) i = 1; //each line is written in multiples of 4 bytes
 			}
 			pos--;
 			if (pos < 0) pos = 7; //going to new byte
 		}
 		pos = 7; //going to new byte
-		while (i != 5) { //each line is written in multiply of 4 bytes
+		while (i != 5) { //each line is written in multiples of 4 bytes
 			sizeimage++;
 			i++;
 		}
@@ -1195,7 +1157,7 @@ void file_bmp_save(FILE *file, gn_bmp *bitmap)
 			if (pos == 7) { //new byte !
 				if (x != 0) fwrite(buffer, 1, sizeof(buffer), file);
 				i++;
-				if(i == 5) i = 1; //each line is written in multiply of 4 bytes
+				if(i == 5) i = 1; //each line is written in multiples of 4 bytes
 				buffer[0] = 0;
 			}
 			if (gn_bmp_point(bitmap, x, y)) buffer[0] |= (1 << pos);
@@ -1204,7 +1166,7 @@ void file_bmp_save(FILE *file, gn_bmp *bitmap)
 		}
 		pos = 7; //going to new byte
 		fwrite(buffer, 1, sizeof(buffer), file);
-		while (i != 5) { //each line is written in multiply of 4 bytes
+		while (i != 5) { //each line is written in multiples of 4 bytes
 			buffer[0] = 0;
 			fwrite(buffer, 1, sizeof(buffer), file);
 			i++;
@@ -1434,9 +1396,17 @@ static int get_next_token(char *src, int delim)
 #define GET_NEXT_TOKEN()	o = get_next_token(line + offset, ';')
 #define STORE_TOKEN(a)		strip_slashes(a, line + offset, sizeof(a) - 1, o - 1)
 
+inline int local_atoi(char *str, int len)
+{
+	int retval;
+	char *aux = strndup(str, len);
+	retval = atoi(aux);
+	free(aux);
+	return retval;
+}
+
 GNOKII_API gn_error gn_file_phonebook_raw_parse(gn_phonebook_entry *entry, char *line)
 {
-	char backline[MAX_INPUT_LINE_LEN];
 	char memory_type_char[3];
 	char number[GN_PHONEBOOK_NUMBER_MAX_LENGTH];
 	int length, o, offset = 0;
@@ -1445,7 +1415,6 @@ GNOKII_API gn_error gn_file_phonebook_raw_parse(gn_phonebook_entry *entry, char
 	memset(entry, 0, sizeof(gn_phonebook_entry));
 
 	length = strlen(line);
-	strcpy(backline, line);
 	entry->empty = true;
 	memory_type_char[2] = 0;
 
@@ -1455,7 +1424,7 @@ GNOKII_API gn_error gn_file_phonebook_raw_parse(gn_phonebook_entry *entry, char
 		return GN_ERR_WRONGDATAFORMAT;
 	case 1:
 		/* empty name: this is a request to delete the entry */
-		return GN_ERR_NONE;
+		break;
 	default:
 		break;
 	}
@@ -1489,11 +1458,11 @@ GNOKII_API gn_error gn_file_phonebook_raw_parse(gn_phonebook_entry *entry, char
 	BUG(offset >= length);
 
 	entry->memory_type = gn_str2memory_type(memory_type_char);
-	/* We can store addressbook entries only in ME or SM */
+	/* We can store addressbook entries only in ME or SM (or ON on some models) */
 	if (entry->memory_type != GN_MT_ME &&
-	    entry->memory_type != GN_MT_SM) {
-		fprintf(stderr, _("Format problem on line [%s]\n"), backline);
-		return GN_ERR_WRONGDATAFORMAT;
+	    entry->memory_type != GN_MT_SM &&
+	    entry->memory_type != GN_MT_ON) {
+		return GN_ERR_INVALIDMEMORYTYPE;
 	}
 
 	BUG(offset >= length);
@@ -1533,6 +1502,11 @@ GNOKII_API gn_error gn_file_phonebook_raw_parse(gn_phonebook_entry *entry, char
 	entry->empty = false;
 
 	for (entry->subentries_count = 0; offset < length; entry->subentries_count++) {
+		if (entry->subentries_count == GN_PHONEBOOK_SUBENTRIES_MAX_NUMBER) {
+			fprintf(stderr, _("Formatting error: too many subentries\n"));
+			error = GN_ERR_WRONGDATAFORMAT;
+			goto endloop;
+		}
 		memset(number, 0, sizeof(number));
 		GET_NEXT_TOKEN();
 		STORE_TOKEN(number);
@@ -1608,7 +1582,20 @@ GNOKII_API gn_error gn_file_phonebook_raw_parse(gn_phonebook_entry *entry, char
 		}
 
 		GET_NEXT_TOKEN();
-		STORE_TOKEN(entry->subentries[entry->subentries_count].data.number);
+		switch (entry->subentries[entry->subentries_count].entry_type) {
+			case GN_PHONEBOOK_ENTRY_Date:
+			case GN_PHONEBOOK_ENTRY_Birthday:
+				entry->subentries[entry->subentries_count].data.date.year   = local_atoi(line + offset, 4);
+				entry->subentries[entry->subentries_count].data.date.month  = local_atoi(line + offset + 4, 2);
+				entry->subentries[entry->subentries_count].data.date.day    = local_atoi(line + offset + 6, 2);
+				entry->subentries[entry->subentries_count].data.date.hour   = local_atoi(line + offset + 8, 2);
+				entry->subentries[entry->subentries_count].data.date.minute = local_atoi(line + offset + 10, 2);
+				entry->subentries[entry->subentries_count].data.date.second = local_atoi(line + offset + 12, 2);
+				break;
+			default:
+				STORE_TOKEN(entry->subentries[entry->subentries_count].data.number);
+				break;
+			}
 		switch (o) {
 		case 0:
 			fprintf(stderr, _("Formatting error: unknown error while reading subentry contents\n"));
@@ -1616,14 +1603,6 @@ GNOKII_API gn_error gn_file_phonebook_raw_parse(gn_phonebook_entry *entry, char
 			goto endloop;
 		case 1:
 			fprintf(stderr, _("Formatting error: empty subentry contents\n"));
-			/* 0x13 Date Type; it is only for Dialed Numbers, etc.
-			   we don't store to this memories so it's an error to use it. */
-			if (entry->subentries[entry->subentries_count].entry_type == GN_PHONEBOOK_ENTRY_Date) {
-				fprintf(stderr, _("Cannot write to read-only memory (Dialed Numbers)\n"));
-				error = GN_ERR_WRONGDATAFORMAT;
-				/* With this error do not write a fake subentry (see below) */
-				goto endfunc;
-			}
 			break;
 		default:
 			break;
@@ -1637,10 +1616,10 @@ endloop:
 		entry->subentries[entry->subentries_count].entry_type   = GN_PHONEBOOK_ENTRY_Number;
 		entry->subentries[entry->subentries_count].number_type  = GN_PHONEBOOK_NUMBER_General;
 		entry->subentries[entry->subentries_count].id = 2;
-		strcpy(entry->subentries[entry->subentries_count].data.number, entry->number);
+		snprintf(entry->subentries[entry->subentries_count].data.number,
+			sizeof(entry->subentries[entry->subentries_count].data.number), "%s", entry->number);
 		entry->subentries_count = 1;
 	}
-endfunc:
 	return error;
 }
 
@@ -1653,10 +1632,46 @@ GNOKII_API gn_error gn_file_phonebook_raw_write(FILE *f, gn_phonebook_entry *ent
 	fprintf(f, "%s;%s;%s;%d;%d", escaped_name,
 		entry->number, memory_type_string,
 		entry->location, entry->caller_group);
+	if (entry->person.has_person) {
+		if (entry->person.honorific_prefixes[0])
+			fprintf(f, ";%d;0;0;%s", GN_PHONEBOOK_ENTRY_FormalName,
+				entry->person.honorific_prefixes);
+		if (entry->person.given_name[0])
+			fprintf(f, ";%d;0;0;%s", GN_PHONEBOOK_ENTRY_FirstName,
+				entry->person.given_name);
+		if (entry->person.family_name[0])
+			fprintf(f, ";%d;0;0;%s", GN_PHONEBOOK_ENTRY_LastName,
+				entry->person.family_name);
+	}
+	if (entry->address.has_address) {
+		if (entry->address.post_office_box[0])
+			fprintf(f, ";%d;0;0;%s", GN_PHONEBOOK_ENTRY_Postal,
+				entry->address.post_office_box);
+		if (entry->address.extended_address[0])
+			fprintf(f, ";%d;0;0;%s", GN_PHONEBOOK_ENTRY_ExtendedAddress,
+				entry->address.extended_address);
+		if (entry->address.street[0])
+			fprintf(f, ";%d;0;0;%s", GN_PHONEBOOK_ENTRY_Street,
+				entry->address.street);
+		if (entry->address.city[0])
+			fprintf(f, ";%d;0;0;%s", GN_PHONEBOOK_ENTRY_City,
+				entry->address.city);
+		if (entry->address.state_province[0])
+			fprintf(f, ";%d;0;0;%s", GN_PHONEBOOK_ENTRY_StateProvince,
+				entry->address.state_province);
+		if (entry->address.zipcode[0])
+			fprintf(f, ";%d;0;0;%s", GN_PHONEBOOK_ENTRY_ZipCode,
+				entry->address.zipcode);
+		if (entry->address.country[0])
+			fprintf(f, ";%d;0;0;%s", GN_PHONEBOOK_ENTRY_Country,
+				entry->address.country);
+	}
+
 	for (i = 0; i < entry->subentries_count; i++) {
 		switch (entry->subentries[i].entry_type) {
+		case GN_PHONEBOOK_ENTRY_Birthday:
 		case GN_PHONEBOOK_ENTRY_Date:
-			fprintf(f, ";%d;0;0;%04u%02u%02u%02u%02u%02u", GN_PHONEBOOK_ENTRY_Date,
+			fprintf(f, ";%d;0;%d;%04u%02u%02u%02u%02u%02u", entry->subentries[i].entry_type, entry->subentries[i].id,
 				entry->subentries[i].data.date.year, entry->subentries[i].data.date.month, entry->subentries[i].data.date.day,
 				entry->subentries[i].data.date.hour, entry->subentries[i].data.date.minute, entry->subentries[i].data.date.second);
 			break;
@@ -1679,4 +1694,3 @@ GNOKII_API gn_error gn_file_phonebook_raw_write(FILE *f, gn_phonebook_entry *ent
 	fprintf(f, "\n");
 	return GN_ERR_NONE;
 }
-
