@@ -1,6 +1,6 @@
 /*
 
-  $Id: gsm-networks.c,v 1.43 2006/06/11 16:39:48 deller Exp $
+  $Id: gsm-networks.c,v 1.47 2007/11/07 18:28:18 pkot Exp $
 
   G N O K I I
 
@@ -267,6 +267,7 @@ static gn_network networks[] = {
 	{"222 01", "Telecom Italia Mobile"},
 	{"222 10", "vodafone"},
 	{"222 88", "Wind Telecomunicazioni SpA"},
+	{"222 98", "Blu SpA"}, /* defunct year 2002 */
 	{"222 99", "H3G"},
 	{"226 01", "CONNEX GSM"},
 	{"226 03", "Cosmorom S.A."},
@@ -320,7 +321,6 @@ static gn_network networks[] = {
 	{"250 01", "Mobile Telesystems"},
 	{"250 01", "SANTEL"},
 	{"250 01", "Tambov RUS"},
-	{"250 02", "Megafon"},
 	{"250 02", "MegaFon"},
 	{"250 02", "MegaFon Moscow"},
 	{"250 03", "NCC"},
@@ -378,7 +378,7 @@ static gn_network networks[] = {
 	{"266 01", "Gibtel GSM"},
 	{"268 01", "vodafone"},
 	{"268 03", "OPTIMUS"},
-	{"268 06", "TMN"},
+	{"268 06", "P TMN"},
 	{"270 01", "LUXGSM"},
 	{"270 77", "TANGO"},
 	{"272 01", "Vodafone"},
@@ -859,8 +859,7 @@ GNOKII_API char *gn_network2country(char *network_code)
 {
 	char ccode[4];
 	
-	strncpy((char *)ccode, network_code, 3);
-	ccode[3] = 0;
+	snprintf(ccode, sizeof(ccode), "%s", network_code);
 
 	return gn_country_name_get(ccode);
 }
