@@ -1,6 +1,6 @@
 /*
 
-  $Id: gsm-sms.c,v 1.165 2007/10/05 20:59:34 pkot Exp $
+  $Id: gsm-sms.c,v 1.172 2007/11/21 20:32:01 pkot Exp $
 
   G N O K I I
 
@@ -34,6 +34,7 @@
 
 #include <stdlib.h>
 #include <string.h>
+#include <time.h>
 
 #include "gnokii-internal.h"
 #include "gnokii.h"
@@ -153,20 +154,28 @@ static char *sms_timestamp_print(u8 *number)
 	/* Ugly hack, but according to the GSM specs, the year is stored
          * as the 2 digit number. */
 	if ((10*(number[0] & 0x0f) + (number[0] >> 4)) < 70)
-		sprintf(buffer, "20");
+		snprintf(buffer, sizeof(buffer), "20");
 	else
-		sprintf(buffer, "19");
+		snprintf(buffer, sizeof(buffer), "19");
 
 	for (i = 0; i < 6; i++) {
 		int c;
 		char buf2[4];
 		switch (i) {
-		case 0: case 1: c = '-'; break;
-		case 3: case 4: c = ':'; break;
-		default: c = ' '; break;
+		case 0:
+		case 1:
+			c = '-';
+			break;
+		case 3:
+		case 4:
+			c = ':';
+			break;
+		default:
+			c = ' ';
+			break;
 		}
 		snprintf(buf2, 4, "%d%d%c", number[i] & 0x0f, number[i] >> 4, c);
-		strcat(buffer, buf2);
+		strncat(buffer, buf2, sizeof(buffer) - strlen(buffer));
 	}
 
 	/* The GSM spec is not clear what is the sign of the timezone when the
@@ -174,11 +183,13 @@ static char *sms_timestamp_print(u8 *number)
 	 * some are not. If your operator does use incompatible SMSC and wrong
 	 * sign disturbs you, change the sign here.
 	 */
-	if (number[6] & 0x08) strcat(buffer, "-");
-	else strcat(buffer, "+");
+	if (number[6] & 0x08)
+		strncat(buffer, "-", sizeof(buffer) - strlen(buffer));
+	else
+		strncat(buffer, "+", sizeof(buffer) - strlen(buffer));
 	/* The timezone is given in quarters. The base is GMT. */
-	sprintf(buf, "%02d00", (10 * (number[6] & 0x07) + (number[6] >> 4)) / 4);
-	strcat(buffer, buf);
+	snprintf(buf, sizeof(buf), "%02d00", (10 * (number[6] & 0x07) + (number[6] >> 4)) / 4);
+	strncat(buffer, buf, sizeof(buffer) - strlen(buffer));
 
 	return buffer;
 #undef LOCAL_DATETIME_MAX_LENGTH
@@ -197,16 +208,20 @@ static char *sms_timestamp_print(u8 *number)
  */
 gn_timestamp *sms_timestamp_unpack(unsigned char *number, gn_timestamp *dt)
 {
-	if (!dt) return NULL;
+	if (!dt)
+		return NULL;
 	memset(dt, 0, sizeof(gn_timestamp));
-	if (!number) return dt;
+	if (!number)
+		return dt;
 
-	dt->year     =  10 * (number[0] & 0x0f) + (number[0] >> 4);
+	dt->year = 10 * (number[0] & 0x0f) + (number[0] >> 4);
 
 	/* Ugly hack, but according to the GSM specs, the year is stored
          * as the 2 digit number. */
-	if (dt->year < 70) dt->year += 2000;
-	else dt->year += 1900;
+	if (dt->year < 70)
+		dt->year += 2000;
+	else
+		dt->year += 1900;
 
 	dt->month    =  10 * (number[1] & 0x0f) + (number[1] >> 4);
 	dt->day      =  10 * (number[2] & 0x0f) + (number[2] >> 4);
@@ -221,7 +236,8 @@ gn_timestamp *sms_timestamp_unpack(unsigned char *number, gn_timestamp *dt)
 	 * some are not. If your operator does use incompatible SMSC and wrong
 	 * sign disturbs you, change the sign here.
 	 */
-	if (number[6] & 0x08) dt->timezone = -dt->timezone;
+	if (number[6] & 0x08)
+		dt->timezone = -dt->timezone;
 
 	return dt;
 }
@@ -234,14 +250,18 @@ gn_timestamp *sms_timestamp_unpack(unsigned char *number, gn_timestamp *dt)
  */
 unsigned char *sms_timestamp_pack(gn_timestamp *dt, unsigned char *number)
 {
-	if (!number) return NULL;
+	if (!number)
+		return NULL;
 	memset(number, 0, GN_SMS_DATETIME_MAX_LENGTH);
-	if (!dt) return number;
+	if (!dt)
+		return number;
 
 	/* Ugly hack, but according to the GSM specs, the year is stored
          * as the 2 digit number. */
-	if (dt->year < 2000) dt->year -= 1900;
-	else dt->year -= 2000;
+	if (dt->year < 2000)
+		dt->year -= 1900;
+	else
+		dt->year -= 2000;
 
 	number[0]    = (dt->year   / 10) | ((dt->year   % 10) << 4);
 	number[1]    = (dt->month  / 10) | ((dt->month  % 10) << 4);
@@ -257,7 +277,8 @@ unsigned char *sms_timestamp_pack(gn_timestamp *dt, unsigned char *number)
 	 * some are not. If your operator does use incompatible SMSC and wrong
 	 * sign disturbs you, change the sign here.
 	 */
-	if (dt->timezone < 0) number[6] |= 0x08;
+	if (dt->timezone < 0)
+		number[6] |= 0x08;
 
 	return number;
 }
@@ -283,7 +304,8 @@ static gn_error sms_status(unsigned char status, gn_sms *sms)
 	sms->user_data[0].type = GN_SMS_DATA_Text;
 	sms->user_data[1].type = GN_SMS_DATA_None;
 	if (status < 0x03) {
-		strcpy(sms->user_data[0].u.text, _("Delivered"));
+		sms->user_data[0].dr_status = GN_SMS_DR_Status_Delivered;
+		snprintf(sms->user_data[0].u.text, sizeof(sms->user_data[0].u.text), "%s", _("Delivered"));
 		switch (status) {
 		case 0x00:
 			dprintf("SM received by the SME");
@@ -296,11 +318,11 @@ static gn_error sms_status(unsigned char status, gn_sms *sms)
 			break;
 		}
 	} else if (status & 0x40) {
-		strcpy(sms->user_data[0].u.text, _("Failed"));
+		snprintf(sms->user_data[0].u.text, sizeof(sms->user_data[0].u.text), "%s", _("Failed"));
 		/* more detailed reason only for debug */
 		if (status & 0x20) {
 			dprintf("Temporary error, SC is not making any more transfer attempts\n");
-
+			sms->user_data[0].dr_status = GN_SMS_DR_Status_Failed_Permanent;
 			switch (status) {
 			case 0x60:
 				dprintf("Congestion");
@@ -326,6 +348,7 @@ static gn_error sms_status(unsigned char status, gn_sms *sms)
 			}
 		} else {
 			dprintf("Permanent error, SC is not making any more transfer attempts\n");
+			sms->user_data[0].dr_status = GN_SMS_DR_Status_Failed_Temporary;
 			switch (status) {
 			case 0x40:
 				dprintf("Remote procedure error");
@@ -363,7 +386,8 @@ static gn_error sms_status(unsigned char status, gn_sms *sms)
 			}
 		}
 	} else if (status & 0x20) {
-		strcpy(sms->user_data[0].u.text, _("Pending"));
+		sms->user_data[0].dr_status = GN_SMS_DR_Status_Pending;
+		snprintf(sms->user_data[0].u.text, sizeof(sms->user_data[0].u.text), "%s", _("Pending"));
 		/* more detailed reason only for debug */
 		dprintf("Temporary error, SC still trying to transfer SM\n");
 		switch (status) {
@@ -390,7 +414,8 @@ static gn_error sms_status(unsigned char status, gn_sms *sms)
 			break;
 		}
 	} else {
-		strcpy(sms->user_data[0].u.text, _("Unknown"));
+		sms->user_data[0].dr_status = GN_SMS_DR_Status_Invalid;
+		snprintf(sms->user_data[0].u.text, sizeof(sms->user_data[0].u.text), "%s", _("Unknown"));
 
 		/* more detailed reason only for debug */
 		dprintf("Reserved/Specific to SC: %x", status);
@@ -643,6 +668,7 @@ static gn_error sms_pdu_decode(gn_sms_raw *rawsms, gn_sms *sms)
 
 	error = sms_header_decode(rawsms, sms, &sms->udh);
 	ERROR();
+	sms->user_data[0].dr_status = GN_SMS_DR_Status_None;
 	switch (sms->type) {
 	case GN_SMS_MT_DeliveryReport:
 		sms_status(rawsms->report_status, sms);
@@ -1324,7 +1350,7 @@ GNOKII_API gn_error gn_sms_send(gn_data *data, struct gn_statemachine *state)
 
 	dprintf("Sending\n");
 	error = gn_sm_functions(GN_OP_SendSMS, data, state);
- cleanup:
+cleanup:
 	free(data->raw_sms);
 	data->raw_sms = NULL;
 	return error;
@@ -1622,3 +1648,154 @@ GNOKII_API void gn_wap_push_init(gn_wap_push *wp)
 	wp->header.charset 	= WAPPush_CHARSET;
 	wp->header.stl 		= 0x00; /* string table length */
 }
+
+static char *status2str(gn_sms_message_status status)
+{
+	switch (status) {
+	case GN_SMS_Unread:
+		return "Unread";
+	case GN_SMS_Sent:
+		return "Sent";
+	case GN_SMS_Unsent:
+		return "Unsent";
+	default:
+		return "Read";
+	}
+}
+
+#define MAX_TEXT_SUMMARY	20
+#define MAX_SUBJECT_LENGTH	25
+#define MAX_DATE_LENGTH		255
+
+/* From snprintf(3) manual */
+static char *allocate(char *fmt, ...)
+{
+	char *str, *nstr;
+	int len, size = 100;
+	va_list ap;
+
+	str = calloc(100, sizeof(char));
+	if (!str)
+		return NULL;
+
+	while (1) {
+		va_start(ap, fmt);
+		len = vsnprintf(str, size, fmt, ap);
+		va_end(ap);
+		if (len >= size) /* too small buffer */
+			size = len + 1;
+		else if (len > -1) /* buffer OK */
+			return str;
+		else /* let's try with larger buffer */
+			size *= 2;
+		nstr = realloc(str, size);
+		if (!nstr) {
+			free(str);
+			return NULL;
+		}
+		str = nstr;
+	}
+}
+
+/* Here we allocate place for the line to append and we append it.
+ * We free() the appended line. */
+#define APPEND(dst, src, size) \
+do { \
+	char *ndst; \
+	int old = size; \
+	size += strlen(src); \
+	ndst = realloc(dst, size + 1); \
+	if (!ndst) { \
+		free(dst); \
+		goto error; \
+	} \
+	dst = ndst; \
+	dst[old] = 0; \
+	strcat(dst, src); \
+	free(src); \
+} while (0)
+
+#define CONCAT(dst, src, size, pattern, ...) \
+do { \
+	src = allocate(pattern, __VA_ARGS__); \
+	if (!src) \
+		goto error; \
+	APPEND(dst, src, size); \
+} while (0);
+
+/* Returns allocated space for mbox-compatible formatted SMS */
+/* TODO:
+ *   o add encoding information
+ *   o support non-text SMS (MMS, EMS, delivery report, ...)
+ */
+GNOKII_API char *gn_sms2mbox(gn_sms *sms, char *from)
+{
+	struct tm t, *loctime;
+	time_t caltime;
+	int size = 0;
+	char *tmp;
+#ifdef ENABLE_NLS
+	char *loc;
+#endif
+	char *buf = NULL, *aux = NULL;
+
+	t.tm_sec = sms->smsc_time.second;
+	t.tm_min = sms->smsc_time.minute;
+	t.tm_hour = sms->smsc_time.hour;
+	t.tm_mday = sms->smsc_time.day;
+	t.tm_mon = sms->smsc_time.month;
+	t.tm_year = sms->smsc_time.year - 1900;
+#ifdef HAVE_TM_GMTON
+	if (sms->smsc_time.timezone)
+		t.tm_gmtoff = sms->smsc_time.timezone * 3600;
+#endif
+	caltime = mktime(&t);
+	loctime = localtime(&caltime);
+
+#ifdef ENABLE_NLS
+	loc = setlocale(LC_ALL, "C");
+#endif
+	CONCAT(buf, tmp, size, "From %s@%s %s", sms->remote.number, from, asctime(loctime));
+
+	tmp = calloc(MAX_DATE_LENGTH, sizeof(char));
+	if (!tmp)
+		goto error;
+	strftime(tmp, MAX_DATE_LENGTH - 1, "Date: %a, %d %b %Y %H:%M:%S %z (%Z)\n", loctime);
+#ifdef ENABLE_NLS
+	setlocale(LC_ALL, loc);
+#endif
+	APPEND(buf, tmp, size);
+
+	CONCAT(buf, tmp, size, "From: %s@%s\n", sms->remote.number, from);
+	CONCAT(buf, tmp, size, "X-GSM-SMSC: %s\n", sms->smsc.number);
+	CONCAT(buf, tmp, size, "X-GSM-Status: %s\n", status2str(sms->status));
+	CONCAT(buf, tmp, size, "X-GSM-Memory: %s\n", gn_memory_type2str(sms->memory_type));
+
+	aux = calloc(16, sizeof(char)); /* assuming location will never have more than 15 digits */
+	if (!aux)
+		goto error;
+	snprintf(aux, 16, "%d", sms->number);
+	CONCAT(buf, tmp, size, "X-GSM-Location: %s\n", aux);
+	free(aux);
+
+	if (strlen(sms->user_data[0].u.text) < MAX_SUBJECT_LENGTH) {
+		CONCAT(buf, tmp, size, "Subject: %s\n\n", sms->user_data[0].u.text);
+	} else {
+		aux = calloc(MAX_TEXT_SUMMARY + 1, sizeof(char));
+		if (!aux)
+			goto error;
+		snprintf(aux, MAX_TEXT_SUMMARY, "%s", sms->user_data[0].u.text);
+		CONCAT(buf, tmp, size, "Subject: %s...\n\n", aux);
+		free(aux);
+	}
+
+	CONCAT(buf, tmp, size, "%s\n\n", sms->user_data[0].u.text);
+
+	return buf;
+error:
+	if (buf)
+		free(buf);
+	if (aux)
+		free(aux);
+	return NULL;
+}
