@@ -1,6 +1,6 @@
 /*
 
-  $Id: gsm-statemachine.c,v 1.61 2006/06/11 16:39:48 deller Exp $
+  $Id: gsm-statemachine.c,v 1.64 2007/11/20 23:45:38 pkot Exp $
 
   G N O K I I
 
@@ -93,6 +93,8 @@ void sm_reset(struct gn_statemachine *state)
 		state->current_state = GN_SM_Initialised;
 		state->waiting_for_number = 0;
 		state->received_number = 0;
+		if (state->link.reset)
+			state->link.reset(state);
 	}
 }
 
@@ -168,7 +170,8 @@ gn_error sm_error_get(unsigned char messagetype, struct gn_statemachine *state)
 	int c, d;
 	gn_error error = GN_ERR_NOTREADY;
 
-	if (state->current_state == GN_SM_ResponseReceived) {
+	switch (state->current_state) {
+	case GN_SM_ResponseReceived:
 		for (c = 0; c < state->received_number; c++)
 			if (state->waiting_for[c] == messagetype) {
 				error = state->response_error[c];
@@ -185,6 +188,13 @@ gn_error sm_error_get(unsigned char messagetype, struct gn_statemachine *state)
 			state->waiting_for_number = 0;
 			state->current_state = GN_SM_Initialised;
 		}
+		break;
+	/* Here we are fine! */
+	case GN_SM_Initialised:
+		error = GN_ERR_NONE;
+		break;
+	default:
+		break;
 	}
 
 	return error;
@@ -192,7 +202,7 @@ gn_error sm_error_get(unsigned char messagetype, struct gn_statemachine *state)
 
 
 
-/* Indicate that the phone code is waiting for an response */
+/* Indicate that the phone code is waiting for a response */
 /* This does not actually wait! */
 gn_error sm_wait_for(unsigned char messagetype, gn_data *data, struct gn_statemachine *state)
 {
@@ -240,7 +250,6 @@ static gn_error __sm_block_timeout(int waitfor, int t, gn_data *data, struct gn_
 			s = gn_sm_loop(1, state);  /* Timeout=100ms */
 			gettimeofday(&now, NULL);
 		} while (timercmp(&next, &now, >) && (s == GN_SM_MessageSent));
-
 		if (s == GN_SM_WaitingForResponse || s == GN_SM_ResponseReceived) break;
 
 		if (state->config.sm_retry) {
@@ -280,9 +289,11 @@ gn_error sm_block_timeout(int waitfor, int t, gn_data *data, struct gn_statemach
 
 	for (retry = 0; retry < 3; retry++) {
 		err = __sm_block_timeout(waitfor, t, data, state);
-		if (err == GN_ERR_NONE || err != GN_ERR_TIMEOUT) return err;
-		if (retry < 2)
+		if (err == GN_ERR_NONE || err != GN_ERR_TIMEOUT)
+			return err;
+		if (retry < 2) {
 			sm_message_send(state->last_msg_size, state->last_msg_type, state->last_msg, state);
+		}
 	}
 	return GN_ERR_TIMEOUT;
 }
