@@ -1,5 +1,5 @@
 /*
-  $Id: libfunctions.c,v 1.26 2006/08/09 19:36:58 pkot Exp $
+  $Id: libfunctions.c,v 1.38 2007/11/22 23:39:49 pkot Exp $
 
   G N O K I I
 
@@ -58,28 +58,30 @@ GNOKII_API unsigned int gn_lib_version()
 
 GNOKII_API gn_error gn_lib_phoneprofile_load_from_file(const char *configfile, const char *configname, struct gn_statemachine **state)
 {
-	gn_error error;
+	gn_error error = GN_ERR_NONE;
 	*state = NULL;
 
 	if (!gn_cfg_info) {
-		if (configfile)
+		if (configfile && *configfile)
 			error = gn_cfg_file_read(configfile);
 		else
 			error = gn_cfg_read_default();
-		if (GN_ERR_NONE != error)
-			return error;
 	}
+	if (GN_ERR_NONE == error) {
+		/* allocate and initialize data structures */
+		*state = malloc(sizeof(**state));
+		if (*state) {
+			memset(*state, 0, sizeof(**state));
 
-	/* allocate and initialize data structures */
-	*state = malloc(sizeof(**state));
-	if (!*state)
-		return GN_ERR_MEMORYFULL;
-	memset(*state, 0, sizeof(**state));
-
-	/* Load the phone configuration */
-	if ((error = gn_cfg_phone_load(configname, *state)) != GN_ERR_NONE) {
-		free(*state);
-		*state = NULL;
+			/* Load the phone configuration */
+			error = gn_cfg_phone_load(configname, *state);
+		} else {
+			error = GN_ERR_MEMORYFULL;
+		}
+	}
+	if (GN_ERR_NONE != error) {
+		gn_lib_phoneprofile_free(state);
+		gn_lib_library_free();
 		return error;
 	}
 	return LASTERROR((*state), GN_ERR_NONE);
@@ -119,12 +121,11 @@ GNOKII_API gn_error gn_lib_lasterror( struct gn_statemachine *state )
 
 GNOKII_API gn_error gn_lib_phone_open( struct gn_statemachine *state )
 {
-	const char *aux;
 	gn_error error;
 
+	state->lockfile = NULL;
 	/* should the device be locked with a lockfile ? */
-	aux = gn_lib_cfg_get("global", "use_locking");
-	if (aux && !strcmp(aux, "yes")) {
+	if (state->config.use_locking) {
 		state->lockfile = gn_device_lock(state->config.port_device);
 		if (state->lockfile == NULL) {
 			fprintf(stderr, _("Lock file error. Exiting.\n"));
@@ -137,6 +138,9 @@ GNOKII_API gn_error gn_lib_phone_open( struct gn_statemachine *state )
 	if (error != GN_ERR_NONE) {
 		fprintf(stderr, _("Telephone interface init failed: %s\nQuitting.\n"),
 			gn_error_print(error));
+		/* gn_device_unlock frees state->lockfile */
+		gn_device_unlock(state->lockfile);
+		state->lockfile = NULL;
 		return LASTERROR(state, error);
 	}
 
@@ -177,13 +181,13 @@ static gn_error gn_lib_get_phone_information( struct gn_statemachine *state )
 	error = gn_sm_functions(GN_OP_Identify, data, state);
 
 	if (!data->model[0])
-		strcpy(data->model,        unknown);
+		snprintf(data->model, GN_MODEL_MAX_LENGTH, "%s", unknown);
 	if (!data->manufacturer[0])
-		strcpy(data->manufacturer, unknown);
+		snprintf(data->manufacturer, GN_MANUFACTURER_MAX_LENGTH, "%s", unknown);
 	if (!data->revision[0])
-		strcpy(data->revision,     unknown);
+		snprintf(data->revision, GN_REVISION_MAX_LENGTH, "%s", unknown);
 	if (!data->imei[0])
-		strcpy(data->imei,         unknown);
+		snprintf(data->imei, GN_IMEI_MAX_LENGTH, "%s", unknown);
 
 	return LASTERROR(state, error);
 }
@@ -336,11 +340,11 @@ GNOKII_API gn_error gn_lib_phonebook_entry_delete( struct gn_statemachine *state
 	gn_error error;
 	gn_data *data = &state->sm_data;
 
-	state->u.pb_entry.memory_type = memory_type;
-	state->u.pb_entry.location = index;
 	data->phonebook_entry = &state->u.pb_entry;
 	memset(data->phonebook_entry, 0, sizeof(*data->phonebook_entry));
 	data->phonebook_entry->empty = 1;
+	data->phonebook_entry->memory_type = memory_type;
+	data->phonebook_entry->location = index;
 	error = gn_sm_functions(GN_OP_WritePhonebook, data, state);
 	return LASTERROR(state, error);
 }
@@ -366,15 +370,15 @@ GNOKII_API gn_error gn_lib_phonebook_write_entry( struct gn_statemachine *state,
 	return LASTERROR(state, error);
 }
 
-GNOKII_API gn_error gn_lib_set_pb_name( struct gn_statemachine *state, const char *name )
+GNOKII_API gn_error gn_lib_set_pb_name(struct gn_statemachine *state, const char *name)
 {
-	strncpy(state->u.pb_entry.name, name, sizeof(state->u.pb_entry.name)-1);
+	snprintf(state->u.pb_entry.name, sizeof(state->u.pb_entry.name), "%s", name);
 	return LASTERROR(state, GN_ERR_NONE);
 }
 
-GNOKII_API gn_error gn_lib_set_pb_number( struct gn_statemachine *state, const char *number )
+GNOKII_API gn_error gn_lib_set_pb_number(struct gn_statemachine *state, const char *number)
 {
-	strncpy(state->u.pb_entry.number, number, sizeof(state->u.pb_entry.number)-1);
+	snprintf(state->u.pb_entry.number, sizeof(state->u.pb_entry.number), "%s", number);
 	return LASTERROR(state, GN_ERR_NONE);
 }
 
@@ -402,8 +406,8 @@ GNOKII_API gn_error gn_lib_set_pb_date( struct gn_statemachine *state, gn_timest
 	return LASTERROR(state, GN_ERR_NONE);
 }
 
-GNOKII_API gn_error gn_lib_set_pb_subentry( struct gn_statemachine *state, const int index, /* index=-1 appends it */
-        gn_phonebook_entry_type entry_type, gn_phonebook_number_type number_type, const char *number )
+GNOKII_API gn_error gn_lib_set_pb_subentry(struct gn_statemachine *state, const int index, /* index=-1 appends it */
+        gn_phonebook_entry_type entry_type, gn_phonebook_number_type number_type, const char *number)
 {
 	int i = (index==-1) ? gn_lib_get_pb_num_subentries(state) : index;
 	if (i<0 || i>=GN_PHONEBOOK_SUBENTRIES_MAX_NUMBER)
@@ -414,7 +418,7 @@ GNOKII_API gn_error gn_lib_set_pb_subentry( struct gn_statemachine *state, const
 
 	state->u.pb_entry.subentries[i].entry_type  = entry_type;
 	state->u.pb_entry.subentries[i].number_type = number_type;
-	strncpy(state->u.pb_entry.subentries[i].data.number, number, sizeof(state->u.pb_entry.subentries[i].data.number)-1);
+	snprintf(state->u.pb_entry.subentries[i].data.number, sizeof(state->u.pb_entry.subentries[i].data.number), "%s", number);
 	return LASTERROR(state, GN_ERR_NONE);
 }
 
@@ -449,13 +453,22 @@ static struct { gn_connection_type ct; const char *str; } connectiontypes[] = {
 	{ GN_CT_Serial,     "serial" },
 	{ GN_CT_DAU9P,      "dau9p" },
 	{ GN_CT_DLR3P,      "dlr3p" },
-	{ GN_CT_Infrared,   "infrared" },
 	{ GN_CT_M2BUS,      "m2bus" },
+#ifdef HAVE_IRDA
+	{ GN_CT_Infrared,   "infrared" },
 	{ GN_CT_Irda,       "irda" },
+#endif
+#ifdef HAVE_BLUETOOTH
 	{ GN_CT_Bluetooth,  "bluetooth" },
+#endif
 	{ GN_CT_DLR3P,      "dku5" },
 	{ GN_CT_DKU2,       "dku2" },
+#ifdef HAVE_LIBUSB
 	{ GN_CT_DKU2LIBUSB, "dku2libusb" },
+#endif
+#ifdef HAVE_PCSC
+	{ GN_CT_PCSC,       "pcsc" },
+#endif
 #ifndef WIN32
 	{ GN_CT_TCP,        "tcp" },
 #endif
@@ -478,6 +491,61 @@ GNOKII_API const char *gn_lib_get_supported_connection(const int num)
 	return connectiontypes[num].str;
 }
 
+GNOKII_API const char *gn_lib_get_connection_name(gn_connection_type ct)
+{
+	int i;
+	for (i = 0; i < sizeof(connectiontypes)/sizeof(connectiontypes[0]); i++)
+		if (ct == connectiontypes[i].ct)
+			return connectiontypes[i].str;
+	return NULL;
+}
+
+GNOKII_API int gn_lib_is_connectiontype_supported(gn_connection_type ct)
+{
+	switch (ct) {
+	case GN_CT_Serial:
+	case GN_CT_DAU9P:
+	case GN_CT_DLR3P:
+	case GN_CT_M2BUS:
+	case GN_CT_Tekram:
+		return 1;
+	case GN_CT_Infrared:
+	case GN_CT_Irda:
+#ifdef HAVE_IRDA
+		return 1;
+#else
+		return 0;
+#endif
+	case GN_CT_Bluetooth:
+#ifdef HAVE_BLUETOOTH
+		return 1;
+#else
+		return 0;
+#endif
+	case GN_CT_DKU2: /* FIXME: How to detect this? */
+		return 1;
+	case GN_CT_DKU2LIBUSB:
+#ifdef HAVE_LIBUSB
+		return 1;
+#else
+		return 0;
+#endif
+	case GN_CT_PCSC:
+#ifdef HAVE_PCSC
+		return 1;
+#else
+		return 0;
+#endif
+	case GN_CT_TCP:
+#ifndef WIN32
+		return 0;
+#else
+		return 1;
+#endif
+	default:
+		return 0;
+	}
+}
 
 GNOKII_API gn_error gn_lib_search_one_connected_phone(struct gn_statemachine **state)
 {
