@@ -301,7 +301,7 @@ static bool cbus_open_serial(char *device, struct gn_statemachine *state)
  */
 static gn_error cbus_loop(struct timeval *timeout, struct gn_statemachine *state)
 {
-	unsigned char buffer[255];
+	unsigned char buffer[BUFFER_SIZE];
 	int count, res;
 
 	res = device_select(timeout, state);
@@ -334,6 +334,7 @@ gn_error cbus_initialise(struct gn_statemachine *state)
 	/* Fill in the link functions */
 	state->link.loop = &cbus_loop;
 	state->link.send_message = &cbus_send_message;
+	state->link.reset = NULL;
 	CBUSINST(state) = businst;
 
 	if (state->config.connection_type == GN_CT_Serial) {
