@@ -1,6 +1,6 @@
 /*
 
-  $Id: fbus-3110.c,v 1.36 2006/01/07 18:41:59 dforsi Exp $
+  $Id: fbus-3110.c,v 1.39 2007/11/22 17:30:20 pkot Exp $
 
   G N O K I I
 
@@ -50,6 +50,7 @@
 #include "device.h"
 
 #include "links/fbus-3110.h"
+#include "links/utils.h"
 
 static bool fb3110_serial_open(struct gn_statemachine *state);
 static void fb3110_rx_frame_handle(fb3110_incoming_frame *i, struct gn_statemachine *state);
@@ -221,12 +222,12 @@ static void fb3110_rx_state_machine(unsigned char rx_byte, struct gn_statemachin
  */
 static gn_error fb3110_loop(struct timeval *timeout, struct gn_statemachine *state)
 {
-	unsigned char buffer[255];
+	unsigned char buffer[BUFFER_SIZE];
 	int count, res;
 
 	res = device_select(timeout, state);
 	if (res > 0) {
-		res = device_read(buffer, 255, state);
+		res = device_read(buffer, sizeof(buffer), state);
 		for (count = 0; count < res; count++)
 			fb3110_rx_state_machine(buffer[count], state);
 	} else
@@ -328,6 +329,12 @@ static void fb3110_tx_ack_send(u8 messagetype, u8 seqno, struct gn_statemachine
 }
 
 
+static void fb3110_reset(struct gn_statemachine *state)
+{
+	/* Init variables */
+	FBUSINST(state)->i.state = FB3110_RX_Sync;
+}
+
 /* 
  * Initialise variables and start the link
  * newlink is actually part of state - but the link code should not
@@ -349,6 +356,7 @@ gn_error fb3110_initialise(struct gn_statemachine *state)
 	/* Fill in the link functions */
 	state->link.loop = &fb3110_loop;
 	state->link.send_message = &fb3110_message_send;
+	state->link.reset = &fb3110_reset;
 
 	/* Check for a valid init length */
 	if (state->config.init_length == 0)
@@ -380,8 +388,7 @@ gn_error fb3110_initialise(struct gn_statemachine *state)
 		device_write(&init_char, 1, state);
 	}
 
-	/* Init variables */
-	FBUSINST(state)->i.state = FB3110_RX_Sync;
+	fb3110_reset(state);
 
 	return GN_ERR_NONE;
 }
