@@ -1,6 +1,6 @@
 /*
 
-  $Id: fbus-phonet.c,v 1.40 2005/04/20 20:43:32 pkot Exp $
+  $Id: fbus-phonet.c,v 1.46 2007/11/22 17:30:20 pkot Exp $
 
   G N O K I I
 
@@ -50,6 +50,7 @@
 #include "device.h"
 #include "links/fbus-phonet.h"
 #include "links/fbus.h"
+#include "links/utils.h"
 #include "gnokii-internal.h"
 
 static void phonet_rx_statemachine(unsigned char rx_byte, struct gn_statemachine *state);
@@ -95,7 +96,8 @@ static bool phonet_open(struct gn_statemachine *state)
 		device_write(&init_sequence, 7, state);
 		while (total < 7) {
 			n = device_read(&init_resp + total, 7 - total, state);
-			total += n;
+			if (n > 0)
+				total += n;
 		}
 		for (i = 0; i < n; i++) {
 			if (init_resp[i] != init_pattern[i]) {
@@ -128,7 +130,6 @@ static void phonet_rx_statemachine(unsigned char rx_byte, struct gn_statemachine
 		break;
 
 	case FBUS_RX_GetDestination:
-
 		i->message_destination = rx_byte;
 		i->state = FBUS_RX_GetSource;
 
@@ -141,7 +142,6 @@ static void phonet_rx_statemachine(unsigned char rx_byte, struct gn_statemachine
 		break;
 
 	case FBUS_RX_GetSource:
-
 		i->message_source = rx_byte;
 		i->state = FBUS_RX_GetType;
 
@@ -153,29 +153,24 @@ static void phonet_rx_statemachine(unsigned char rx_byte, struct gn_statemachine
 		break;
 
 	case FBUS_RX_GetType:
-
 		i->message_type = rx_byte;
 		i->state = FBUS_RX_GetLength1;
 
 		break;
 
 	case FBUS_RX_GetLength1:
-
 		i->message_length = rx_byte << 8;
 		i->state = FBUS_RX_GetLength2;
 
 		break;
 
 	case FBUS_RX_GetLength2:
-
 		i->message_length = i->message_length + rx_byte;
 		i->state = FBUS_RX_GetMessage;
 		i->buffer_count = 0;
-
 		break;
 
 	case FBUS_RX_GetMessage:
-
 		i->message_buffer[i->buffer_count] = rx_byte;
 		i->buffer_count++;
 
@@ -186,7 +181,6 @@ static void phonet_rx_statemachine(unsigned char rx_byte, struct gn_statemachine
 		}
 
 		/* Is that it? */
-
 		if (i->buffer_count == i->message_length) {
 			sm_incoming_function(i->message_type, i->message_buffer, i->message_length, state);
 			i->state = FBUS_RX_Sync;
@@ -205,13 +199,13 @@ static void phonet_rx_statemachine(unsigned char rx_byte, struct gn_statemachine
 static gn_error phonet_loop(struct timeval *timeout, struct gn_statemachine *state)
 {
 	gn_error	error = GN_ERR_INTERNALERROR;
-	unsigned char	buffer[255];
+	unsigned char	buffer[BUFFER_SIZE];
 	int		count, res;
 
 	res = device_select(timeout, state);
 
 	if (res > 0) {
-		res = device_read(buffer, 255, state);
+		res = device_read(buffer, sizeof(buffer), state);
 		for (count = 0; count < res; count++) {
 			phonet_rx_statemachine(buffer[count], state);
 		}
@@ -241,18 +235,23 @@ static gn_error phonet_send_message(unsigned int messagesize, unsigned char mess
 
 	/* Now construct the message header. */
 
-	if (state->config.connection_type == GN_CT_Bluetooth) {
+	switch (state->config.connection_type) {
+	case GN_CT_Bluetooth:
 		out_buffer[current++] = FBUS_PHONET_BLUETOOTH_FRAME_ID;
 		out_buffer[current++] = FBUS_DEVICE_PHONE;
 		out_buffer[current++] = FBUS_PHONET_BLUETOOTH_DEVICE_PC;
-	} else if (state->config.connection_type == GN_CT_DKU2) {
+		break;
+	case GN_CT_DKU2:
+	case GN_CT_DKU2LIBUSB:
 		out_buffer[current++] = FBUS_PHONET_DKU2_FRAME_ID;
 		out_buffer[current++] = FBUS_DEVICE_PHONE;
 		out_buffer[current++] = FBUS_PHONET_DKU2_DEVICE_PC;
-	} else {
+		break;
+	default:
 		out_buffer[current++] = FBUS_PHONET_FRAME_ID;
 		out_buffer[current++] = FBUS_DEVICE_PHONE; /* Destination */
 		out_buffer[current++] = FBUS_DEVICE_PC;    /* Source */
+		break;
 	}
 
 	out_buffer[current++] = messagetype; /* Type */
@@ -284,6 +283,13 @@ static gn_error phonet_send_message(unsigned int messagesize, unsigned char mess
 	return GN_ERR_NONE;
 }
 
+static void phonet_reset(struct gn_statemachine *state)
+{
+	/* Init variables */
+	FBUSINST(state)->state = FBUS_RX_Sync;
+	FBUSINST(state)->buffer_count = 0;
+}
+
 /* Initialise variables and start the link */
 gn_error phonet_initialise(struct gn_statemachine *state)
 {
@@ -295,6 +301,7 @@ gn_error phonet_initialise(struct gn_statemachine *state)
 	/* Fill in the link functions */
 	state->link.loop = &phonet_loop;
 	state->link.send_message = &phonet_send_message;
+	state->link.reset = &phonet_reset;
 
 	if ((FBUSINST(state) = calloc(1, sizeof(phonet_incoming_message))) == NULL)
 		return GN_ERR_MEMORYFULL;
@@ -303,6 +310,7 @@ gn_error phonet_initialise(struct gn_statemachine *state)
 	case GN_CT_Infrared:
 	case GN_CT_Irda:
 	case GN_CT_DKU2:
+	case GN_CT_DKU2LIBUSB:
 	case GN_CT_Bluetooth:
 		if (phonet_open(state) == true)
 			error = GN_ERR_NONE;
@@ -316,9 +324,7 @@ gn_error phonet_initialise(struct gn_statemachine *state)
 		return error;
 	}
 
-	/* Init variables */
-	FBUSINST(state)->state = FBUS_RX_Sync;
-	FBUSINST(state)->buffer_count = 0;
+	phonet_reset(state);
 
 	return error;
 }
