@@ -1,6 +1,6 @@
 /*
 
-  $Id: fbus.c,v 1.63 2006/05/08 19:20:07 deller Exp $
+  $Id: fbus.c,v 1.73 2007/11/22 17:30:20 pkot Exp $
 
   G N O K I I
 
@@ -51,6 +51,7 @@
 #include "gnokii.h"
 #include "device.h"
 #include "links/fbus.h"
+#include "links/utils.h"
 #include "gnokii-internal.h"
 
 static void fbus_rx_statemachine(unsigned char rx_byte, struct gn_statemachine *state);
@@ -78,7 +79,8 @@ static bool fbus_serial_open(bool dlr3, struct gn_statemachine *state)
 	else
 		type = GN_CT_Serial;
 
-	if (dlr3) dlr3 = 1;
+	if (dlr3)
+		dlr3 = 1;
 	/* Open device. */
 	if (!device_open(state->config.port_device, false, false, false, type, state)) {
 		perror(_("Couldn't open FBUS device"));
@@ -96,11 +98,11 @@ static int send_command(char *cmd, int len, struct gn_statemachine *state)
 {
 	struct timeval timeout;
 	unsigned char buffer[255];
-	int res;
+	int res, offset = 0, waitformore = 1;
 
 	/* Communication with the phone looks strange here. I am unable to
 	 * read the whole answer from the port with DKU-5 cable and
-	 * Nokia 6020. The following code seems to work reliable.
+	 * Nokia 6020. The following code seems to work reliably.
 	 */
 	device_write(cmd, len, state);
 	/* Experimental timeout */
@@ -109,10 +111,19 @@ static int send_command(char *cmd, int len, struct gn_statemachine *state)
         
 	res = device_select(&timeout, state);
 	/* Read from the port only when select succeeds */
-	while (res > 0) {
+	while (res > 0 && waitformore) {
 		/* Avoid 'device temporarily unavailable' error */
-		usleep(50);
-		res = device_read(buffer, 255, state);
+		usleep(50000);
+		res = device_read(buffer + offset, sizeof(buffer) - offset, state);
+		/* The whole answer is read */
+		if (strstr(buffer, "OK"))
+			waitformore = 0;
+		if (res > 0)
+			offset += res;
+		res = offset;
+		/* The phone is already in AT mode */
+		if (strchr(buffer, 0x55))
+			res = 0;
 	}
 	return res;
 }
@@ -138,12 +149,14 @@ static bool at2fbus_serial_open(struct gn_statemachine *state, gn_connection_typ
 	usleep(1000000);
 	device_changespeed(19200, state);
 	dprintf("Switching to FBUS mode\n");
-	/* Here we can be either switch to FBUS or not. Assume 0.5 second
+	/* Here we can be either switched to FBUS or not. Assume 0.5 second
 	 * timeout for the answer.
 	 */
 	res = send_command("AT\r\n", 4, state);
-	res = send_command("AT&F\r\n", 6, state);
-	res = send_command("AT*NOKIAFBUS\r\n", 14, state);
+	if (res)
+		res = send_command("AT&F\r\n", 6, state);
+	if (res)
+		res = send_command("AT*NOKIAFBUS\r\n", 14, state);
 	device_changespeed(115200, state);
 
 	if (type != GN_CT_Bluetooth && type != GN_CT_TCP) { 
@@ -327,7 +340,7 @@ static void fbus_rx_statemachine(unsigned char rx_byte, struct gn_statemachine *
 	case FBUS_RX_GetMessage:
 
 		if (i->buffer_count >= FBUS_FRAME_MAX_LENGTH) {
-			dprintf("FBUS: Message buffer overun - resetting\n");
+			dprintf("FBUS: Message buffer overrun - resetting\n");
 			i->state = FBUS_RX_Sync;
 			break;
 		}
@@ -365,9 +378,7 @@ static void fbus_rx_statemachine(unsigned char rx_byte, struct gn_statemachine *
 					frm_num = i->message_buffer[i->frame_length - 2];
 					seq_num = i->message_buffer[i->frame_length - 1];
 
-
 					/* 0x40 in the sequence number indicates first frame of a message */
-
 					if ((seq_num & 0x40) == 0x40) {
 						/* Fiddle around and malloc some memory */
 						m->message_length = 0;
@@ -454,7 +465,7 @@ static void fbus_rx_statemachine(unsigned char rx_byte, struct gn_statemachine *
 	case FBUS_RX_EchoMessage:
 
 		if (i->buffer_count >= FBUS_FRAME_MAX_LENGTH) {
-			dprintf("FBUS: Message buffer overun - resetting\n");
+			dprintf("FBUS: Message buffer overrun - resetting\n");
 			i->state = FBUS_RX_Sync;
 			break;
 		}
@@ -476,12 +487,12 @@ static void fbus_rx_statemachine(unsigned char rx_byte, struct gn_statemachine *
 
 static gn_error fbus_loop(struct timeval *timeout, struct gn_statemachine *state)
 {
-	unsigned char buffer[255];
+	unsigned char buffer[BUFFER_SIZE];
 	int count, res;
 
 	res = device_select(timeout, state);
 	if (res > 0) {
-		res = device_read(buffer, 255, state);
+		res = device_read(buffer, sizeof(buffer), state);
 		for (count = 0; count < res; count++)
 			fbus_rx_statemachine(buffer[count], state);
 	} else
@@ -622,19 +633,32 @@ static int fbus_tx_send_ack(u8 message_type, u8 message_seq, struct gn_statemach
 	request[0] = message_type;
 	request[1] = message_seq;
 
-	dprintf("[Sending Ack of type %02x, seq: %x]\n",message_type, message_seq);
+	dprintf("[Sending Ack of type %02x, seq: %x]\n", message_type, message_seq);
 
 	return fbus_tx_send_frame(2, 0x7f, request, state);
 }
 
 
-/* Initialise variables and start the link */
-/* state is only passed around to allow for muliple state machines (one day...) */
+static void fbus_reset(struct gn_statemachine *state)
+{
+	int count;
 
+	/* Init variables */
+	FBUSINST(state)->i.state = FBUS_RX_Sync;
+	FBUSINST(state)->i.buffer_count = 0;
+	for (count = 0; count < FBUS_MESSAGE_MAX_TYPES; count++) {
+		FBUSINST(state)->messages[count].malloced = 0;
+		FBUSINST(state)->messages[count].frames_to_go = 0;
+		FBUSINST(state)->messages[count].message_length = 0;
+		FBUSINST(state)->messages[count].message_buffer = NULL;
+	}
+}
+
+/* Initialise variables and start the link */
+/* state is only passed around to allow for multiple state machines (one day...) */
 gn_error fbus_initialise(int attempt, struct gn_statemachine *state)
 {
 	unsigned char init_char = 0x55;
-	int count;
 	bool connection = false;
 
 	if (!state)
@@ -643,6 +667,7 @@ gn_error fbus_initialise(int attempt, struct gn_statemachine *state)
 	/* Fill in the link functions */
 	state->link.loop = &fbus_loop;
 	state->link.send_message = &fbus_send_message;
+	state->link.reset = &fbus_reset;
 
 	/* Check for a valid init length */
 	if (state->config.init_length == 0)
@@ -707,21 +732,14 @@ gn_error fbus_initialise(int attempt, struct gn_statemachine *state)
 	/* I believe that we need/can do this for any phone to get the UART synced */
 
 	if (state->config.connection_type != GN_CT_Bluetooth && state->config.connection_type != GN_CT_TCP) {
+		int count;
 		for (count = 0; count < state->config.init_length; count++) {
 			usleep(100);
 			device_write(&init_char, 1, state);
 		}
 	}
 
-	/* Init variables */
-	FBUSINST(state)->i.state = FBUS_RX_Sync;
-	FBUSINST(state)->i.buffer_count = 0;
-	for (count = 0; count < FBUS_MESSAGE_MAX_TYPES; count++) {
-		FBUSINST(state)->messages[count].malloced = 0;
-		FBUSINST(state)->messages[count].frames_to_go = 0;
-		FBUSINST(state)->messages[count].message_length = 0;
-		FBUSINST(state)->messages[count].message_buffer = NULL;
-	}
+	fbus_reset(state);
 
 	return GN_ERR_NONE;
 }
