@@ -1,6 +1,6 @@
 /*
 
-  $Id: gnbus.c,v 1.5 2005/11/23 18:55:11 bozo Exp $
+  $Id: gnbus.c,v 1.8 2007/11/22 17:30:20 pkot Exp $
 
   G N O K I I
 
@@ -47,6 +47,7 @@
 #include "gnokii.h"
 #include "device.h"
 #include "links/gnbus.h"
+#include "links/utils.h"
 
 #include "gnokii-internal.h"
 
@@ -178,7 +179,7 @@ static void gnbus_rx_statemachine(unsigned char rx_byte, struct gn_statemachine
 
 static gn_error gnbus_loop(struct timeval *timeout, struct gn_statemachine *state)
 {
-	unsigned char buffer[256];
+	unsigned char buffer[BUFFER_SIZE];
 	int count, res;
 
 	res = device_select(timeout, state);
@@ -268,10 +269,15 @@ static gn_error gnbus_send_message(unsigned int messagesize, unsigned char messa
 }
 
 
+static void gnbus_reset(struct gn_statemachine *state)
+{
+	GNBUSINST(state)->i.state = GNBUS_RX_Sync;
+	GNBUSINST(state)->i.checksum_idx = 0;
+}
+
 /*
  * Initialise variables and start the link
  */
-
 gn_error gnbus_initialise(struct gn_statemachine *state)
 {
 	int conn_type;
@@ -280,15 +286,15 @@ gn_error gnbus_initialise(struct gn_statemachine *state)
 		return GN_ERR_FAILED;
 
 	/* Fill in the link functions */
-	state->link.loop = gnbus_loop;
-	state->link.send_message = gnbus_send_message;
+	state->link.loop = &gnbus_loop;
+	state->link.send_message = &gnbus_send_message;
+	state->link.reset = &gnbus_reset;
 
 	/* Start up the link */
 	if ((GNBUSINST(state) = calloc(1, sizeof(gnbus_link))) == NULL)
 		return GN_ERR_MEMORYFULL;
 
-	GNBUSINST(state)->i.state = GNBUS_RX_Sync;
-	GNBUSINST(state)->i.checksum_idx = 0;
+	gnbus_reset(state);
 
 	if (state->config.connection_type == GN_CT_Irda && strcasecmp(state->config.port_device, "IrDA:IrCOMM"))
 		conn_type = GN_CT_Serial;
