@@ -1,6 +1,6 @@
 /*
 
-  $Id: m2bus.c,v 1.21 2005/04/20 20:43:32 pkot Exp $
+  $Id: m2bus.c,v 1.24 2007/11/22 17:30:20 pkot Exp $
 
   G N O K I I
 
@@ -48,6 +48,7 @@
 #include "gnokii.h"
 #include "device.h"
 #include "links/m2bus.h"
+#include "links/utils.h"
 
 #include "gnokii-internal.h"
 
@@ -268,7 +269,7 @@ static void m2bus_rx_statemachine(unsigned char rx_byte, struct gn_statemachine
 
 static gn_error m2bus_loop(struct timeval *timeout, struct gn_statemachine *state)
 {
-	unsigned char buffer[255];
+	unsigned char buffer[BUFFER_SIZE];
 	int count, res;
 
 	res = device_select(timeout, state);
@@ -446,9 +447,13 @@ static gn_error m2bus_tx_send_ack(u8 message_seq, struct gn_statemachine *state)
 }
 
 
+static void m2bus_reset(struct gn_statemachine *state)
+{
+	M2BUSINST(state)->i.state = M2BUS_RX_Sync;
+}
+
 /* Initialise variables and start the link */
 /* state is only passed around to allow for muliple state machines (one day...) */
-
 gn_error m2bus_initialise(struct gn_statemachine *state)
 {
 	gn_error err;
@@ -457,15 +462,16 @@ gn_error m2bus_initialise(struct gn_statemachine *state)
 		return GN_ERR_FAILED;
 
 	/* Fill in the link functions */
-	state->link.loop = m2bus_loop;
-	state->link.send_message = m2bus_send_message;
+	state->link.loop = &m2bus_loop;
+	state->link.send_message = &m2bus_send_message;
+	state->link.reset = &m2bus_reset;
 
 	/* Start up the link */
 	if ((M2BUSINST(state) = calloc(1, sizeof(m2bus_link))) == NULL)
 		return GN_ERR_MEMORYFULL;
 
 	M2BUSINST(state)->request_sequence_number = 2;
-	M2BUSINST(state)->i.state = M2BUS_RX_Sync;
+	m2bus_reset(state);
 
 	if (state->config.connection_type == GN_CT_Infrared) {
 		err = GN_ERR_FAILED;
