@@ -1,6 +1,6 @@
 /*
 
-  $Id: misc.c,v 1.127 2007/09/23 20:58:51 pkot Exp $
+  $Id: misc.c,v 1.135 2007/11/10 19:00:58 pkot Exp $
 
   G N O K I I
 
@@ -126,6 +126,7 @@ static gn_phone_model models[] = {
 	{"2300",  "RM-4",	PM_OLD_DEFAULT },
 	{"2300b", "RM-5",	PM_OLD_DEFAULT },
 	{"2600",  "RH-59",	PM_DEFAULT | PM_XGNOKIIBREAKAGE },
+	{"2630",  "RM-298",	PM_DEFAULT },
 	{"2650",  "RH-53",	PM_DEFAULT | PM_XGNOKIIBREAKAGE },
 	{"3100",  "RH-19",	PM_DEFAULT | PM_XGNOKIIBREAKAGE },
 	{"3100b", "RH-50",	PM_DEFAULT },
@@ -133,6 +134,7 @@ static gn_phone_model models[] = {
 	{"3108",  "RH-6",	PM_OLD_DEFAULT },
 	{"3110",  "NHE-8",	PM_OLD_DEFAULT | PM_DATA },
 	{"3110",  "0310" ,	PM_OLD_DEFAULT | PM_DATA }, /* NHE-8 */
+	{"3110c", "RM-237",     PM_DEFAULT }, /* Nokia 3110 classic */
 	{"3120",  "RH-19",	PM_DEFAULT },
 	{"3120b", "RH-50",	PM_DEFAULT },
 	{"3200",  "RH-30",	PM_DEFAULT | PM_XGNOKIIBREAKAGE },
@@ -199,7 +201,9 @@ static gn_phone_model models[] = {
 	{"6015i", "RH-55",	PM_DEFAULT | PM_NETMONITOR },
 	{"6020",  "RM-30",	PM_DEFAULT | PM_NETMONITOR },
 	{"6021",  "RM-94",	PM_DEFAULT | PM_NETMONITOR | PM_XGNOKIIBREAKAGE},
+	{"6030",  "RM-74",      PM_DEFAULT },
 	{"6050",  "NME-1",	PM_OLD_DEFAULT },
+	{"6070",  "RM-166",	PM_DEFAULT },
 	{"6080",  "NME-2",	PM_OLD_DEFAULT },
 	{"6081",  "NME-2A",	PM_OLD_DEFAULT },
 	{"6081",  "NME-2E",	PM_OLD_DEFAULT },
@@ -255,6 +259,7 @@ static gn_phone_model models[] = {
 	{"6810",  "RM-2",	PM_DEFAULT | PM_XGNOKIIBREAKAGE },
 	{"6820",  "NHL-9",	PM_DEFAULT | PM_XGNOKIIBREAKAGE },
 	{"6820b", "RH-26",	PM_DEFAULT | PM_XGNOKIIBREAKAGE },
+	{"6822",  "RM-69",	PM_DEFAULT },
 	{"7110",  "NSE-5",	PM_DEFAULT | PM_NETMONITOR },
 	{"7160",  "NSW-5",	PM_OLD_DEFAULT },
 	{"7190",  "NSB-5",	PM_DEFAULT | PM_NETMONITOR },
@@ -311,6 +316,7 @@ static gn_phone_model models[] = {
 	{"N-Gage", "NEM-4",	PM_OLD_DEFAULT },
 	{"RinGo", "NHX-7",	PM_OLD_DEFAULT },
 	{"sx1",  "SX1",		PM_DEFAULT },
+	{"SIM/USIM", "pcsc",	0 },
 	{NULL,    NULL, 0 } /* keep last one as NULL */
 };
 
@@ -455,10 +461,10 @@ GNOKII_API char *gn_device_lock(const char* port)
 		return NULL;
 	}
 	/* I think we don't need to use strncpy, as we should have enough
-	 * buffer due to strlen results
+	 * buffer due to strlen results, but it's safer to do so...
 	 */
-	strcpy(lock_file, lock_path);
-	strcat(lock_file, aux);
+	strncpy(lock_file, lock_path, len);
+	strncat(lock_file, aux, len - strlen(lock_file));
 
 	/* Check for the stale lockfile.
 	 * The code taken from minicom by Miquel van Smoorenburg */
@@ -514,12 +520,16 @@ GNOKII_API char *gn_device_lock(const char* port)
 			fprintf(stderr, _("Cannot create lockfile %s. Please check for existence of the path."), lock_file);
 		goto failed;
 	}
-	sprintf(buffer, "%10ld gnokii\n", (long)getpid());
-	/* Probably we should add some error checking in here */
-	write(fd, buffer, strlen(buffer));
+	snprintf(buffer, sizeof(buffer), "%10ld gnokii\n", (long)getpid());
+	if (write(fd, buffer, strlen(buffer)) < 0) {
+		fprintf(stderr, _("Failed to write to the lockfile %s.\n"), lock_file);
+		goto failed;
+	}
 	close(fd);
 	return lock_file;
 failed:
+	if (fd > -1)
+		close(fd);
 	free(lock_file);
 	return NULL;
 #else
