@@ -1,6 +1,6 @@
 /*
 
-  $Id: atgen.c,v 1.154 2007/08/05 16:28:04 pkot Exp $
+  $Id: atgen.c,v 1.166 2007/11/19 14:51:50 pkot Exp $
 
   G N O K I I
 
@@ -46,6 +46,7 @@
 #include "phones/atgen.h"
 #include "phones/atbosch.h"
 #include "phones/ateric.h"
+#include "phones/atmot.h"
 #include "phones/atnok.h"
 #include "phones/atsie.h"
 #include "phones/atsoer.h"
@@ -85,13 +86,13 @@ static gn_error AT_GetModel(gn_data *data, struct gn_statemachine *state);
 static gn_error AT_GetRevision(gn_data *data, struct gn_statemachine *state);
 static gn_error AT_GetIMEI(gn_data *data, struct gn_statemachine *state);
 static gn_error AT_GetManufacturer(gn_data *data, struct gn_statemachine *state);
-static gn_error AT_GetBattery(gn_data *data,  struct gn_statemachine *state);
-static gn_error AT_GetRFLevel(gn_data *data,  struct gn_statemachine *state);
-static gn_error AT_GetMemoryStatus(gn_data *data,  struct gn_statemachine *state);
-static gn_error AT_GetMemoryRange(gn_data *data,  struct gn_statemachine *state);
-static gn_error AT_ReadPhonebook(gn_data *data,  struct gn_statemachine *state);
-static gn_error AT_WritePhonebook(gn_data *data,  struct gn_statemachine *state);
-static gn_error AT_DeletePhonebook(gn_data *data,  struct gn_statemachine *state);
+static gn_error AT_GetBattery(gn_data *data, struct gn_statemachine *state);
+static gn_error AT_GetRFLevel(gn_data *data, struct gn_statemachine *state);
+static gn_error AT_GetMemoryStatus(gn_data *data, struct gn_statemachine *state);
+static gn_error AT_GetMemoryRange(gn_data *data, struct gn_statemachine *state);
+static gn_error AT_ReadPhonebook(gn_data *data, struct gn_statemachine *state);
+static gn_error AT_WritePhonebook(gn_data *data, struct gn_statemachine *state);
+static gn_error AT_DeletePhonebook(gn_data *data, struct gn_statemachine *state);
 static gn_error AT_CallDivert(gn_data *data, struct gn_statemachine *state);
 static gn_error AT_SetPDUMode(gn_data *data, struct gn_statemachine *state);
 static gn_error AT_GetSMSStatus(gn_data *data, struct gn_statemachine *state);
@@ -114,6 +115,7 @@ static gn_error AT_CancelCall(gn_data *data, struct gn_statemachine *state);
 static gn_error AT_SetCallNotification(gn_data *data, struct gn_statemachine *state);
 static gn_error AT_SetDateTime(gn_data *data, struct gn_statemachine *state);
 static gn_error AT_GetDateTime(gn_data *data, struct gn_statemachine *state);
+static gn_error AT_PrepareDateTime(gn_data *data, struct gn_statemachine *state);
 static gn_error AT_SendDTMF(gn_data *data, struct gn_statemachine *state);
 static gn_error AT_GetActiveCalls(gn_data *data, struct gn_statemachine *state);
 static gn_error AT_OnSMS(gn_data *data, struct gn_statemachine *state);
@@ -165,6 +167,7 @@ static at_function_init_type at_function_init[] = {
 	{ GN_OP_GetNetworkInfo,        AT_GetNetworkInfo,        ReplyGetNetworkInfo },
 	{ GN_OP_SetDateTime,           AT_SetDateTime,           Reply },
 	{ GN_OP_GetDateTime,           AT_GetDateTime,           ReplyGetDateTime },
+	{ GN_OP_AT_PrepareDateTime,    AT_PrepareDateTime,       Reply },
 	{ GN_OP_SendDTMF,              AT_SendDTMF,              Reply },
 	{ GN_OP_GetActiveCalls,        AT_GetActiveCalls,        ReplyGetActiveCalls },
 	{ GN_OP_OnSMS,                 AT_OnSMS,                 Reply },
@@ -206,12 +209,11 @@ static void reply_simpletext(char *l1, char *l2, char *c, char *t, size_t maxlen
 	if ((strncmp(l1, c, n - 2) == 0) && (t != NULL)) {
 		if (strncmp(l2, c, n) == 0)  {
 			for (i = n; isspace(l2[i]); i++) ;
-			strncpy(t, strip_quotes(l2 + i), maxlength);
+			snprintf(t, maxlength, "%s", strip_quotes(l2 + i));
 		} else {
 			for (i = 0; isspace(l2[i]); i++) ;
-			strncpy(t, l2 + i, maxlength);
+			snprintf(t, maxlength, "%s", l2 + i);
 		}
-		t[maxlength - 1] = '\0';
 	}
 }
 
@@ -545,14 +547,15 @@ gn_error at_memory_type_set(gn_memory_type mt, struct gn_statemachine *state)
 	if (mt != drvinst->memorytype) {
 		if (mt >= NR_MEMORIES)
 			return GN_ERR_INVALIDMEMORYTYPE;
-		sprintf(req, "AT+CPBS=\"%s\"\r", memorynames[mt]);
+		snprintf(req, sizeof(req), "AT+CPBS=\"%s\"\r", memorynames[mt]);
 		ret = sm_message_send(13, GN_OP_Init, req, state);
 		if (ret)
 			return GN_ERR_NOTREADY;
 		gn_data_clear(&data);
 		ret = sm_block_no_retry(GN_OP_Init, &data, state);
-		if (ret == GN_ERR_NONE)
-			drvinst->memorytype = mt;
+		if (ret)
+			return ret;
+		drvinst->memorytype = mt;
 
 		gn_data_clear(&data);
 		ret = state->driver.functions(GN_OP_AT_GetMemoryRange, &data, state);
@@ -581,7 +584,7 @@ gn_error AT_SetSMSMemoryType(gn_memory_type mt, struct gn_statemachine *state)
 		if (mt >= NR_MEMORIES)
 			return GN_ERR_INVALIDMEMORYTYPE;
 		gn_data_clear(&data);
-		sprintf(req, "AT+CPMS=\"%s\"\r", memorynames[mt]);
+		snprintf(req, sizeof(req), "AT+CPMS=\"%s\"\r", memorynames[mt]);
 		ret = sm_message_send(13, GN_OP_Init, req, state);
 		if (ret == GN_ERR_NONE)
 			ret = sm_block_no_retry(GN_OP_Init, &data, state);
@@ -809,7 +812,7 @@ static gn_error AT_ReadPhonebook(gn_data *data, struct gn_statemachine *state)
 	ret = at_memory_type_set(data->phonebook_entry->memory_type, state);
 	if (ret)
 		return ret;
-	sprintf(req, "AT+CPBR=%d\r", data->phonebook_entry->location+drvinst->memoryoffset);
+	snprintf(req, sizeof(req), "AT+CPBR=%d\r", data->phonebook_entry->location + drvinst->memoryoffset);
 	if (sm_message_send(strlen(req), GN_OP_ReadPhonebook, req, state))
 		return GN_ERR_NOTREADY;
 	return sm_block_no_retry(GN_OP_ReadPhonebook, data, state);
@@ -836,7 +839,7 @@ static gn_error AT_WritePhonebook(gn_data *data, struct gn_statemachine *state)
 			       data->phonebook_entry->number,
 			       data->phonebook_entry->number[0] == '+' ? "145" : "129");
 		tmp = req + ofs;
-		len = at_encode(drvinst->charset, tmp, sizeof(req - ofs),
+		len = at_encode(drvinst->charset, tmp, sizeof(req) - ofs,
 				data->phonebook_entry->name,
 				strlen(data->phonebook_entry->name));
 		tmp[len-1] = '"';
@@ -862,7 +865,7 @@ static gn_error AT_DeletePhonebook(gn_data *data, struct gn_statemachine *state)
 	if (ret)
 		return ret;
 
-	len = sprintf(req, "AT+CPBW=%d\r", data->phonebook_entry->location+drvinst->memoryoffset);
+	len = snprintf(req, sizeof(req), "AT+CPBW=%d\r", data->phonebook_entry->location+drvinst->memoryoffset);
 
 	if (sm_message_send(len, GN_OP_DeletePhonebook, req, state))
 		return GN_ERR_NOTREADY;
@@ -873,40 +876,48 @@ static gn_error AT_CallDivert(gn_data *data, struct gn_statemachine *state)
 {
 	char req[64], req2[64];
 
-	if (!data->call_divert) return GN_ERR_UNKNOWN;
+	if (!data->call_divert)
+		return GN_ERR_UNKNOWN;
 
-	sprintf(req, "AT+CCFC=");
+	snprintf(req, sizeof(req), "%s", "AT+CCFC=");
 
 	switch (data->call_divert->type) {
 	case GN_CDV_AllTypes:
-		strcat(req, "4");
+		strncat(req, "4", sizeof(req) - strlen(req));
 		break;
 	case GN_CDV_Busy:
-		strcat(req, "1");
+		strncat(req, "1", sizeof(req) - strlen(req));
 		break;
 	case GN_CDV_NoAnswer:
-		strcat(req, "2");
+		strncat(req, "2", sizeof(req) - strlen(req));
 		break;
 	case GN_CDV_OutOfReach:
-		strcat(req, "3");
+		strncat(req, "3", sizeof(req) - strlen(req));
 		break;
 	default:
 		dprintf("3. %d\n", data->call_divert->type);
 		return GN_ERR_NOTIMPLEMENTED;
 	}
 	if (data->call_divert->operation == GN_CDV_Register) {
-		sprintf(req2, ",%d,\"%s\",%d,,,%d",
-			data->call_divert->operation,
-			data->call_divert->number.number,
-			data->call_divert->number.type,
-			data->call_divert->timeout);
-		strcat(req, req2);
+		snprintf(req2, sizeof(req2), ",%d,\"%s\",%d,,,%d",
+			 data->call_divert->operation,
+			 data->call_divert->number.number,
+			 data->call_divert->number.type,
+			 data->call_divert->timeout);
+		if (strlen(req2) + strlen (req) + 1 > sizeof(req))
+			return GN_ERR_FAILED;
+		strncat(req, req2, strlen(req2));
 	} else {
-		sprintf(req2, ",%d", data->call_divert->operation);
-		strcat(req, req2);
+		snprintf(req2, sizeof(req2), ",%d", data->call_divert->operation);
+		if (strlen(req2) + strlen (req) + 1 > sizeof(req))
+			return GN_ERR_FAILED;
+		strncat(req, req2, strlen(req2));
 	}
 
-	strcat(req, "\r");
+	if (strlen(req) + 2 > sizeof(req))
+		return GN_ERR_FAILED;
+
+	strncat(req, "\r", strlen("\r"));
 
 	dprintf("%s", req);
 	if (sm_message_send(strlen(req), GN_OP_CallDivert, req, state))
@@ -1005,9 +1016,9 @@ static gn_error AT_WriteSMS(gn_data *data, struct gn_statemachine *state,
 	/* Length in AT mode is the length of the full message minus
 	 * SMSC field length */
 	if(drvinst->no_smsc) {
-		sprintf(req, "AT+%s=%d\r", cmd, length);
+		snprintf(req, sizeof(req), "AT+%s=%d\r", cmd, length);
 	} else {
-		sprintf(req, "AT+%s=%d\r", cmd, length - data->raw_sms->message_center[0] - 1);
+		snprintf(req, sizeof(req), "AT+%s=%d\r", cmd, length - data->raw_sms->message_center[0] - 1);
 	}
 	dprintf("Sending initial sequence\n");
 	if (sm_message_send(strlen(req), GN_OP_AT_Prompt, req, state))
@@ -1031,7 +1042,8 @@ static gn_error AT_WriteSMS(gn_data *data, struct gn_statemachine *state,
 
 static gn_error AT_GetSMS(gn_data *data, struct gn_statemachine *state)
 {
-	unsigned char req[16];
+	/* Sony Ericsson can return on notification a location that is 9-digits long */
+	unsigned char req[32];
 	gn_error err = AT_SetSMSMemoryType(data->raw_sms->memory_type,  state);
 
 	if (err)
@@ -1044,8 +1056,7 @@ static gn_error AT_GetSMS(gn_data *data, struct gn_statemachine *state)
 	}
 	dprintf("PDU mode set\n");
 
-	sprintf(req, "AT+CMGR=%d\r", data->raw_sms->number);
-	dprintf("%s", req);
+	snprintf(req, sizeof(req), "AT+CMGR=%d\r", data->raw_sms->number);
 	if (sm_message_send(strlen(req), GN_OP_GetSMS, req, state))
 		return GN_ERR_NOTREADY;
 	return sm_block_no_retry(GN_OP_GetSMS, data, state);
@@ -1053,12 +1064,11 @@ static gn_error AT_GetSMS(gn_data *data, struct gn_statemachine *state)
 
 static gn_error AT_DeleteSMS(gn_data *data, struct gn_statemachine *state)
 {
-	unsigned char req[16];
+	unsigned char req[32];
 	gn_error err = AT_SetSMSMemoryType(data->raw_sms->memory_type,  state);
 	if (err)
 		return err;
-	sprintf(req, "AT+CMGD=%d\r", data->sms->number);
-	dprintf("%s", req);
+	snprintf(req, sizeof(req), "AT+CMGD=%d\r", data->sms->number);
 
 	if (sm_message_send(strlen(req), GN_OP_DeleteSMS, req, state))
  		return GN_ERR_NOTREADY;
@@ -1094,7 +1104,7 @@ static gn_error AT_EnterSecurityCode(gn_data *data, struct gn_statemachine *stat
 	if (data->security_code->type != GN_SCT_Pin)
 		return GN_ERR_NOTIMPLEMENTED;
 
-	sprintf(req, "AT+CPIN=\"%s\"\r", data->security_code->code);
+	snprintf(req, sizeof(req), "AT+CPIN=\"%s\"\r", data->security_code->code);
  	if (sm_message_send(strlen(req), GN_OP_EnterSecurityCode, req, state))
 		return GN_ERR_NOTREADY;
 	return sm_block_no_retry(GN_OP_EnterSecurityCode, data, state);
@@ -1184,15 +1194,30 @@ static gn_error AT_GetNetworkInfo(gn_data *data, struct gn_statemachine *state)
 
 static gn_error AT_SetDateTime(gn_data *data, struct gn_statemachine *state)
 {
-	char req[29];
-
+	at_driver_instance *drvinst = AT_DRVINST(state);
+	char req[64];
+	gn_timestamp aux;
 	gn_timestamp *dt = data->datetime;
 
-	sprintf(req, "AT+CCLK=\"%02d/%02d/%02d,%02d:%02d:%02d\"\r",
-		dt->year % 100, dt->month, dt->day,
-		dt->hour, dt->minute, dt->second);
+	memset(&aux, 0, sizeof(gn_timestamp));
+	/* Make sure GetDateTime doesn't overwrite dt */
+	data->datetime = &aux;
+	/* ignore errors */
+	AT_GetDateTime(data, state);
+	AT_PrepareDateTime(data, state);
 
-	if (sm_message_send(28, GN_OP_SetDateTime, req, state))
+	data->datetime = dt;
+	memset(req, 0, 64);
+	if (drvinst->timezone)
+		snprintf(req, sizeof(req), "AT+CCLK=\"%02d/%02d/%02d,%02d:%02d:%02d%s\"\r",
+			 dt->year % 100, dt->month, dt->day,
+			 dt->hour, dt->minute, dt->second, drvinst->timezone);
+	else
+		snprintf(req, sizeof(req), "AT+CCLK=\"%02d/%02d/%02d,%02d:%02d:%02d\"\r",
+			 dt->year % 100, dt->month, dt->day,
+			 dt->hour, dt->minute, dt->second);
+
+	if (sm_message_send(strlen(req), GN_OP_SetDateTime, req, state))
 		return GN_ERR_NOTREADY;
 	return sm_block_no_retry(GN_OP_SetDateTime, data, state);
 }
@@ -1204,6 +1229,13 @@ static gn_error AT_GetDateTime(gn_data *data, struct gn_statemachine *state)
 	return sm_block_no_retry(GN_OP_GetDateTime, data, state);
 }
 
+static gn_error AT_PrepareDateTime(gn_data *data, struct gn_statemachine *state)
+{
+	if (sm_message_send(10, GN_OP_AT_PrepareDateTime, "AT+CCLK=?\r", state))
+		return GN_ERR_NOTREADY;
+	return sm_block_no_retry(GN_OP_AT_PrepareDateTime, data, state);
+}
+
 
 /*
  * This command allows to send DTMF tones and arbitrary tones.
@@ -1252,17 +1284,19 @@ static gn_error AT_GetActiveCalls(gn_data *data, struct gn_statemachine *state)
 
 static gn_error AT_OnSMS(gn_data *data, struct gn_statemachine *state)
 {
+	at_driver_instance *drvinst = AT_DRVINST(state);
 	gn_error error;
-	int i = 2;
+	int mode;
 	char req[13];
 
+	mode = drvinst->cnmi_mode;
 	do {
-		snprintf(req, 13, "AT+CNMI=%d,1\r", i);
-		if (sm_message_send(12, GN_OP_OnSMS, req, state))
+		snprintf(req, sizeof(req), "AT+CNMI=%d,1\r", mode);
+		if (sm_message_send(strlen(req), GN_OP_OnSMS, req, state))
 			return GN_ERR_NOTREADY;
 
 		error = sm_block_no_retry(GN_OP_OnSMS, data, state);
-	} while (i-- && error);
+	} while (mode-- && error);
 	if (error == GN_ERR_NONE) {
 		AT_DRVINST(state)->on_sms = data->on_sms;
 		AT_DRVINST(state)->sms_callback_data = data->callback_data;
@@ -1320,7 +1354,7 @@ static gn_error ReplyReadPhonebook(int messagetype, unsigned char *buffer, int l
 			endpos = strchr(++pos, '\"');
 		if (endpos) {
 			*endpos = '\0';
-			strcpy(data->phonebook_entry->number, pos);
+			snprintf(data->phonebook_entry->number, sizeof(data->phonebook_entry->number), "%s", pos);
 		}
 
 		/* store name */
@@ -1355,8 +1389,7 @@ static gn_error ReplyGetSMSCenter(int messagetype, unsigned char *buffer, int le
 		if (pos) {
 			*pos++ = '\0';
 			data->message_center->id = 1;
-			strncpy(data->message_center->smsc.number, buf.line2 + 8, GN_BCD_STRING_MAX_LENGTH);
-			data->message_center->smsc.number[GN_BCD_STRING_MAX_LENGTH - 1] = '\0';
+			snprintf(data->message_center->smsc.number, GN_BCD_STRING_MAX_LENGTH, "%s", buf.line2 + 8);
 			/* Now we look for the number type */
 			data->message_center->smsc.type = 0;
 			aux = strchr(pos, ',');
@@ -1368,13 +1401,13 @@ static gn_error ReplyGetSMSCenter(int messagetype, unsigned char *buffer, int le
 				data->message_center->smsc.type = GN_GSM_NUMBER_Unknown;
 		} else {
 			data->message_center->id = 0;
-			strncpy(data->message_center->name, "SMS Center", GN_SMS_CENTER_NAME_MAX_LENGTH);
+			snprintf(data->message_center->name, GN_SMS_CENTER_NAME_MAX_LENGTH, "SMS Center");
 			data->message_center->smsc.type = GN_GSM_NUMBER_Unknown;
 		}
 		data->message_center->default_name = 1; /* use default name */
 		data->message_center->format = GN_SMS_MF_Text; /* whatever */
 		data->message_center->validity = GN_SMS_VP_Max;
-		strcpy(data->message_center->recipient.number, "") ;
+		data->message_center->recipient.number[0] = 0;
 	}
 	return GN_ERR_NONE;
 }
@@ -1413,6 +1446,13 @@ static gn_error ReplyMemoryStatus(int messagetype, unsigned char *buffer, int le
 	return GN_ERR_NONE;
 }
 
+/*
+ * Response of of a form:
+ * +CPBR: (list of supported indexes), <nlength>, <tlength>
+ * 	list of supported indexes is usually in 1, 2500 form
+ * 	nlength is the maximal length of the number
+ * 	tlength is the maximal length of the name
+ */
 static gn_error ReplyMemoryRange(int messagetype, unsigned char *buffer, int length, gn_data *data, struct gn_statemachine *state)
 {
 	at_driver_instance *drvinst = AT_DRVINST(state);
@@ -1423,7 +1463,8 @@ static gn_error ReplyMemoryRange(int messagetype, unsigned char *buffer, int len
 	drvinst->memoryoffset = 0;
 	drvinst->memorysize = 100;
 
-	if ((error = at_error_get(buffer, state)) != GN_ERR_NONE) return error;
+	if ((error = at_error_get(buffer, state)) != GN_ERR_NONE)
+		return error;
 
 	buf.line1 = buffer + 1;
 	buf.length= length;
@@ -1607,7 +1648,7 @@ static gn_error ReplyGetSMS(int messagetype, unsigned char *buffer, int length,
 {
 	at_line_buffer buf;
 	gn_error ret = GN_ERR_NONE;
-	unsigned int sms_len, l, offset = 0;
+	unsigned int sms_len, l, extraoffset, offset = 0;
 	unsigned char *tmp;
 	gn_error error;
  	at_driver_instance *drvinst = AT_DRVINST(state);
@@ -1714,10 +1755,12 @@ static gn_error ReplyGetSMS(int messagetype, unsigned char *buffer, int length,
 		data->raw_sms->more_messages       = tmp[offset] & 0x04;
 		// bits 3 and 4 of the first octet unused;
 		data->raw_sms->report_status       = tmp[offset] & 0x20;
+		extraoffset = 10;
 	} else if (data->raw_sms->type == GN_SMS_MT_Submit) {
 		dprintf("SMS-SUBMIT found\n");
 		data->raw_sms->reject_duplicates   = tmp[offset] & 0x04;
 		data->raw_sms->validity_indicator  = (tmp[offset] & 0x18) >> 3;
+		extraoffset = 3;
 	} else {
 		dprintf("Unknown PDU type %d\n", data->raw_sms->type);
 		return GN_ERR_INTERNALERROR;
@@ -1734,7 +1777,7 @@ static gn_error ReplyGetSMS(int messagetype, unsigned char *buffer, int length,
 
 	l = (tmp[offset] % 2) ? tmp[offset] + 1 : tmp[offset];
 	l = l / 2 + 2;
-	if (l + offset + 10 > sms_len || l > GN_SMS_NUMBER_MAX_LENGTH) {
+	if (l + offset + extraoffset > sms_len || l > GN_SMS_NUMBER_MAX_LENGTH) {
 		dprintf("Invalid remote number length (%d)\n", l);
 		ret = GN_ERR_INTERNALERROR;
 		goto out;
@@ -2099,8 +2142,10 @@ static gn_error ReplyIncomingSMS(int messagetype, unsigned char *buffer, int len
 		drvinst->on_sms(data->sms, state, drvinst->sms_callback_data);
 	}
 
-	if (freesms)
+	if (freesms) {
 		free(data->sms);
+		data->sms = NULL;
+	}
 
 	return error;
 }
@@ -2209,9 +2254,12 @@ static gn_error ReplyGetNetworkInfo(int messagetype, unsigned char *buffer, int
 
 static gn_error ReplyGetDateTime(int messagetype, unsigned char *buffer, int length, gn_data *data, struct gn_statemachine *state)
 {
+	at_driver_instance *drvinst = AT_DRVINST(state);
+	int cnt;
 	at_line_buffer buf;
 	gn_error error;
 	gn_timestamp *dt;
+	char timezone[6];
 
 	if ((error = at_error_get(buffer, state)) != GN_ERR_NONE)
 		return error;
@@ -2222,10 +2270,20 @@ static gn_error ReplyGetDateTime(int messagetype, unsigned char *buffer, int len
 	splitlines(&buf);
 
 	dt = data->datetime;
-	if (sscanf(buf.line2, "+CCLK: \"%d/%d/%d,%d:%d:%d\"",
-		   &dt->year, &dt->month, &dt->day,
-		   &dt->hour, &dt->minute, &dt->second) != 6)
+	memset(timezone, 0, 6);
+	cnt = sscanf(buf.line2, "+CCLK: \"%d/%d/%d,%d:%d:%d%[+-1234567890]\"",
+		     &dt->year, &dt->month, &dt->day,
+		     &dt->hour, &dt->minute, &dt->second, timezone);
+	switch (cnt) {
+	case 7:
+		drvinst->timezone = realloc(drvinst->timezone, strlen(timezone) + 1);
+		strcpy(drvinst->timezone, timezone);
+		break;
+	case 6:
+		break;
+	default:
 		return GN_ERR_FAILED;
+	}
 
 	if (dt->year < 100)
 		dt->year += 2000;
@@ -2332,6 +2390,8 @@ static gn_error Reply(int messagetype, unsigned char *buffer, int length, gn_dat
 	return at_error_get(buffer, state);
 }
 
+#define at_manufacturer_compare(pattern)	strncasecmp(manufacturer, pattern, strlen(pattern))
+
 static gn_error Initialise(gn_data *setupdata, struct gn_statemachine *state)
 {
 	at_driver_instance *drvinst;
@@ -2375,6 +2435,9 @@ static gn_error Initialise(gn_data *setupdata, struct gn_statemachine *state)
 	drvinst->prev_state = GN_CALL_Idle;
 	drvinst->mememorysize = -1;
 	drvinst->smmemorysize = -1;
+	drvinst->timezone = NULL;
+	/* FIXME: detect if from AT+CNMI=? */
+	drvinst->cnmi_mode = 3;
 
 	drvinst->if_pos = 0;
 	for (i = 0; i < GN_OP_AT_Max; i++) {
@@ -2423,18 +2486,20 @@ static gn_error Initialise(gn_data *setupdata, struct gn_statemachine *state)
 	if (ret)
 		goto out;
 
-	if (!strncasecmp(manufacturer, "bosch", 5))
+	if (!at_manufacturer_compare("bosch"))
 		at_bosch_init(model, setupdata->model, state);
-	else if (!strncasecmp(manufacturer, "ericsson", 8))
+	else if (!at_manufacturer_compare("ericsson"))
 		at_ericsson_init(model, setupdata->model, state);
-	else if (!strncasecmp(manufacturer, "nokia", 5))
+	else if (!at_manufacturer_compare("nokia"))
 		at_nokia_init(model, setupdata->model, state);
-	else if (!strncasecmp(manufacturer, "siemens", 7))
+	else if (!at_manufacturer_compare("siemens"))
 		at_siemens_init(model, setupdata->model, state);
-	else if (!strncasecmp(manufacturer, "sony ericsson", 14))
+	else if (!at_manufacturer_compare("sony ericsson"))
 		at_sonyericsson_init(model, setupdata->model, state);
-	else if (!strncasecmp(manufacturer, "samsung", 7))
+	else if (!at_manufacturer_compare("samsung"))
 		at_samsung_init(model, setupdata->model, state);
+	else if (!at_manufacturer_compare("motorola"))
+		at_motorola_init(model, setupdata->model, state);
 	
 	StoreDefaultCharset(state);
 
@@ -2451,6 +2516,10 @@ out:
 static gn_error Terminate(gn_data *data, struct gn_statemachine *state)
 {
 	if (AT_DRVINST(state)) {
+		if (AT_DRVINST(state)->timezone) {
+			free(AT_DRVINST(state)->timezone);
+			AT_DRVINST(state)->timezone = NULL;
+		}
 		free(AT_DRVINST(state));
 		AT_DRVINST(state) = NULL;
 	}
