@@ -1,6 +1,6 @@
 /*
 
-  $Id: atsoer.c,v 1.6 2007/10/05 20:42:58 pkot Exp $
+  $Id: atsoer.c,v 1.10 2007/11/19 13:43:57 pkot Exp $
 
   G N O K I I
 
@@ -56,14 +56,16 @@ static gn_error se_at_memory_type_set(gn_memory_type mt, struct gn_statemachine
 
 		len = at_encode(drvinst->charset, memtype, sizeof(memtype),
 				memorynames[mt], strlen(memorynames[mt]));
-		sprintf(req, "AT+CPBS=\"%s\"\r", memtype);
+		snprintf(req, sizeof(req), "AT+CPBS=\"%s\"\r", memtype);
 		ret = sm_message_send(11 + len - 1, GN_OP_Init, req, state);
 		if (ret)
 			return GN_ERR_NOTREADY;
 		gn_data_clear(&data);
 		ret = sm_block_no_retry(GN_OP_Init, &data, state);
-		if (ret == GN_ERR_NONE)
-			drvinst->memorytype = mt;
+		if (ret)
+			return ret;
+
+		drvinst->memorytype = mt;
 
 		gn_data_clear(&data);
 		ret = state->driver.functions(GN_OP_AT_GetMemoryRange, &data, state);
@@ -129,6 +131,33 @@ static gn_error ReplyReadPhonebook(int messagetype, unsigned char *buffer, int l
 	return GN_ERR_NONE;
 }
 
+static gn_error ReplyMemoryStatus(int messagetype, unsigned char *buffer, int length, gn_data *data, struct gn_statemachine *state)
+{
+	at_driver_instance *drvinst = AT_DRVINST(state);
+	gn_error error;
+	char *buf = buffer;
+	int counter = 0;
+
+	if (!data->memory_status)
+		return GN_ERR_INTERNALERROR;
+
+	if ((error = at_error_get(buffer, state)) != GN_ERR_NONE)
+		return error;
+
+	while ((buf = strchr(buf, '\r'))) {
+		buf++;
+		if (strlen(buf) > 6 &&
+			(!strncmp(buf, "+CPBR:", 6) || !strncmp(buf + 1, "+CPBR:", 6)))
+			counter++;
+	}
+	data->memory_status->used = counter;
+	data->memory_status->free = drvinst->memorysize - counter;
+
+	dprintf("Got %d entries\n", counter);
+	return error;
+}
+
+
 static gn_error AT_ReadPhonebook(gn_data *data, struct gn_statemachine *state)
 {
 	at_driver_instance *drvinst = AT_DRVINST(state);
@@ -141,12 +170,32 @@ static gn_error AT_ReadPhonebook(gn_data *data, struct gn_statemachine *state)
 	ret = se_at_memory_type_set(data->phonebook_entry->memory_type, state);
 	if (ret)
 		return ret;
-	sprintf(req, "AT+CPBR=%d\r", data->phonebook_entry->location+drvinst->memoryoffset);
+	snprintf(req, sizeof(req), "AT+CPBR=%d\r", data->phonebook_entry->location+drvinst->memoryoffset);
 	if (sm_message_send(strlen(req), GN_OP_ReadPhonebook, req, state))
 		return GN_ERR_NOTREADY;
 	return sm_block_no_retry(GN_OP_ReadPhonebook, data, state);
 }
 
+static gn_error AT_GetMemoryStatus(gn_data *data, struct gn_statemachine *state)
+{
+	at_driver_instance *drvinst = AT_DRVINST(state);
+	gn_error ret = GN_ERR_NONE;
+	char req[32];
+
+	ret = se_at_memory_type_set(data->memory_status->memory_type,  state);
+	if (ret)
+		return ret;
+	ret = state->driver.functions(GN_OP_AT_GetMemoryRange, data, state);
+	if (ret)
+		return ret;
+	memset(req, 0, 32);
+	snprintf(req, 31, "AT+CPBR=%d,%d\r", drvinst->memoryoffset + 1, drvinst->memorysize + drvinst->memoryoffset);
+	ret = sm_message_send(strlen(req), GN_OP_GetMemoryStatus, req, state);
+	if (ret)
+		return GN_ERR_NOTREADY;
+	return sm_block_no_retry(GN_OP_GetMemoryStatus, data, state);
+}
+
 static gn_error AT_WritePhonebook(gn_data *data, struct gn_statemachine *state)
 {
 	at_driver_instance *drvinst = AT_DRVINST(state);
@@ -195,7 +244,7 @@ static gn_error AT_DeletePhonebook(gn_data *data, struct gn_statemachine *state)
 	if (ret)
 		return ret;
 
-	len = sprintf(req, "AT+CPBW=%d\r", data->phonebook_entry->location+drvinst->memoryoffset);
+	len = snprintf(req, sizeof(req), "AT+CPBW=%d\r", data->phonebook_entry->location+drvinst->memoryoffset);
 
 	if (sm_message_send(len, GN_OP_DeletePhonebook, req, state))
 		return GN_ERR_NOTREADY;
@@ -215,11 +264,17 @@ static gn_error AT_GetNetworkInfo(gn_data *data, struct gn_statemachine *state)
 	return sm_block_no_retry(GN_OP_GetNetworkInfo, data, state);
 }
 
+
 void at_sonyericsson_init(char* foundmodel, char* setupmodel, struct gn_statemachine *state)
 {
+	/* Sony Ericssons support just mode 2 */
+	AT_DRVINST(state)->cnmi_mode = 2;
+
 	at_insert_send_function(GN_OP_ReadPhonebook, AT_ReadPhonebook, state);
 	at_insert_recv_function(GN_OP_ReadPhonebook, ReplyReadPhonebook, state);
 	at_insert_send_function(GN_OP_WritePhonebook, AT_WritePhonebook, state);
 	at_insert_send_function(GN_OP_DeletePhonebook, AT_DeletePhonebook, state);
 	at_insert_send_function(GN_OP_GetNetworkInfo, AT_GetNetworkInfo, state);
+	at_insert_send_function(GN_OP_GetMemoryStatus, AT_GetMemoryStatus, state);
+	at_insert_recv_function(GN_OP_GetMemoryStatus, ReplyMemoryStatus, state);
 }
