@@ -1,6 +1,6 @@
 /*
 
-  $Id: nokia.c,v 1.42 2006/05/08 17:44:07 deller Exp $
+  $Id: nokia.c,v 1.44 2007/11/07 18:28:19 pkot Exp $
 
   G N O K I I
 
@@ -135,7 +135,7 @@ size_t pnok_string_encode(unsigned char *dest, size_t max, const unsigned char *
 
 	MBSTATE_ENC_CLEAR(mbs);
 	for (i = 0, j = 0; i < max && src[j]; i++, j += n) {
-		n = char_uni_alphabet_encode(src + j, &wch, &mbs);
+		n = char_uni_alphabet_encode(src + j, max - j, &wch, &mbs);
 		dest[i] = pnok_uni_to_nokia(wch);
 	}
 	return i;
@@ -373,7 +373,7 @@ gn_error pnok_call_make(gn_data *data, struct gn_statemachine *state)
 	if ((err = pnok_extended_cmds_enable(0x01, data, state)))
 		return err;
 
-	strcpy(req + 4, data->call_info->number);
+	snprintf(req + 4, GN_PHONEBOOK_NUMBER_MAX_LENGTH + 1, "%s", data->call_info->number);
 
 	if (sm_message_send(5 + n, 0x40, req, state)) return GN_ERR_NOTREADY;
 	return sm_block(0x40, data, state);
