@@ -1,6 +1,6 @@
 /*
 
-  $Id: gnokii-phonebook.c,v 1.17 2007/08/23 12:07:59 pkot Exp $
+  $Id: gnokii-phonebook.c,v 1.21 2007/11/08 21:20:13 dforsi Exp $
 
   G N O K I I
 
@@ -88,7 +88,7 @@ int getphonebook(int argc, char *argv[], gn_data *data, struct gn_statemachine *
 {
 	gn_phonebook_entry entry;
 	gn_memory_status memstat;
-	int i, count, start_entry, end_entry, num_entries = INT_MAX;
+	int i, count, start_entry, end_entry, num_entries = INT_MAX, explicit_end = true;
 	gn_error error = GN_ERR_NONE;
 	char *memory_type_string;
 	char location[32];
@@ -143,9 +143,13 @@ int getphonebook(int argc, char *argv[], gn_data *data, struct gn_statemachine *
 	}
 
 	if (end_entry == INT_MAX) {
+		explicit_end = false;
 		data->memory_status = &memstat;
 		if ((error = gn_sm_functions(GN_OP_GetMemoryStatus, data, state)) == GN_ERR_NONE) {
-			num_entries = memstat.used - start_entry + 1;
+			num_entries = memstat.used;
+			end_entry = memstat.used + memstat.free;
+			/* FIXME warn the user like parse_end_value_option() when end_entry < start_entry */
+			if (end_entry < start_entry) end_entry = start_entry;
 		}
 	}
 
@@ -162,15 +166,15 @@ int getphonebook(int argc, char *argv[], gn_data *data, struct gn_statemachine *
 		switch (error) {
 			int i;
 		case GN_ERR_NONE:
-			num_entries--;
 			if (entry.empty != false)
 				break;
+			num_entries--;
 			switch (type) {
 			case 1:
 				gn_file_phonebook_raw_write(stdout, &entry, memory_type_string);
 				break;
 			case 2:
-				sprintf(location, "%s%d", memory_type_string, entry.location);
+				snprintf(location, sizeof(location), "%s%d", memory_type_string, entry.location);
 				gn_phonebook2vcard(stdout, &entry, location);
 				break;
 			case 3:
@@ -265,26 +269,19 @@ int getphonebook(int argc, char *argv[], gn_data *data, struct gn_statemachine *
 		case GN_ERR_EMPTYLOCATION:
 			fprintf(stderr, _("Empty memory location. Skipping.\n"));
 			break;
-		case GN_ERR_INVALIDLOCATION:
-			if (end_entry == INT_MAX) {
-				/* Ensure that we quit the loop */
-				num_entries = 0;
-			} else {
-				/* Only print an error if we got a valid end index */
-				fprintf(stderr, _("Error reading from the location %d in memory %s\n"), count, memory_type_string);
-				fprintf(stderr, _("Error: %s\n"), gn_error_print(error));
-			}
-			break;
-		case GN_ERR_TIMEOUT:
-			/* On timeout just exit the loop */
-			num_entries = 0;
 		default:
+			fprintf(stderr, _("Error reading from the location %d in memory %s\n"), count, memory_type_string);
 			fprintf(stderr, _("Error: %s\n"), gn_error_print(error));
-			break;
+			/* some older phones might return GN_ERR_INVALIDLOCATION for lower numbered locations
+			so make it non fatal when a numeric end location was given */
+			if (!((error == GN_ERR_INVALIDLOCATION) && explicit_end)) {
+				return error;
+			}
 		}
 		count++;
 	}
-	return error;
+	/* ignore non fatal errors that might have occurred above */
+	return GN_ERR_NONE;
 }
 
 /* Displays usage of --getphonebook command */
@@ -464,7 +461,7 @@ int writephonebook(int argc, char *argv[], gn_data *data, struct gn_statemachine
 		error = gn_sm_functions(GN_OP_WritePhonebook, data, state);
 
 		if (error == GN_ERR_NONE) {
-			fprintf (stderr, 
+			fprintf(stderr, 
 				 _("Write Succeeded: memory type: %s, loc: %d, name: %s, number: %s\n"), 
 				 gn_memory_type2str(entry.memory_type), entry.location, entry.name, entry.number);
 			/* If the location was not specified and there are
@@ -472,7 +469,7 @@ int writephonebook(int argc, char *argv[], gn_data *data, struct gn_statemachine
 			 * location */
 			default_location++;
 		} else
-			fprintf (stderr, _("Write FAILED (%s): memory type: %s, loc: %d, name: %s, number: %s\n"), 
+			fprintf(stderr, _("Write FAILED (%s): memory type: %s, loc: %d, name: %s, number: %s\n"), 
 				 gn_error_print(error), gn_memory_type2str(entry.memory_type), entry.location, entry.name, entry.number);
 	}
 out:
