@@ -1,6 +1,6 @@
 /*
 
-  $Id: gnokii-sms.c,v 1.4 2006/08/11 10:49:46 dforsi Exp $
+  $Id: gnokii-sms.c,v 1.36 2007/11/07 18:28:20 pkot Exp $
 
   G N O K I I
 
@@ -41,13 +41,20 @@
 #include "misc.h"
 #include "compat.h"
 
+#if defined(WIN32) && !defined(CYGWIN)
+#  include <process.h>
+#  define getpid _getpid
+#endif
+
 #include <stdio.h>
 #include <sys/stat.h>
-#include <unistd.h>
-#define _GNU_SOURCE
+#ifndef _GNU_SOURCE
+#  define _GNU_SOURCE 1
+#endif
 #include <getopt.h>
 #include <time.h>
 #include <signal.h>
+#include <errno.h>
 
 #include "gnokii-app.h"
 #include "gnokii.h"
@@ -57,15 +64,17 @@ void sms_usage(FILE *f)
 {
 	fprintf(f, _("SMS options:\n"
 		     "          --sendsms destination [--smsc message_center_number |\n"
-		     "                 --smscno message_center_index] [-r|--remote] [-8|8bit]\n"
+		     "                 --smscno message_center_index] [-r|--report] [-8|--8bit]\n"
 		     "                 [-C|--class n] [-v|--validity n] [-l|--long n]\n"
 		     "                 [-i|--imelody] [-a|--animation file;file;file;file]\n"
 		     "                 [-o|--concat this;total;serial] [-w|--wappush url]\n"
 		     "          --savesms [--sender from] [--smsc message_center_number |\n"
 		     "                 --smscno message_center_index] [-f|--folder folder_id]\n"
 		     "                 [-l|--location number] [-s|--sent|-r|--read] [-d|--deliver]\n"
-		     "                 [-t|--datetime YYMMDDHHMMSS] \n"
-		     "          --getsms memory_type start [end] [-f file] [-F file] [-d]\n"
+		     "                 [-t|--datetime YYMMDDHHMMSS]\n"
+		     "          --getsms memory_type start [end] [-f|--file file]\n"
+		     "                 [-F|--force-file file] [-a|--append-file file]\n"
+		     "                 [-d|--delete]\n"
 		     "          --deletesms memory_type start [end]\n"
 		     "          --getsmsc [start_number [end_number]] [-r|--raw]\n"
 		     "          --setsmsc\n"
@@ -141,7 +150,7 @@ int sendsms(int argc, char *argv[], gn_data *data, struct gn_statemachine *state
 	gn_sms_default_submit(&sms);
 
 	memset(&sms.remote.number, 0, sizeof(sms.remote.number));
-	strncpy(sms.remote.number, optarg, sizeof(sms.remote.number) - 1);
+	snprintf(sms.remote.number, sizeof(sms.remote.number), "%s", optarg);
 	if (sms.remote.number[0] == '+') 
 		sms.remote.type = GN_GSM_NUMBER_International;
 	else 
@@ -150,7 +159,7 @@ int sendsms(int argc, char *argv[], gn_data *data, struct gn_statemachine *state
 	while ((i = getopt_long(argc, argv, "l:rv:C:8ia:o:w:", options, NULL)) != -1) {
 		switch (i) {       /* -c for compression. not yet implemented. */
 		case '1': /* SMSC number */
-			strncpy(sms.smsc.number, optarg, sizeof(sms.smsc.number) - 1);
+			snprintf(sms.smsc.number, sizeof(sms.smsc.number), "%s", optarg);
 			if (sms.smsc.number[0] == '+') 
 				sms.smsc.type = GN_GSM_NUMBER_International;
 			else
@@ -159,20 +168,22 @@ int sendsms(int argc, char *argv[], gn_data *data, struct gn_statemachine *state
 
 		case '2': /* SMSC number index in phone memory */
 			data->message_center = calloc(1, sizeof(gn_sms_message_center));
-			data->message_center->id = atoi(optarg);
-			if (data->message_center->id < 1 || data->message_center->id > 5) {
+			data->message_center->id = gnokii_atoi(optarg);
+			if (errno || data->message_center->id < 1 || data->message_center->id > 5) {
 				free(data->message_center);
 				sendsms_usage(stderr, -1);
 			}
 			if (gn_sm_functions(GN_OP_GetSMSCenter, data, state) == GN_ERR_NONE) {
-				strcpy(sms.smsc.number, data->message_center->smsc.number);
+				snprintf(sms.smsc.number, sizeof(sms.smsc.number), "%s", data->message_center->smsc.number);
 				sms.smsc.type = data->message_center->smsc.type;
 			}
 			free(data->message_center);
 			break;
 
 		case 'l': /* we send long message */
-			input_len = atoi(optarg);
+			input_len = gnokii_atoi(optarg);
+			if (errno || input_len < 0)
+				sendsms_usage(stderr, -1);
 			if (input_len > 255 * GN_SMS_MAX_LENGTH) {
 				fprintf(stderr, _("Input too long! (%d, maximum is %d)\n"), input_len, 255 * GN_SMS_MAX_LENGTH);
 				return -1;
@@ -182,7 +193,7 @@ int sendsms(int argc, char *argv[], gn_data *data, struct gn_statemachine *state
 		case 'a': /* Animation */ {
 			char buf[10240];
 			char *s = buf, *t;
-			strcpy(buf, optarg);
+			snprintf(buf, sizeof(buf), "%s", optarg);
 			sms.user_data[curpos].type = GN_SMS_DATA_Animation;
 			for (i = 0; i < 4; i++) {
 				t = strchr(s, ';');
@@ -215,16 +226,27 @@ int sendsms(int argc, char *argv[], gn_data *data, struct gn_statemachine *state
 
 		case 'C': /* class Message */
 			switch (*optarg) {
-			case '0': sms.dcs.u.general.m_class = 1; break;
-			case '1': sms.dcs.u.general.m_class = 2; break;
-			case '2': sms.dcs.u.general.m_class = 3; break;
-			case '3': sms.dcs.u.general.m_class = 4; break;
-			default: sendsms_usage(stderr, -1);
+			case '0':
+				sms.dcs.u.general.m_class = 1;
+				break;
+			case '1':
+				sms.dcs.u.general.m_class = 2;
+				break;
+			case '2':
+				sms.dcs.u.general.m_class = 3;
+				break;
+			case '3':
+				sms.dcs.u.general.m_class = 4;
+				break;
+			default:
+				sendsms_usage(stderr, -1);
 			}
 			break;
 
 		case 'v':
-			sms.validity = atoi(optarg);
+			sms.validity = gnokii_atoi(optarg);
+			if (errno || sms.validity < 0)
+				sendsms_usage(stderr, -1);
 			break;
 
 		case '8':
@@ -286,12 +308,16 @@ int sendsms(int argc, char *argv[], gn_data *data, struct gn_statemachine *state
 			sendsms_usage(stderr, -1);
 		}
 	}
+	if (argc > optind) {
+		/* There are too many arguments that don't start with '-' */
+		sendsms_usage(stderr, -1);
+	}
 
 	if (!sms.smsc.number[0]) {
 		data->message_center = calloc(1, sizeof(gn_sms_message_center));
 		data->message_center->id = 1;
 		if (gn_sm_functions(GN_OP_GetSMSCenter, data, state) == GN_ERR_NONE) {
-			strcpy(sms.smsc.number, data->message_center->smsc.number);
+			snprintf(sms.smsc.number, sizeof(sms.smsc.number), "%s", data->message_center->smsc.number);
 			sms.smsc.type = data->message_center->smsc.type;
 		} else {
 			fprintf(stderr, _("Cannot read the SMSC number from your phone. If the sms send will fail, please use --smsc option explicitely giving the number.\n"));
@@ -360,10 +386,7 @@ int savesms(int argc, char *argv[], gn_data *data, struct gn_statemachine *state
 {
 	gn_sms sms;
 	gn_error error = GN_ERR_INTERNALERROR;
-	/* The maximum length of an uncompressed concatenated short message is
-	   255 * 153 = 39015 default alphabet characters */
-	char message_buffer[255 * GN_SMS_MAX_LENGTH];
-	int input_len, chars_read, i;
+	int input_len, i;
 	char tmp[3];
 #if 0
 	int confirm = -1;
@@ -383,7 +406,7 @@ int savesms(int argc, char *argv[], gn_data *data, struct gn_statemachine *state
 		{ NULL,       0,                 NULL, 0 }
 	};
 
-	char memory_type[20];
+	char memory_type[20] = "";
 	struct tm		*now;
 	time_t			nowh;
 
@@ -405,7 +428,7 @@ int savesms(int argc, char *argv[], gn_data *data, struct gn_statemachine *state
 	/* nokia 7110 will choke if no number is present when we
 	 * try to store a SMS on the phone. maybe others do too
 	 * TODO should this be handled here? report an error instead
-	 * of setting an default? */
+	 * of setting a default? */
 	strcpy(sms.remote.number, "0");
 	sms.remote.type = GN_GSM_NUMBER_International;
 	sms.number = 0;
@@ -423,13 +446,13 @@ int savesms(int argc, char *argv[], gn_data *data, struct gn_statemachine *state
 			break;
 		case '1': /* SMSC number index in phone memory */
 			data->message_center = calloc(1, sizeof(gn_sms_message_center));
-			data->message_center->id = atoi(optarg);
-			if (data->message_center->id < 1 || data->message_center->id > 5) {
+			data->message_center->id = gnokii_atoi(optarg);
+			if (errno || data->message_center->id < 1 || data->message_center->id > 5) {
 				free(data->message_center);
-				sendsms_usage(stderr, -1);
+				savesms_usage(stderr, -1);
 			}
 			if (gn_sm_functions(GN_OP_GetSMSCenter, data, state) == GN_ERR_NONE) {
-				strcpy(sms.smsc.number, data->message_center->smsc.number);
+				snprintf(sms.smsc.number, sizeof(sms.smsc.number), "%s", data->message_center->smsc.number);
 				sms.smsc.type = data->message_center->smsc.type;
 			}
 			free(data->message_center);
@@ -442,7 +465,9 @@ int savesms(int argc, char *argv[], gn_data *data, struct gn_statemachine *state
 			snprintf(sms.remote.number, GN_BCD_STRING_MAX_LENGTH, "%s", optarg);
 			break;
 		case '3': /* location to write to */
-			sms.number = atoi(optarg);
+			sms.number = gnokii_atoi(optarg);
+			if (errno || sms.number < 0)
+				savesms_usage(stderr, -1);
 			break;
 		case 's': /* mark the message as sent */
 		case 'r': /* mark the message as read */
@@ -464,7 +489,6 @@ int savesms(int argc, char *argv[], gn_data *data, struct gn_statemachine *state
 			sms.type = GN_SMS_MT_Deliver;
 			break;
 		case 't': /* set specific date and time of message delivery */
-			tmp[2] = 0;
 			if (strlen(optarg) != 12) {
 				fprintf(stderr, _("Invalid datetime format: %s (should be YYMMDDHHMISS, all digits)!\n"), optarg);
 				return -1;
@@ -474,17 +498,17 @@ int savesms(int argc, char *argv[], gn_data *data, struct gn_statemachine *state
 					fprintf(stderr, _("Invalid datetime format: %s (should be YYMMDDHHMISS, all digits)!\n"), optarg);
 					return -1;
 				}
-			strncpy(tmp, optarg, 2);
+			snprintf(tmp, sizeof(tmp), "%s", optarg);
 			sms.smsc_time.year	= atoi(tmp) + 1900;
-			strncpy(tmp, optarg+2, 2);
+			snprintf(tmp, sizeof(tmp), "%s", optarg+2);
 			sms.smsc_time.month	= atoi(tmp);
-			strncpy(tmp, optarg+4, 2);
+			snprintf(tmp, sizeof(tmp), "%s", optarg+4);
 			sms.smsc_time.day	= atoi(tmp);
-			strncpy(tmp, optarg+6, 2);
+			snprintf(tmp, sizeof(tmp), "%s", optarg+6);
 			sms.smsc_time.hour	= atoi(tmp);
-			strncpy(tmp, optarg+8, 2);
+			snprintf(tmp, sizeof(tmp), "%s", optarg+8);
 			sms.smsc_time.minute	= atoi(tmp);
-			strncpy(tmp, optarg+10, 2);
+			snprintf(tmp, sizeof(tmp), "%s", optarg+10);
 			sms.smsc_time.second	= atoi(tmp);
 			if (!gn_timestamp_isvalid(sms.smsc_time)) {
 				fprintf(stderr, _("Invalid datetime: %s.\n"), optarg);
@@ -495,6 +519,10 @@ int savesms(int argc, char *argv[], gn_data *data, struct gn_statemachine *state
 			savesms_usage(stderr, -1);
 		}
 	}
+	if (argc > optind) {
+		/* There are too many arguments that don't start with '-' */
+		savesms_usage(stderr, -1);
+	}
 #if 0
 	if (interactive) {
 		gn_sms aux;
@@ -536,20 +564,10 @@ int savesms(int argc, char *argv[], gn_data *data, struct gn_statemachine *state
 
 	if (!sms.smsc.type) sms.smsc.type = GN_GSM_NUMBER_Unknown;
 
-	fprintf(stderr, _("Please enter SMS text. End your input with <cr><control-D>:"));
-
-	chars_read = fread(message_buffer, 1, sizeof(message_buffer), stdin);
-
-	if (chars_read == 0) {
-		fprintf(stderr, _("Couldn't read from stdin!\n"));
-		return -1;
-	} else if (chars_read > input_len || chars_read > sizeof(sms.user_data[0].u.text) - 1) {
-		fprintf(stderr, _("Input too long!\n"));
-		return -1;
-	}
-
-	if (chars_read > 0 && message_buffer[chars_read - 1] == '\n') message_buffer[--chars_read] = 0x00;
-	if (chars_read < 1) {
+	error = readtext(&sms.user_data[0], input_len);
+	if (error != GN_ERR_NONE)
+		return error;
+	if (sms.user_data[0].length < 1) {
 		fprintf(stderr, _("Empty message. Quitting.\n"));
 		return -1;
 	}
@@ -557,8 +575,6 @@ int savesms(int argc, char *argv[], gn_data *data, struct gn_statemachine *state
 		sms.memory_type = gn_str2memory_type(memory_type);
 
 	sms.user_data[0].type = GN_SMS_DATA_Text;
-	strncpy(sms.user_data[0].u.text, message_buffer, chars_read);
-	sms.user_data[0].u.text[chars_read] = 0;
 	sms.user_data[1].type = GN_SMS_DATA_None;
 	if (!gn_char_def_alphabet(sms.user_data[0].u.text))
 		sms.dcs.u.general.alphabet = GN_SMS_DCS_UCS2;
@@ -584,14 +600,20 @@ void getsms_usage(FILE *f, int exitval)
 			"                                    location start and ending with end;\n"
 			"                                    if end option is omitted, just one entry\n"
 			"                                    is read;\n"
+			"                                    start must be a number, end may be a number\n"
+			"                                    or 'end' string;\n"
 			"                                    if 'end' is used entries are being read\n"
 			"                                    until empty location\n"
-			"        -F file                     save sms to the file overwriting it if\n"
+			"        --force-file filename\n"
+			"        -F filename                 save sms to the file overwriting it if\n"
 			"                                    the file already exists;\n"
-			"                                    if multiple SMS are being read, message\n"
-			"                                    identifier is appended to the file name\n"
-			"        -f file                     as above but user is prompted before\n"
+			"                                    file is in mbox format\n"
+			"        --file filename\n"
+			"        -f filename                 as above but user is prompted before\n"
 			"                                    overwriting the file\n"
+			"        --append-file filename\n"
+			"        -a filename                 as above but append to the exising file\n"
+			"        --delete\n"
 			"        -d                          delete message after reading\n"
 			"\n"
 		));
@@ -613,6 +635,16 @@ int getsms(int argc, char *argv[], gn_data *data, struct gn_statemachine *state)
 	gn_phone *phone = &state->driver.phone;
 	char ans[5];
 	struct stat buf;
+	char *message_text;
+
+	struct option options[] = {
+		{ "delete",     no_argument,       NULL, 'd' },
+		{ "file",       required_argument, NULL, 'f' },
+		{ "force-file", required_argument, NULL, 'F' },
+		{ "append-file",required_argument, NULL, 'a' },
+		{ NULL,         0,                 NULL, 0 }
+	};
+
 
 	/* Handle command line args that set type, start and end locations. */
 	memory_type_string = optarg;
@@ -621,26 +653,33 @@ int getsms(int argc, char *argv[], gn_data *data, struct gn_statemachine *state)
 		return -1;
 	}
 
-	start_message = atoi(argv[optind]);
+	start_message = gnokii_atoi(argv[optind]);
+	if (errno || start_message < 0)
+		getsms_usage(stderr, -1);
 	end_message = parse_end_value_option(argc, argv, optind + 1, start_message);
+	if (errno || end_message < 0)
+		getsms_usage(stderr, -1);
 
-	memset(&filename, 0, sizeof(filename));
+	*filename = '\0';
 	/* parse all options (beginning with '-') */
-	while ((i = getopt(argc, argv, "f:F:d")) != -1) {
+	while ((i = getopt_long(argc, argv, "f:F:a:d", options, NULL)) != -1) {
 		switch (i) {
 		case 'd':
 			del = 1;
+			dprintf("del\n");
 			break;
+		/* append mode */
+		case 'a':
+			mode = 2;
+			goto parsefile;
 		/* force mode -- don't ask to overwrite */
 		case 'F':
 			mode = 0;
 		case 'f':
+parsefile:
 			if (optarg) {
-				fprintf(stderr, _("Saving into %s\n"), optarg);
-				memset(&filename, 0, sizeof(filename));
-				strncpy(filename, optarg, sizeof(filename) - 1);
-				if (strlen(optarg) > sizeof(filename) - 1)
-					fprintf(stderr, _("Filename too long - will be truncated to 63 characters.\n"));
+				snprintf(filename, sizeof(filename), "%s", optarg);
+				fprintf(stderr, _("Saving into %s\n"), filename);
 			} else
 				getsms_usage(stderr, -1);
 			break;
@@ -648,6 +687,11 @@ int getsms(int argc, char *argv[], gn_data *data, struct gn_statemachine *state)
 			getsms_usage(stderr, -1);
 		}
 	}
+	if (argc - optind > 3) {
+		/* There are too many arguments that don't start with '-' */
+		getsms_usage(stderr, -1);
+	}
+
 	folder.folder_id = 0;
 	data->sms_folder = &folder;
 	data->sms_folder_list = &folderlist;
@@ -664,9 +708,10 @@ int getsms(int argc, char *argv[], gn_data *data, struct gn_statemachine *state)
 		error = gn_sms_get(data, state);
 		switch (error) {
 		case GN_ERR_NONE:
+			message_text = NULL;
 			switch (message.type) {
-			case GN_SMS_MT_Submit:
-				fprintf(stdout, _("%d. MO Message "), message.number);
+			case GN_SMS_MT_DeliveryReport:
+				fprintf(stdout, _("%d. Delivery Report "), message.number);
 				switch (message.status) {
 				case GN_SMS_Read:
 					fprintf(stdout, _("(read)\n"));
@@ -684,14 +729,6 @@ int getsms(int argc, char *argv[], gn_data *data, struct gn_statemachine *state)
 					fprintf(stdout, _("(read)\n"));
 					break;
 				}
-				fprintf(stdout, _("Text: %s\n\n"), message.user_data[0].u.text);
-				break;
-			case GN_SMS_MT_DeliveryReport:
-				fprintf(stdout, _("%d. Delivery Report "), message.number);
-				if (message.status)
-					fprintf(stdout, _("(read)\n"));
-				else
-					fprintf(stdout, _("(not read)\n"));
 				fprintf(stdout, _("Sending date/time: %02d/%02d/%04d %02d:%02d:%02d "), \
 					message.smsc_time.day, message.smsc_time.month, message.smsc_time.year, \
 					message.smsc_time.hour, message.smsc_time.minute, message.smsc_time.second);
@@ -713,27 +750,19 @@ int getsms(int argc, char *argv[], gn_data *data, struct gn_statemachine *state)
 				}
 				fprintf(stdout, "\n");
 				fprintf(stdout, _("Receiver: %s Msg Center: %s\n"), message.remote.number, message.smsc.number);
-				fprintf(stdout, _("Text: %s\n\n"), message.user_data[0].u.text);
+				fprintf(stdout, _("Text:\n"));
+				message_text = message.user_data[0].u.text;
 				break;
 			case GN_SMS_MT_Picture:
-				fprintf(stdout, _("Picture Message\n"));
-				fprintf(stdout, _("Date/time: %02d/%02d/%04d %02d:%02d:%02d "), \
-					message.smsc_time.day, message.smsc_time.month, message.smsc_time.year, \
-					message.smsc_time.hour, message.smsc_time.minute, message.smsc_time.second);
-				if (message.smsc_time.timezone) {
-					if (message.smsc_time.timezone > 0)
-						fprintf(stdout,_("+%02d00"), message.smsc_time.timezone);
-					else
-						fprintf(stdout,_("%02d00"), message.smsc_time.timezone);
+			case GN_SMS_MT_PictureTemplate:
+				switch (message.type) {
+				case GN_SMS_MT_PictureTemplate:
+					fprintf(stdout, _("%d. Picture Message Template "), message.number);
+					break;
+				default:
+					fprintf(stdout, _("%d. Picture Message "), message.number);
+					break;
 				}
-				fprintf(stdout, "\n");
-				fprintf(stdout, _("Sender: %s Msg Center: %s\n"), message.remote.number, message.smsc.number);
-				fprintf(stdout, _("Bitmap:\n"));
-				gn_bmp_print(&message.user_data[0].u.bitmap, stdout);
-				fprintf(stdout, _("Text: %s\n\n"), message.user_data[1].u.text);
-				break;
-			default:
-				fprintf(stdout, _("%d. Inbox Message "), message.number);
 				switch (message.status) {
 				case GN_SMS_Read:
 					fprintf(stdout, _("(read)\n"));
@@ -762,8 +791,60 @@ int getsms(int argc, char *argv[], gn_data *data, struct gn_statemachine *state)
 				}
 				fprintf(stdout, "\n");
 				fprintf(stdout, _("Sender: %s Msg Center: %s\n"), message.remote.number, message.smsc.number);
+				fprintf(stdout, _("Bitmap:\n"));
+				gn_bmp_print(&message.user_data[0].u.bitmap, stdout);
+				fprintf(stdout, _("Text:\n"));
+				message_text = message.user_data[1].u.text;
+				break;
+			default:
+				switch (message.type) {
+				case GN_SMS_MT_Submit:
+				case GN_SMS_MT_SubmitSent:
+					fprintf(stdout, _("%d. MO Message "), message.number);
+					break;
+				default:
+					fprintf(stdout, _("%d. Inbox Message "), message.number);
+					break;
+				}
+				switch (message.status) {
+				case GN_SMS_Read:
+					fprintf(stdout, _("(read)\n"));
+					break;
+				case GN_SMS_Unread:
+					fprintf(stdout, _("(unread)\n"));
+					break;
+				case GN_SMS_Sent:
+					fprintf(stdout, _("(sent)\n"));
+					break;
+				case GN_SMS_Unsent:
+					fprintf(stdout, _("(unsent)\n"));
+					break;
+				default:
+					fprintf(stdout, _("(read)\n"));
+					break;
+				}
+				fprintf(stdout, _("Date/time: %02d/%02d/%04d %02d:%02d:%02d "), \
+					message.smsc_time.day, message.smsc_time.month, message.smsc_time.year, \
+					message.smsc_time.hour, message.smsc_time.minute, message.smsc_time.second);
+				if (message.smsc_time.timezone) {
+					if (message.smsc_time.timezone > 0)
+						fprintf(stdout,_("+%02d00"), message.smsc_time.timezone);
+					else
+						fprintf(stdout,_("%02d00"), message.smsc_time.timezone);
+				}
+				fprintf(stdout, "\n");
+				switch (message.type) {
+				case GN_SMS_MT_Submit:
+				case GN_SMS_MT_SubmitSent:
+					fprintf(stdout, _("Receiver: %s Msg Center: %s\n"), message.remote.number, message.smsc.number);
+					break;
+				default:
+					fprintf(stdout, _("Sender: %s Msg Center: %s\n"), message.remote.number, message.smsc.number);
+					break;
+				}
 				/* No UDH */
-				if (!message.udh.number) message.udh.udh[0].type = GN_SMS_UDH_None;
+				if (!message.udh.number)
+					message.udh.udh[0].type = GN_SMS_UDH_None;
 				switch (message.udh.udh[0].type) {
 				case GN_SMS_UDH_None:
 					fprintf(stdout, _("Text:\n"));
@@ -826,13 +907,18 @@ int getsms(int argc, char *argv[], gn_data *data, struct gn_statemachine *state)
 				}
 				if (done)
 					break;
-				fprintf(stdout, "%s\n", message.user_data[0].u.text);
+				message_text = message.user_data[0].u.text;
+				break;
+			}
+			if (message_text) {
+				fprintf(stdout, "%s\n", message_text);
 				if ((mode != -1) && *filename) {
 					char buf[1024];
-					sprintf(buf, "%s%d", filename, count);
-					mode = gn_file_text_save(buf, message.user_data[0].u.text, mode);
+					char *mbox = gn_sms2mbox(&message, "gnokii");
+					snprintf(buf, 1023, "%s", filename);
+					mode = writefile(buf, mbox, mode);
+					free(mbox);
 				}
-				break;
 			}
 			if (del) {
 				data->sms = &message;
@@ -848,6 +934,8 @@ int getsms(int argc, char *argv[], gn_data *data, struct gn_statemachine *state)
 			fprintf(stderr, _("GetSMS %s %d failed! (%s)\n"), memory_type_string, count, gn_error_print(error));
 			if (error == GN_ERR_INVALIDMEMORYTYPE)
 				fprintf(stderr, _("See the gnokii manual page for the supported memory types with the phone\nyou use.\n"));
+			if (error != GN_ERR_EMPTYLOCATION)
+				return error;
 			break;
 		}
 	}
@@ -891,8 +979,12 @@ int deletesms(int argc, char *argv[], gn_data *data, struct gn_statemachine *sta
 		return -1;
 	}
 
-	start_message = atoi(argv[optind]);
+	start_message = gnokii_atoi(argv[optind]);
+	if (errno || start_message < 0)
+		deletesms_usage(stderr, -1);
 	end_message = parse_end_value_option(argc, argv, optind + 1, start_message);
+	if (errno || end_message < 0)
+		deletesms_usage(stderr, -1);
 
 	/* Now delete the requested entries. */
 	for (count = start_message; count <= end_message; count++) {
@@ -957,9 +1049,18 @@ int getsmsc(int argc, char *argv[], gn_data *data, struct gn_statemachine *state
 		}
 	}
 
+	if (argc > optind + 2) {
+		/* There are too many arguments that don't start with '-' */
+		getsmsc_usage(stderr, -1);
+	}
+
 	if (argc > optind) {
-		start = atoi(argv[optind]);
-		stop = (argc > optind+1) ? atoi(argv[optind+1]) : start;
+		start = gnokii_atoi(argv[optind]);
+		if (errno || start < 0)
+			getsmsc_usage(stderr, -1);
+		stop = (argc > optind+1) ? gnokii_atoi(argv[optind+1]) : start;
+		if (errno || stop < 0)
+			getsmsc_usage(stderr, -1);
 
 		if (start > stop) {
 			fprintf(stderr, _("Starting SMS center number is greater than stop\n"));
@@ -1110,7 +1211,7 @@ int setsmsc(gn_data *data, struct gn_statemachine *state)
 			break;
 		default:
 			fprintf(stderr, _("Input line format isn't valid\n"));
-			return -1;
+			return GN_ERR_WRONGDATAFORMAT;
 		}
 
 		error = gn_sm_functions(GN_OP_SetSMSCenter, data, state);
@@ -1120,7 +1221,7 @@ int setsmsc(gn_data *data, struct gn_statemachine *state)
 		}
 	}
 
-	return 0;
+	return GN_ERR_NONE;
 }
 
 /* Displays usage of --createsmsfolder command */
@@ -1168,8 +1269,8 @@ int deletesmsfolder(char *number, gn_data *data, struct gn_statemachine *state)
 
 	gn_data_clear(data);
 
-	folder.folder_id = atoi(number);
-	if (folder.folder_id > 0 && folder.folder_id <= GN_SMS_FOLDER_MAX_NUMBER)
+	folder.folder_id = gnokii_atoi(number);
+	if (!errno && folder.folder_id > 0 && folder.folder_id <= GN_SMS_FOLDER_MAX_NUMBER)
 		data->sms_folder = &folder;
 	else
 		fprintf(stderr, _("Error: Number must be between 1 and %i!\n"), GN_SMS_FOLDER_MAX_NUMBER);
@@ -1221,7 +1322,7 @@ int showsmsfolderstatus(gn_data *data, struct gn_statemachine *state)
 }
 
 /* SMS handler for --smsreader mode */
-static gn_error smsslave(gn_sms *message)
+static gn_error smsslave(gn_sms *message, struct gn_statemachine *state, void *callback_data)
 {
 	FILE *output;
 	char *s = message->user_data[0].u.text;
@@ -1236,7 +1337,7 @@ static gn_error smsslave(gn_sms *message)
 	if (p[0] == '+') {
 		p++;
 	}
-	strcpy(number, p);
+	snprintf(number, sizeof(number), "%s", p);
 	fprintf(stderr, _("SMS received from number: %s\n"), number);
 
 	/* From Pavel Machek's email to the gnokii-ml (msgid: <20020414212455.GB9528@elf.ucw.cz>):
@@ -1251,7 +1352,8 @@ static gn_error smsslave(gn_sms *message)
 		snprintf(buf, sizeof(buf), "%s/mail_%d_", smsdir, msgno);
 	else if (sscanf(s, "%d/%d:%d-%d-", &i1, &i2, &msgno, &msgpart) == 4)
 		snprintf(buf, sizeof(buf), "%s/mail_%d_%03d", smsdir, msgno, msgpart);
-	else	snprintf(buf, sizeof(buf), "%s/sms_%s_%d_%d", smsdir, number, getpid(), unknown++);
+	else
+		snprintf(buf, sizeof(buf), "%s/sms_%s_%d_%d", smsdir, number, getpid(), unknown++);
 	if ((output = fopen(buf, "r")) != NULL) {
 		fprintf(stderr, _("### Exists?!\n"));
 		return GN_ERR_FAILED;
@@ -1294,6 +1396,7 @@ int smsreader(gn_data *data, struct gn_statemachine *state)
 	gn_error error;
 
 	data->on_sms = smsslave;
+	data->callback_data = NULL;
 	error = gn_sm_functions(GN_OP_OnSMS, data, state);
 	if (!error) {
 		/* We do not want to see texts forever - press Ctrl+C to stop. */
