@@ -1,6 +1,6 @@
 /*
 
-  $Id: gnokii-utils.c,v 1.6 2006/11/29 16:06:35 dforsi Exp $
+  $Id: gnokii-utils.c,v 1.10 2007/11/07 18:28:20 pkot Exp $
 
   G N O K I I
 
@@ -47,6 +47,50 @@
 #include "gnokii-app.h"
 #include "gnokii.h"
 
+/* FIXME - this should not ask for confirmation here - I'm not sure what calls it though */
+/* mode == 0 -> overwrite
+ * mode == 1 -> ask
+ * mode == 2 -> append
+ */
+int writefile(char *filename, char *text, int mode)
+{
+	FILE *file;
+	int confirm = -1;
+	char ans[5];
+	struct stat buf;
+
+	/* Ask before overwriting */
+	if ((mode == 1) && (stat(filename, &buf) == 0)) {
+		fprintf(stdout, _("File %s exists.\n"), filename);
+		while (confirm < 0) {
+			fprintf(stdout, _("Overwrite? (yes/no) "));
+			gn_line_get(stdin, ans, 4);
+			if (!strcmp(ans, _("yes")))
+				confirm = 1;
+			else if (!strcmp(ans, _("no")))
+				confirm = 0;
+		}
+		if (!confirm)
+			return -1;
+	}
+
+	/* append */
+	if (mode == 2)
+		file = fopen(filename, "a");
+	/* overwrite */
+	else
+		file = fopen(filename, "w");
+
+	if (!file) {
+		fprintf(stderr, _("Can't open file %s for writing!\n"),  filename);
+		return -1;
+	}
+	fprintf(file, "%s\n", text);
+	fclose(file);
+	/* Return value is used as new mode. Set it to append mode */
+	return 2;
+}
+
 gn_error readtext(gn_sms_user_data *udata, int input_len)
 {
 	/* The maximum length of an uncompressed concatenated short message is
@@ -57,7 +101,7 @@ gn_error readtext(gn_sms_user_data *udata, int input_len)
 #ifndef	WIN32
 	if (isatty(0))
 #endif
-		fprintf(stderr, _("Please enter SMS text. End your input with <cr><control-D>:"));
+		fprintf(stderr, _("Please enter SMS text. End your input with <cr><control-D>:\n"));
 
 	/* Get message text from stdin. */
 	chars_read = fread(message_buffer, 1, sizeof(message_buffer), stdin);
@@ -73,10 +117,7 @@ gn_error readtext(gn_sms_user_data *udata, int input_len)
 		return GN_ERR_INTERNALERROR;
 	}
 
-	/* No need to null terminate the source string because we use strncpy(). */
-	strncpy(udata->u.text, message_buffer, chars_read);
-	udata->u.text[chars_read] = 0;
-	udata->length = chars_read;
+	udata->length = snprintf(udata->u.text, sizeof(udata->u.text), "%s", message_buffer);
 
 	return GN_ERR_NONE;
 }
@@ -162,3 +203,19 @@ void console_raw(void)
 	tcsetattr(fileno(stdin), TCSANOW, &it);
 #endif
 }
+
+#ifndef HAVE_GETLINE
+int getline(char **line, size_t *len, FILE *stream)
+{
+	size_t size = 0, last = 0;
+
+	do {
+		size += BUFSIZ;
+		*line = realloc(*line, size);
+		fgets(*line + last, size, stream);
+		*len = strlen(*line);
+		last = *len - 1;
+	} while (!feof(stream) && *line[last] != '\n');
+	return 0;
+}
+#endif
