@@ -1,6 +1,6 @@
 /*
 
-  $Id: gnokii.h,v 1.114 2007/10/05 21:01:20 pkot Exp $
+  $Id: gnokii.h,v 1.120 2007/11/24 11:26:23 pkot Exp $
 
   G N O K I I
 
@@ -69,10 +69,10 @@ typedef unsigned char uint8_t;
  * LIBGNOKII_VERSION_MINOR = GNOKII_MINOR
  * LIBGNOKII_VERSION_RELEASE = GNOKII_RELEASE
  */
-#define LIBGNOKII_VERSION_STRING "3.2.1"
+#define LIBGNOKII_VERSION_STRING "3.6.0"
 #define LIBGNOKII_VERSION_MAJOR 3
-#define LIBGNOKII_VERSION_MINOR 2
-#define LIBGNOKII_VERSION_RELEASE 1
+#define LIBGNOKII_VERSION_MINOR 6
+#define LIBGNOKII_VERSION_RELEASE 0
 #define LIBGNOKII_MAKE_VERSION( a,b,c ) (((a) << 16) | ((b) << 8) | (c))
 
 #define LIBGNOKII_VERSION \
@@ -199,6 +199,10 @@ GNOKII_API const char *gn_lib_get_supported_connection( const int num );
    The state variable will be initialized if a phone was found. */
 GNOKII_API gn_error gn_lib_search_one_connected_phone( struct gn_statemachine **state );
 
+/* use this function to return connection name to be used eg. in the
+   config file */
+GNOKII_API const char *gn_lib_get_connection_name(gn_connection_type ct);
+
 /* use this function to check whether the connection type is enabled
    0 - not supported
    1 - supported */
@@ -238,9 +242,6 @@ GNOKII_API void gn_timestamp_get(gn_timestamp *dt, int *year, int *month, int *d
  * filled in before the phone initialization */
 extern GNOKII_API struct gn_cfg_header *gn_cfg_info GNOKII_DEPRECATED;
 
-/* Files */
-GNOKII_API int gn_file_text_save(char *filename, char *text, int mode) GNOKII_DEPRECATED;
-
 /* Zero the gn_data structure */
 GNOKII_API void gn_data_clear(gn_data *data);
 extern GNOKII_API gn_phone *gn_gsm_info;
@@ -320,6 +321,8 @@ GNOKII_API gn_error gn_sms_get_folder_changes(gn_data *data, struct gn_statemach
 GNOKII_API gn_error gn_sms_delete(gn_data *data, struct gn_statemachine *state);
 GNOKII_API gn_error gn_sms_delete_no_validate(gn_data *data, struct gn_statemachine *state);
 
+GNOKII_API char *gn_sms2mbox(gn_sms *sms, char *from);
+
 /* Call service */
 GNOKII_API gn_error gn_call_dial(int *call_id, gn_data *data, struct gn_statemachine *state);
 GNOKII_API gn_error gn_call_check_active(struct gn_statemachine *state);
