@@ -1,6 +1,6 @@
 /*
 
-  $Id: common.h,v 1.147 2006/06/11 20:41:39 deller Exp $
+  $Id: common.h,v 1.156 2007/11/20 18:46:37 dforsi Exp $
 
   G N O K I I
 
@@ -52,7 +52,8 @@ typedef enum {
 	GN_CT_TCP,      /* TCP network connection */
 	GN_CT_M2BUS,	/* Serial connection with M2BUS protocol */
 	GN_CT_DKU2,	/* DKU2 usb connection using nokia_dku2 kernel driver */
-	GN_CT_DKU2LIBUSB /* DKU2 usb connection using libusb */
+	GN_CT_DKU2LIBUSB, /* DKU2 usb connection using libusb */
+	GN_CT_PCSC      /* PC/SC SIM Card reader using libpsclite */
 } gn_connection_type;
 
 /* Maximum length of device name for serial port */
@@ -75,6 +76,8 @@ typedef enum {
 	GN_MT_RC, /* Received numbers */
 	GN_MT_MC, /* Missed numbers */
 	GN_MT_LD, /* Last dialed */
+	GN_MT_BD, /* Barred Dialing Numbers */
+	GN_MT_SD, /* Service Dialing Numbers */
 	GN_MT_MT, /* combined ME and SIM phonebook */
 	GN_MT_TA, /* for compatibility only: TA=computer memory */
 	GN_MT_CB, /* Currently selected memory */
@@ -102,6 +105,7 @@ typedef enum {
 	GN_MT_F18,
 	GN_MT_F19,
 	GN_MT_F20, /* 20th CUSTOM FOLDER */
+	GN_MT_LAST = GN_MT_F20,
 	GN_MT_XX = 0xff	/* Error code for unknown memory type (returned by fbus-xxxx functions). */
 } gn_memory_type;
 
@@ -187,6 +191,8 @@ typedef struct {
 /* Some phones (at the moment 7110/6510 series) supports extended phonebook
    with additional data.  Here we have structures for them */
 typedef enum {
+	GN_PHONEBOOK_NUMBER_None    = 0x00,
+	GN_PHONEBOOK_NUMBER_Common  = 0x01,
 	GN_PHONEBOOK_NUMBER_Home    = 0x02,
 	GN_PHONEBOOK_NUMBER_Mobile  = 0x03,
 	GN_PHONEBOOK_NUMBER_Fax     = 0x04,
@@ -195,19 +201,37 @@ typedef enum {
 } gn_phonebook_number_type;
 
 typedef enum {
-	GN_PHONEBOOK_ENTRY_Name       = 0x07,
-	GN_PHONEBOOK_ENTRY_Email      = 0x08,
-	GN_PHONEBOOK_ENTRY_Postal     = 0x09,
-	GN_PHONEBOOK_ENTRY_Note       = 0x0a,
-	GN_PHONEBOOK_ENTRY_Number     = 0x0b,
-	GN_PHONEBOOK_ENTRY_Ringtone   = 0x0c,
-	GN_PHONEBOOK_ENTRY_Date       = 0x13,   /* Date is used for DC,RC,etc (last calls) */
-	GN_PHONEBOOK_ENTRY_Pointer    = 0x1a,   /* Pointer to the other memory */
-	GN_PHONEBOOK_ENTRY_Logo       = 0x1b,
-	GN_PHONEBOOK_ENTRY_LogoSwitch = 0x1c,
-	GN_PHONEBOOK_ENTRY_Group      = 0x1e,
-	GN_PHONEBOOK_ENTRY_URL        = 0x2c,
-	GN_PHONEBOOK_ENTRY_RingtoneAdv= 0x37,
+	GN_PHONEBOOK_ENTRY_Name            = 0x07,
+	GN_PHONEBOOK_ENTRY_Email           = 0x08,
+	GN_PHONEBOOK_ENTRY_Postal          = 0x09,
+	GN_PHONEBOOK_ENTRY_Note            = 0x0a,
+	GN_PHONEBOOK_ENTRY_Number          = 0x0b,
+	GN_PHONEBOOK_ENTRY_Ringtone        = 0x0c,
+	GN_PHONEBOOK_ENTRY_Date            = 0x13, /* Date is used for DC,RC,etc (last calls) */
+	GN_PHONEBOOK_ENTRY_Pointer         = 0x1a, /* Pointer to the other memory */
+	GN_PHONEBOOK_ENTRY_Logo            = 0x1b,
+	GN_PHONEBOOK_ENTRY_LogoSwitch      = 0x1c,
+	GN_PHONEBOOK_ENTRY_Group           = 0x1e,
+	GN_PHONEBOOK_ENTRY_URL             = 0x2c,
+	GN_PHONEBOOK_ENTRY_Location        = 0x2f, /* ??? */
+	GN_PHONEBOOK_ENTRY_Image           = 0x33,
+	GN_PHONEBOOK_ENTRY_RingtoneAdv     = 0x37,
+	GN_PHONEBOOK_ENTRY_UserID          = 0x38,
+	GN_PHONEBOOK_ENTRY_PTTAddress      = 0x3f, /* string */
+	GN_PHONEBOOK_ENTRY_FirstName       = 0x46,
+	GN_PHONEBOOK_ENTRY_LastName        = 0x47,
+	GN_PHONEBOOK_ENTRY_PostalAddress   = 0x4a,
+	GN_PHONEBOOK_ENTRY_ExtendedAddress = 0x4b,
+	GN_PHONEBOOK_ENTRY_Street          = 0x4c,
+	GN_PHONEBOOK_ENTRY_City            = 0x4d,
+	GN_PHONEBOOK_ENTRY_StateProvince   = 0x4e,
+	GN_PHONEBOOK_ENTRY_ZipCode         = 0x4f,
+	GN_PHONEBOOK_ENTRY_Country         = 0x50,
+	GN_PHONEBOOK_ENTRY_FormalName      = 0x52,
+	GN_PHONEBOOK_ENTRY_JobTitle        = 0x54,
+	GN_PHONEBOOK_ENTRY_Company         = 0x55,
+	GN_PHONEBOOK_ENTRY_Nickname        = 0x56,
+	GN_PHONEBOOK_ENTRY_Birthday        = 0x57,
 } gn_phonebook_entry_type;
 
 typedef enum {
@@ -219,12 +243,43 @@ typedef enum {
 	GN_PHONEBOOK_GROUP_None,
 } gn_phonebook_group_type;
 
+#define GN_PHONEBOOK_PERSON_MAX_LENGTH 64
+
+typedef struct {
+	int has_person;
+	char family_name[GN_PHONEBOOK_PERSON_MAX_LENGTH + 1];		/* GN_PHONEBOOK_ENTRY_LastName */
+	char given_name[GN_PHONEBOOK_PERSON_MAX_LENGTH + 1];            /* GN_PHONEBOOK_ENTRY_FirstName */
+	char additional_names[GN_PHONEBOOK_PERSON_MAX_LENGTH + 1];
+	char honorific_prefixes[GN_PHONEBOOK_PERSON_MAX_LENGTH + 1];    /* GN_PHONEBOOK_ENTRY_FormalName */
+	char honorific_suffixes[GN_PHONEBOOK_PERSON_MAX_LENGTH + 1];
+} gn_phonebook_person;
+
+#define GN_PHONEBOOK_ADDRESS_MAX_LENGTH 64
+
+typedef struct {
+	int has_address;
+	char post_office_box[GN_PHONEBOOK_ADDRESS_MAX_LENGTH + 1];	/* GN_PHONEBOOK_ENTRY_PostalAddress */
+	char extended_address[GN_PHONEBOOK_ADDRESS_MAX_LENGTH + 1];	/* GN_PHONEBOOK_ENTRY_ExtendedAddress */
+	char street[GN_PHONEBOOK_ADDRESS_MAX_LENGTH + 1];		/* GN_PHONEBOOK_ENTRY_Street */
+	char city[GN_PHONEBOOK_ADDRESS_MAX_LENGTH + 1];			/* GN_PHONEBOOK_ENTRY_City */
+	char state_province[GN_PHONEBOOK_ADDRESS_MAX_LENGTH + 1];	/* GN_PHONEBOOK_ENTRY_StateProvince */
+	char zipcode[GN_PHONEBOOK_ADDRESS_MAX_LENGTH + 1];		/* GN_PHONEBOOK_ENTRY_ZipCode */
+	char country[GN_PHONEBOOK_ADDRESS_MAX_LENGTH + 1];		/* GN_PHONEBOOK_ENTRY_Country */
+} gn_phonebook_address;
+
 typedef struct {
 	gn_phonebook_entry_type entry_type;
 	gn_phonebook_number_type number_type;
 	union {
 		char number[GN_PHONEBOOK_NAME_MAX_LENGTH + 1];   /* Number, Name, Address, eMail... */
+		/* GN_PHONEBOOK_ENTRY_Date
+		 * GN_PHONEBOOK_ENTRY_Birthday
+		 */
 		gn_timestamp date;                               /* or the last calls list */
+		/* GN_PHONEBOOK_ENTRY_Image
+		 * GN_PHONEBOOK_ENTRY_RingtoneAdv
+		 */
+		char fileid[6];                                  /* Bitmap or ringtone fileid */
 	} data;
 	int id;
 } gn_phonebook_subentry;
@@ -232,15 +287,17 @@ typedef struct {
 /* Define datatype for phonebook entry, used for getting/writing phonebook
    entries. */
 typedef struct {
-	int empty;                                       /* Is this entry empty? */
+	int empty;                                        /* Is this entry empty? */
 	char name[GN_PHONEBOOK_NAME_MAX_LENGTH + 1];      /* Plus 1 for
 							     nullterminator. */
 	char number[GN_PHONEBOOK_NUMBER_MAX_LENGTH + 1];  /* Number */
 	gn_memory_type memory_type;                       /* Type of memory */
-	int caller_group;          /* Caller group - gn_phonebook_group_type */
+	gn_phonebook_group_type caller_group;             /* Caller group - gn_phonebook_group_type */
 	int location;                                     /* Location */
 	gn_timestamp date;                                /* The record date and time
 							     of the number. */
+	gn_phonebook_person person;                       /* Personal information */
+	gn_phonebook_address address;                     /* Address information */
 	gn_phonebook_subentry subentries[GN_PHONEBOOK_SUBENTRIES_MAX_NUMBER];
 	/* For phones with
 	 * additional phonebook
@@ -324,6 +381,7 @@ typedef struct {
 	char phone_number[GN_CALNOTE_NUMBER_MAX_LENGTH]; /* For Call only: the phone number */
 	char mlocation[GN_CALNOTE_MAX_LENGTH];           /* For Meeting only: the location field */
 	gn_calnote_recurrence recurrence;                /* Recurrence of the note */
+	int occurrences;                                 /* Number of recurrence events; 0 for infinity */
 } gn_calnote;
 
 /* List of Calendar Notes in phone */
@@ -704,7 +762,29 @@ typedef enum {
 	GN_LOG_T_STDERR = 1
 } gn_log_target;
 
+
+typedef enum {
+	GN_FT_None = 0,
+	GN_FT_NOL,
+	GN_FT_NGG,
+	GN_FT_NSL,
+	GN_FT_NLM,
+	GN_FT_BMP,
+	GN_FT_OTA,
+	GN_FT_XPMF,
+	GN_FT_RTTTL,
+	GN_FT_OTT,
+	GN_FT_MIDI,
+	GN_FT_NOKRAW_TONE,
+	GN_FT_GIF,
+	GN_FT_JPG,
+	GN_FT_MID,
+	GN_FT_NRT,
+	GN_FT_PNG,
+} gn_filetypes;
+
 typedef struct {
+	gn_filetypes filetype;   /* file type */
 	unsigned char *id;	/* file id */
 	char name[512];		/* file name */
 	int year;		/* datetime of creation/modification */
@@ -716,6 +796,7 @@ typedef struct {
 	int file_length;	/* size of the file */
 	int togo;		/* amount of bytes to be sent yet */
 	int just_sent;		/* ??? */
+	int folderId;           /* folder id of the file */
 	unsigned char *file;	/* file contents */
 } gn_file;
 
