@@ -1,6 +1,6 @@
 /*
 
-  $Id: data.h,v 1.76 2006/01/15 23:50:04 pkot Exp $
+  $Id: data.h,v 1.84 2007/11/20 23:45:38 pkot Exp $
 
   G N O K I I
 
@@ -39,8 +39,8 @@
 
 /* For models table */
 typedef struct {
-	char *model;
-	char *number;
+	const char *model;	  /* e.g. 6310 */
+	const char *product_name; /* e.g. NPE-4 */
 	int flags;
 } gn_phone_model;
 
@@ -82,13 +82,13 @@ typedef struct {
 	gn_calnote_alarm *alarm;
 	gn_raw_data *raw_data;
 	gn_call_divert *call_divert;
-	gn_error (*on_sms)(gn_sms *message);
+	gn_error (*on_sms)(gn_sms *message, struct gn_statemachine *state, void *callback_data);
 	int *display_status;
-	void (*on_cell_broadcast)(gn_cb_message *message);
+	void (*on_cell_broadcast)(gn_cb_message *message, struct gn_statemachine *state, void *callback_data);
 	gn_netmonitor *netmonitor;
 	gn_call_info *call_info;
 	void (*call_notification)(gn_call_status call_status, gn_call_info *call_info,
-				  struct gn_statemachine *state);
+				  struct gn_statemachine *state, void *callback_data);
 	gn_rlp_f96_frame *rlp_frame;
 	int rlp_out_dtx;
 	void (*rlp_rx_callback)(gn_rlp_f96_frame *frame);
@@ -104,7 +104,7 @@ typedef struct {
 	gn_call_active *call_active;
 	gn_file_list *file_list;
 	gn_file *file;
-	
+	void *callback_data; /* this is a pointer to some data that will be needed by any callback function */
 } gn_data;
 
 /* 
@@ -120,6 +120,7 @@ typedef struct {
 	gn_error (*send_message)(unsigned int messagesize, unsigned char messagetype, unsigned char *message,
 				 struct gn_statemachine *state);
 	void *link_instance;
+	void (*reset)(struct gn_statemachine *state);
 } gn_link;
 
 typedef struct {
@@ -136,7 +137,15 @@ typedef struct {
 	char connect_script[256];			/* Script to run when device connection established */
 	char disconnect_script[256];			/* Script to run when device connection closed */
 	uint8_t rfcomm_cn;				/* RFCOMM channel number to connect */
-	unsigned int sm_retry;				/* Indicates whether statemachine should to retries. Defaults to off. Use with caution -- may break newer DCT4 phones */
+	unsigned int sm_retry;				/* Indicates whether statemachine should do retries. Defaults to off. */
+							/* Use with caution -- may break newer DCT4 phones */
+
+	unsigned int use_locking;			/* Should we use locking system or not */
+	/* do not change the following values from userspace */
+	char m_model[GN_MODEL_MAX_LENGTH];
+	char m_manufacturer[GN_MANUFACTURER_MAX_LENGTH];
+	char m_revision[GN_REVISION_MAX_LENGTH];
+	char m_imei[GN_IMEI_MAX_LENGTH];
 } gn_config;
 
 typedef struct {
@@ -187,7 +196,7 @@ typedef enum {
 	GN_OP_GetToDo,
 	GN_OP_GetCalendarNote,
 	GN_OP_CallDivert,
-	GN_OP_OnSMS,
+	GN_OP_OnSMS, /* set data->on_sms and data->callback_data */
 	GN_OP_PollSMS,
 	GN_OP_SetAlarm,
 	GN_OP_SetDateTime,
@@ -201,12 +210,12 @@ typedef enum {
 	GN_OP_GetDisplayStatus,
 	GN_OP_PollDisplay,
 	GN_OP_SaveSMS,
-	GN_OP_SetCellBroadcast,
+	GN_OP_SetCellBroadcast, /* set data->on_cell_broadcast and data->callback_data */
 	GN_OP_NetMonitor,
 	GN_OP_MakeCall,
 	GN_OP_AnswerCall,
 	GN_OP_CancelCall,
-	GN_OP_SetCallNotification,
+	GN_OP_SetCallNotification, /* set data->call_notification and data->callback_data */
 	GN_OP_SendRLPFrame,
 	GN_OP_SetRLPRXCallback,
 	GN_OP_EnterSecurityCode,
