@@ -1,6 +1,6 @@
 /*
 
-  $Id: sms.h,v 1.67 2006/11/08 19:45:37 dforsi Exp $
+  $Id: sms.h,v 1.70 2007/11/21 20:32:00 pkot Exp $
 
   G N O K I I
 
@@ -80,6 +80,7 @@ typedef struct {
 	unsigned int folders_count;
 	/* Message store used for new received messages (this is used
 	 * internally by AT_SetSMSMemoryType() in common/phones/atgen.c). */
+	/* Note: this field is unused since revision 1.128 of common/phones/atgen.c */
 	gn_memory_type new_message_store;
 } gn_sms_status;
 
@@ -293,6 +294,16 @@ typedef struct {
 	gn_sms_command_type type;
 } gn_sms_message_command;
 
+/* Datatype for SMS Delivery Report Statuses */
+typedef enum {
+	GN_SMS_DR_Status_None = 0,
+	GN_SMS_DR_Status_Invalid,
+	GN_SMS_DR_Status_Delivered,
+	GN_SMS_DR_Status_Pending,
+	GN_SMS_DR_Status_Failed_Temporary,
+	GN_SMS_DR_Status_Failed_Permanent /* FIXME: add more reasons for failure? */
+} gn_sms_delivery_report_status;
+
 /* Datatype for SMS status */
 typedef enum {
 	GN_SMS_Read   = 0x01,
@@ -360,6 +371,11 @@ typedef struct {
 		gn_bmp animation[4];
 		gn_sms_concat concat;
 	} u;
+	/* That should be in the union, but for delivery reports we already
+	 * set text there. Currently we don't want to break API, so I put it here
+	 * Pawel Kot, 2007-11-21
+	 */
+	gn_sms_delivery_report_status dr_status;
 } gn_sms_user_data;
 
 /* Define datatype for SMS messages exported to the user applications. */
@@ -369,7 +385,7 @@ typedef struct {
 	int delivery_report;                /* Do we request the delivery report? Only for setting. */
 	gn_sms_message_status status;       /* Status of the message read/unread/sent/unsent. */
 	unsigned int validity;              /* Message validity in minutes. Only for setting. */
-	gn_memory_type memory_type  ;       /* Memory type where the message is/should be stored. */
+	gn_memory_type memory_type;         /* Memory type where the message is/should be stored. */
 	unsigned int number;                /* Location of the message in the memory/folder. */
 
 	/* Number related fields */
