@@ -1,6 +1,6 @@
 /*
 
-  $Id: gnapplet.h,v 1.20 2006/01/05 00:25:31 bozo Exp $
+  $Id: gnapplet.h,v 1.22 2007/11/03 22:07:18 bozo Exp $
 
   G N O K I I
 
@@ -36,7 +36,7 @@
 
 
 #define	GNAPPLET_MAJOR_VERSION	0
-#define	GNAPPLET_MINOR_VERSION	18
+#define	GNAPPLET_MINOR_VERSION	19
 
 #define	GNAPPLET_MSG_INFO	1
 #define	GNAPPLET_MSG_PHONEBOOK	2
@@ -130,7 +130,7 @@
 typedef struct {
 	int proto_major;
 	int proto_minor;
-	char manufacturer[20];
+	char manufacturer[GN_MANUFACTURER_MAX_LENGTH];
 	char model[GN_MODEL_MAX_LENGTH];
 	char imei[GN_IMEI_MAX_LENGTH];
 	char sw_version[GN_REVISION_MAX_LENGTH];
