@@ -1,6 +1,6 @@
 /*
 
-  $Id: xgnokii.h,v 1.27 2006/05/23 20:07:48 deller Exp $
+  $Id: xgnokii.h,v 1.28 2006/12/17 17:17:43 pkot Exp $
 
   X G N O K I I
 
@@ -67,6 +67,7 @@ typedef struct {
 	gchar *help_locale;	/* Locale for HTML help */
 	gn_sms_message_center smsSetting[MAX_SMS_CENTER];
 	UserInf user;
+	gint allowBreakage;
 	gchar *callerGroups[6];
 	gint smsSets:4;
 	bool alarmSupported:1;
@@ -84,6 +85,6 @@ extern gint max_phonebook_sim_number_length;
 extern void GUI_InitCallerGroupsInf(void);
 extern void GUI_InitSMSSettings(void);
 extern void GUI_ShowAbout(void);
-extern void MainExit();
+extern void MainExit(gchar *);
 
 #endif				/* XGNOKII_H */
