@@ -1,6 +1,6 @@
 /*
 
-  $Id: xgnokii_lowlevel.c,v 1.99 2007/08/23 14:12:47 pkot Exp $
+  $Id: xgnokii_lowlevel.c,v 1.102 2007/11/19 17:26:18 dforsi Exp $
   
   X G N O K I I
 
@@ -464,11 +464,12 @@ static gint A_GetMemoryLocationAll(gpointer data)
 	gn_data_clear(&gdat);
 
 	error = mla->status = GN_ERR_NONE;
-	entry.memory_type = mla->type;
 	gdat.phonebook_entry = &entry;
 
 	pthread_mutex_lock(&memoryMutex);
 	for (i = mla->min; i <= mla->max; i++) {
+		memset(&entry, 0, sizeof(entry));
+		entry.memory_type = mla->type;
 		entry.location = i;
 		error = gn_sm_functions(GN_OP_ReadPhonebook, &gdat, statemachine);
 		if (error != GN_ERR_NONE && error != GN_ERR_INVALIDLOCATION &&
@@ -504,7 +505,7 @@ static gint A_GetMemoryLocationAll(gpointer data)
 				return GN_ERR_NONE;
 			}
 		}
-		if (error == GN_ERR_NONE) read++;
+		if ((error == GN_ERR_NONE) && (entry.empty == false)) read++;
 		dprintf("Name: %s\n", entry.name);
 		error = mla->InsertEntry(&entry); 
 		/* FIXME: It only works this way at the moment */
@@ -566,6 +567,31 @@ static gint A_WriteMemoryLocationAll(gpointer data)
 	return error;
 }
 
+
+static gint A_DeleteMemoryLocation(gpointer data)
+{
+	gn_error error;
+	D_MemoryLocation *ml = (D_MemoryLocation *) data;
+	gn_data gdat;
+
+	if (!data) return GN_ERR_INTERNALERROR;
+
+	gn_data_clear(&gdat);
+
+	error = ml->status = GN_ERR_UNKNOWN;
+
+	if (ml->entry) {
+		gn_phonebook_entry_sanitize(ml->entry);
+		gdat.phonebook_entry = (ml->entry);
+		pthread_mutex_lock(&memoryMutex);
+		error = ml->status = gn_sm_functions(GN_OP_DeletePhonebook, &gdat, statemachine);
+		pthread_cond_signal(&memoryCond);
+		pthread_mutex_unlock(&memoryMutex);
+	}
+
+	return (error);
+}
+
 static gint A_GetCalendarNote(gpointer data)
 {
 	gn_error error;
@@ -1165,6 +1191,7 @@ gint(*DoAction[])(gpointer) = {
 	    A_GetMemoryLocationAll,
 	    A_WriteMemoryLocation,
 	    A_WriteMemoryLocationAll,
+	    A_DeleteMemoryLocation,
 	    A_GetCalendarNote,
 	    A_GetCalendarNoteAll,
 	    A_WriteCalendarNote,
