@@ -1,6 +1,6 @@
 /*
 
-  $Id: xgnokii_xring.c,v 1.9 2006/02/17 18:24:43 dforsi Exp $
+  $Id: xgnokii_xring.c,v 1.13 2007/11/07 18:28:20 pkot Exp $
 
   X G N O K I I
 
@@ -52,7 +52,9 @@
  * HISTORY 
  */
 
-#include <unistd.h>
+#ifndef WIN32
+#  include <unistd.h>
+#endif
 #include <stdio.h>
 #include <assert.h>
 #include <errno.h>
@@ -106,7 +108,8 @@ static char *whiteend[] = {
 
 
 static void writecolors(char *buf0, char *col0,
-			char *buf1, char *col1, char *buf2, char *col2)
+			char *buf1, char *col1,
+			char *buf2, char *col2)
 {
 	sprintf(buf0, ".	c #%s", col0);
 	sprintf(buf1, "+	c #%s", col1);
@@ -818,19 +821,19 @@ void GUI_CreateXringWindow(void)
 		{NULL, NULL,		clear_ringtone,	  0, NULL}
 	};
 
-	menu_items[0].path = g_strdup(_("/_File"));
-	menu_items[1].path = g_strdup(_("/File/_Open"));
-	menu_items[2].path = g_strdup(_("/File/_Save"));
-	menu_items[3].path = g_strdup(_("/File/Save _as ..."));
-	menu_items[4].path = g_strdup(_("/File/S1"));
-	menu_items[5].path = g_strdup(_("/File/_Get ringtone"));
-	menu_items[6].path = g_strdup(_("/File/Se_t ringtone"));
-	menu_items[7].path = g_strdup(_("/File/Delete ringtone"));
-	menu_items[8].path = g_strdup(_("/File/_Play"));
-	menu_items[9].path = g_strdup(_("/File/S2"));
-	menu_items[10].path = g_strdup(_("/File/_Close"));
-	menu_items[11].path = g_strdup(_("/_Edit"));
-	menu_items[12].path = g_strdup(_("/Edit/_Clear"));
+	menu_items[0].path = _("/_File");
+	menu_items[1].path = _("/File/_Open");
+	menu_items[2].path = _("/File/_Save");
+	menu_items[3].path = _("/File/Save _as ...");
+	menu_items[4].path = _("/File/S1");
+	menu_items[5].path = _("/File/_Get ringtone");
+	menu_items[6].path = _("/File/Se_t ringtone");
+	menu_items[7].path = _("/File/Delete ringtone");
+	menu_items[8].path = _("/File/_Play");
+	menu_items[9].path = _("/File/S2");
+	menu_items[10].path = _("/File/_Close");
+	menu_items[11].path = _("/_Edit");
+	menu_items[12].path = _("/Edit/_Clear");
 
 	/* create toplevel window */
 
@@ -1005,5 +1008,6 @@ void GUI_ShowXring(void)
 	gi.pressed = 0;
 
 	gtk_widget_show_all(gi.w);
+	gtk_window_present(GTK_WINDOW(gi.w));
 	gtk_widget_grab_focus(gi.f);
 }
